@@ -22,7 +22,6 @@ from osprofiler import opts as profiler
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LW
 from heat.common import wsgi
 
 
@@ -102,6 +101,11 @@ engine_opts = [
     cfg.StrOpt('deferred_auth_method',
                choices=['password', 'trusts'],
                default='trusts',
+               deprecated_for_removal=True,
+               deprecated_reason='Stored password based deferred auth is '
+                                 'broken when used with keystone v3 and '
+                                 'is not supported.',
+               deprecated_since='9.0.0',
                help=_('Select deferred auth method, '
                       'stored password or trusts.')),
     cfg.StrOpt('reauthentication_auth_method',
@@ -110,10 +114,6 @@ engine_opts = [
                help=_('Allow reauthentication on token expiry, such that'
                       ' long-running tasks may complete. Note this defeats'
                       ' the expiry of any provided user tokens.')),
-    cfg.IntOpt('stale_token_duration',
-               default=30,
-               help=_('Gap, in seconds, to determine whether the given token '
-                      'is about to expire.'),),
     cfg.ListOpt('trusts_delegated_roles',
                 default=[],
                 help=_('Subset of trustor roles to be delegated to heat.'
@@ -375,8 +375,8 @@ def startup_sanity_check():
             not cfg.CONF.stack_user_domain_name):
         # FIXME(shardy): Legacy fallback for folks using old heat.conf
         # files which lack domain configuration
-        LOG.warning(_LW('stack_user_domain_id or stack_user_domain_name not '
-                        'set in heat.conf falling back to using default'))
+        LOG.warning('stack_user_domain_id or stack_user_domain_name not '
+                    'set in heat.conf falling back to using default')
     else:
         domain_admin_user = cfg.CONF.stack_domain_admin
         domain_admin_password = cfg.CONF.stack_domain_admin_password
@@ -389,7 +389,7 @@ def startup_sanity_check():
     auth_key_len = len(cfg.CONF.auth_encryption_key)
     if auth_key_len in (16, 24):
         LOG.warning(
-            _LW('Please update auth_encryption_key to be 32 characters.'))
+            'Please update auth_encryption_key to be 32 characters.')
     elif auth_key_len != 32:
         raise exception.Error(_('heat.conf misconfigured, auth_encryption_key '
                                 'must be 32 characters'))
