@@ -31,6 +31,8 @@ LOG = logging.getLogger(__name__)
 DEFAULT_RULES = policy.Rules.from_dict({'default': '!'})
 DEFAULT_RESOURCE_RULES = policy.Rules.from_dict({'default': '@'})
 
+ENFORCER = None
+
 
 class Enforcer(object):
     """Responsible for loading and enforcing rules."""
@@ -62,7 +64,7 @@ class Enforcer(object):
            :returns: A non-False value if access is allowed.
         """
         do_raise = False if not exc else True
-        credentials = context.to_dict()
+        credentials = context.to_policy_values()
         return self.enforcer.enforce(rule, target, credentials,
                                      do_raise, exc=exc, *args, **kwargs)
 
@@ -88,14 +90,20 @@ class Enforcer(object):
         return self._check(context, 'context_is_admin', target={}, exc=None)
 
 
+def get_enforcer():
+    global ENFORCER
+    if ENFORCER is None:
+        ENFORCER = Enforcer()
+    return ENFORCER
+
+
 class ResourceEnforcer(Enforcer):
     def __init__(self, default_rule=DEFAULT_RESOURCE_RULES['default'],
                  **kwargs):
         super(ResourceEnforcer, self).__init__(
             default_rule=default_rule, **kwargs)
 
-    def enforce(self, context, res_type, scope=None, target=None):
-        # NOTE(pas-ha): try/except just to log the exception
+    def _enforce(self, context, res_type, scope=None, target=None):
         try:
             result = super(ResourceEnforcer, self).enforce(
                 context, res_type,
@@ -107,8 +115,20 @@ class ResourceEnforcer(Enforcer):
         if not result:
             if self.exc:
                 raise self.exc(action=res_type)
-            else:
-                return result
+        return result
+
+    def enforce(self, context, res_type, scope=None, target=None):
+        # NOTE(pas-ha): try/except just to log the exception
+        result = self._enforce(context, res_type, scope, target)
+
+        if result:
+            # check for wildcard resource types
+            subparts = res_type.split("::")[:-1]
+            subparts.append('*')
+            res_type_wc = "::".join(subparts)
+            return self._enforce(context, res_type_wc, scope, target)
+
+        return result
 
     def enforce_stack(self, stack, scope=None, target=None):
         for res in stack.resources.values():
