@@ -29,14 +29,13 @@ import six
 import sqlalchemy
 from sqlalchemy import and_
 from sqlalchemy import func
+from sqlalchemy import or_
 from sqlalchemy import orm
 from sqlalchemy.orm import aliased as orm_aliased
 
 from heat.common import crypt
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LE
-from heat.common.i18n import _LI
 from heat.db.sqlalchemy import filters as db_filters
 from heat.db.sqlalchemy import migration
 from heat.db.sqlalchemy import models
@@ -175,7 +174,7 @@ def raw_template_files_get(context, files_id):
     return result
 
 
-def resource_get(context, resource_id, refresh=False):
+def resource_get(context, resource_id, refresh=False, refresh_data=False):
     result = context.session.query(models.Resource).get(resource_id)
 
     if not result:
@@ -183,8 +182,9 @@ def resource_get(context, resource_id, refresh=False):
                                  resource_id)
     if refresh:
         context.session.refresh(result)
-        # ensure data is loaded (lazy or otherwise)
-        result.data
+        if refresh_data:
+            # ensure data is loaded (lazy or otherwise)
+            result.data
 
     return result
 
@@ -230,19 +230,29 @@ def resource_get_all(context):
 
 def resource_purge_deleted(context, stack_id):
     filters = {'stack_id': stack_id, 'action': 'DELETE', 'status': 'COMPLETE'}
-    query = context.session.query(models.Resource.id)
+    query = context.session.query(models.Resource)
     result = query.filter_by(**filters)
-    result.delete()
+    attr_ids = [r.attr_data_id for r in result if r.attr_data_id is not None]
+    with context.session.begin(subtransactions=True):
+        result.delete()
+        if attr_ids:
+            context.session.query(models.ResourcePropertiesData).filter(
+                models.ResourcePropertiesData.id.in_(attr_ids)).delete(
+                    synchronize_session=False)
+
+
+def _add_atomic_key_to_values(values, atomic_key):
+    if atomic_key is None:
+        values['atomic_key'] = 1
+    else:
+        values['atomic_key'] = atomic_key + 1
 
 
 def resource_update(context, resource_id, values, atomic_key,
                     expected_engine_id=None):
     session = context.session
     with session.begin(subtransactions=True):
-        if atomic_key is None:
-            values['atomic_key'] = 1
-        else:
-            values['atomic_key'] = atomic_key + 1
+        _add_atomic_key_to_values(values, atomic_key)
         rows_updated = session.query(models.Resource).filter_by(
             id=resource_id, engine_id=expected_engine_id,
             atomic_key=atomic_key).update(values)
@@ -261,6 +271,48 @@ def resource_delete(context, resource_id):
         resource = session.query(models.Resource).get(resource_id)
         if resource:
             session.delete(resource)
+            if resource.attr_data_id is not None:
+                attr_prop_data = session.query(
+                    models.ResourcePropertiesData).get(resource.attr_data_id)
+                session.delete(attr_prop_data)
+
+
+def resource_attr_id_set(context, resource_id, atomic_key, attr_id):
+    session = context.session
+    with session.begin(subtransactions=True):
+        values = {'attr_data_id': attr_id}
+        _add_atomic_key_to_values(values, atomic_key)
+        rows_updated = session.query(models.Resource).filter(and_(
+            models.Resource.id == resource_id,
+            models.Resource.atomic_key == atomic_key,
+            models.Resource.engine_id.is_(None),
+            or_(models.Resource.attr_data_id == attr_id,
+                models.Resource.attr_data_id.is_(None)))).update(
+                    values)
+        if rows_updated > 0:
+            return True
+        else:
+            # Someone else set the attr_id first and/or we have a stale
+            # view of the resource based on atomic_key, so delete the
+            # resource_properties_data (attr) db row.
+            LOG.debug('Not updating res_id %(rid)s with attr_id %(aid)s',
+                      {'rid': resource_id, 'aid': attr_id})
+            session.query(
+                models.ResourcePropertiesData).filter(
+                    models.ResourcePropertiesData.attr_id == attr_id).delete()
+            return False
+
+
+def resource_attr_data_delete(context, resource_id, attr_id):
+    session = context.session
+    with session.begin(subtransactions=True):
+        resource = session.query(models.Resource).get(resource_id)
+        attr_prop_data = session.query(
+            models.ResourcePropertiesData).get(attr_id)
+        if resource:
+            resource.update({'attr_data_id': None})
+        if attr_prop_data:
+            session.delete(attr_prop_data)
 
 
 def resource_data_get_all(context, resource_id, data=None):
@@ -433,13 +485,22 @@ def engine_get_all_locked_by_stack(context, stack_id):
     return set(i[0] for i in query.all())
 
 
-def resource_prop_data_create(context, values):
-    obj_ref = models.ResourcePropertiesData()
+def resource_prop_data_create_or_update(context, values, rpd_id=None):
+    obj_ref = None
+    if rpd_id is not None:
+        obj_ref = context.session.query(
+            models.ResourcePropertiesData).filter_by(id=rpd_id).first()
+    if obj_ref is None:
+        obj_ref = models.ResourcePropertiesData()
     obj_ref.update(values)
     obj_ref.save(context.session)
     return obj_ref
 
 
+def resource_prop_data_create(context, values):
+    return resource_prop_data_create_or_update(context, values)
+
+
 def resource_prop_data_get(context, resource_prop_data_id):
     result = context.session.query(models.ResourcePropertiesData).get(
         resource_prop_data_id)
@@ -658,33 +719,36 @@ def stack_create(context, values):
 @oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
                            retry_interval=0.5, inc_retry_interval=True)
 def stack_update(context, stack_id, values, exp_trvsl=None):
-    stack = stack_get(context, stack_id)
-
-    if stack is None:
-        raise exception.NotFound(_('Attempt to update a stack with id: '
-                                 '%(id)s %(msg)s') % {
-                                     'id': stack_id,
-                                     'msg': 'that does not exist'})
-
-    if (exp_trvsl is not None
-            and stack.current_traversal != exp_trvsl):
-        # stack updated by another update
-        return False
-
     session = context.session
-
     with session.begin(subtransactions=True):
-        rows_updated = (session.query(models.Stack)
-                        .filter(models.Stack.id == stack.id)
-                        .filter(models.Stack.current_traversal
-                                == stack.current_traversal)
-                        .update(values, synchronize_session=False))
+        query = (session.query(models.Stack)
+                 .filter(and_(models.Stack.id == stack_id),
+                         (models.Stack.deleted_at.is_(None))))
+        if not context.is_admin:
+            query = query.filter(sqlalchemy.or_(
+                models.Stack.tenant == context.tenant_id,
+                models.Stack.stack_user_project_id == context.tenant_id))
+        if exp_trvsl is not None:
+            query = query.filter(models.Stack.current_traversal == exp_trvsl)
+        rows_updated = query.update(values, synchronize_session=False)
+        if not rows_updated:
+            LOG.debug('Did not set stack state with values '
+                      '%(vals)s, stack id: %(id)s with '
+                      'expected traversal: %(trav)s',
+                      {'id': stack_id, 'vals': str(values),
+                       'trav': str(exp_trvsl)})
+            if not stack_get(context, stack_id, eager_load=False):
+                raise exception.NotFound(
+                    _('Attempt to update a stack with id: '
+                      '%(id)s %(msg)s') % {
+                          'id': stack_id,
+                          'msg': 'that does not exist'})
     session.expire_all()
     return (rows_updated is not None and rows_updated > 0)
 
 
 def stack_delete(context, stack_id):
-    s = stack_get(context, stack_id)
+    s = stack_get(context, stack_id, eager_load=False)
     if not s:
         raise exception.NotFound(_('Attempt to delete a stack with id: '
                                  '%(id)s %(msg)s') % {
@@ -692,8 +756,17 @@ def stack_delete(context, stack_id):
                                      'msg': 'that does not exist'})
     session = context.session
     with session.begin():
+        attr_ids = []
+        # normally the resources are deleted already by this point
         for r in s.resources:
+            if r.attr_data_id is not None:
+                attr_ids.append(r.attr_data_id)
             session.delete(r)
+        if attr_ids:
+            session.query(
+                models.ResourcePropertiesData.id).filter(
+                    models.ResourcePropertiesData.id.in_(attr_ids)).delete(
+                        synchronize_session=False)
         delete_softly(context, s)
 
 
@@ -751,11 +824,11 @@ def stack_lock_release(context, stack_id, engine_id):
 
 
 def stack_get_root_id(context, stack_id):
-    s = stack_get(context, stack_id)
+    s = stack_get(context, stack_id, eager_load=False)
     if not s:
         return None
     while s.owner_id:
-        s = stack_get(context, s.owner_id)
+        s = stack_get(context, s.owner_id, eager_load=False)
     return s.id
 
 
@@ -825,20 +898,6 @@ def user_creds_delete(context, user_creds_id):
         context.session.delete(creds)
 
 
-def event_get(context, event_id):
-    result = context.session.query(models.Event).get(event_id)
-    return result
-
-
-def event_get_all(context):
-    stacks = soft_delete_aware_query(context, models.Stack)
-    stack_ids = [stack.id for stack in stacks]
-    results = context.session.query(
-        models.Event
-    ).filter(models.Event.stack_id.in_(stack_ids)).all()
-    return results
-
-
 def event_get_all_by_tenant(context, limit=None, marker=None,
                             sort_keys=None, sort_dir=None, filters=None):
     query = context.session.query(models.Event)
@@ -859,6 +918,9 @@ def _query_all_by_stack(context, stack_id):
 def event_get_all_by_stack(context, stack_id, limit=None, marker=None,
                            sort_keys=None, sort_dir=None, filters=None):
     query = _query_all_by_stack(context, stack_id)
+    if filters and 'uuid' in filters:
+        # retrieving a single event, so eager load its rsrc_prop_data detail
+        query = query.options(orm.joinedload("rsrc_prop_data"))
     return _events_filter_and_page_query(context, query, limit, marker,
                                          sort_keys, sort_dir, filters).all()
 
@@ -930,8 +992,7 @@ def _delete_event_rows(context, stack_id, limit):
     # delete the events
     retval = session.query(models.Event.id).filter(
         models.Event.id <= max_id).filter(
-            models.Event.stack_id == stack_id).delete(
-                synchronize_session=False)
+            models.Event.stack_id == stack_id).delete()
 
     # delete unreferenced resource_properties_data
     rsrc_prop_ids = set(rsrc_prop_ids)
@@ -1304,7 +1365,7 @@ def _purge_stacks(stack_infos, engine, meta):
     syncpoint = sqlalchemy.Table('sync_point', meta, autoload=True)
 
     stack_info_str = ','.join([str(i) for i in stack_infos])
-    LOG.info("Purging stacks %s" % stack_info_str)
+    LOG.info("Purging stacks %s", stack_info_str)
 
     # TODO(cwolfe): find a way to make this re-entrant with
     # reasonably sized transactions (good luck), or add
@@ -1335,6 +1396,11 @@ def _purge_stacks(stack_infos, engine, meta):
             resource.c.stack_id.in_(stack_ids))
     rsrc_prop_data_ids = set(
         [i[0] for i in list(engine.execute(rsrc_prop_data_where))])
+    rsrc_prop_data_where = sqlalchemy.select(
+        [resource.c.attr_data_id]).where(
+            resource.c.stack_id.in_(stack_ids))
+    rsrc_prop_data_ids.update(
+        [i[0] for i in list(engine.execute(rsrc_prop_data_where))])
     rsrc_prop_data_where = sqlalchemy.select(
         [event.c.rsrc_prop_data_id]).where(
             event.c.stack_id.in_(stack_ids))
@@ -1485,8 +1551,8 @@ def _db_encrypt_or_decrypt_template_params(
             for raw_template in next_batch:
                 try:
                     if verbose:
-                        LOG.info(_LI("Processing raw_template %(id)d..."),
-                                 {'id': raw_template.id})
+                        LOG.info("Processing raw_template %s...",
+                                 raw_template.id)
                     env = raw_template.environment
                     needs_update = False
 
@@ -1534,16 +1600,16 @@ def _db_encrypt_or_decrypt_template_params(
                         raw_template_update(ctxt, raw_template.id,
                                             {'environment': newenv})
                 except Exception as exc:
-                    LOG.exception(_LE('Failed to %(crypt_action)s parameters '
-                                      'of raw template %(id)d'),
+                    LOG.exception('Failed to %(crypt_action)s parameters '
+                                  'of raw template %(id)d',
                                   {'id': raw_template.id,
                                    'crypt_action': _crypt_action(encrypt)})
                     excs.append(exc)
                     continue
                 finally:
                     if verbose:
-                        LOG.info(_LI("Finished %(crypt_action)s processing of "
-                                     "raw_template %(id)d."),
+                        LOG.info("Finished %(crypt_action)s processing of "
+                                 "raw_template %(id)d.",
                                  {'id': raw_template.id,
                                   'crypt_action': _crypt_action(encrypt)})
         next_batch = list(itertools.islice(template_batches, batch_size))
@@ -1570,8 +1636,8 @@ def _db_encrypt_or_decrypt_resource_prop_data_legacy(
                     continue
                 try:
                     if verbose:
-                        LOG.info(_LI("Processing resource %(id)d..."),
-                                 {'id': resource.id})
+                        LOG.info("Processing resource %s...",
+                                 resource.id)
                     if encrypt:
                         result = crypt.encrypted_dict(resource.properties_data,
                                                       encryption_key)
@@ -1583,16 +1649,16 @@ def _db_encrypt_or_decrypt_resource_prop_data_legacy(
                                      'properties_data_encrypted': encrypt},
                                     resource.atomic_key)
                 except Exception as exc:
-                    LOG.exception(_LE('Failed to %(crypt_action)s '
-                                      'properties_data of resource %(id)d') %
+                    LOG.exception('Failed to %(crypt_action)s '
+                                  'properties_data of resource %(id)d' %
                                   {'id': resource.id,
                                    'crypt_action': _crypt_action(encrypt)})
                     excs.append(exc)
                     continue
                 finally:
                     if verbose:
-                        LOG.info(_LI("Finished processing resource "
-                                     "%(id)d."), {'id': resource.id})
+                        LOG.info("Finished processing resource %s.",
+                                 resource.id)
         next_batch = list(itertools.islice(resource_batches, batch_size))
     return excs
 
@@ -1617,8 +1683,8 @@ def _db_encrypt_or_decrypt_resource_prop_data(
                     continue
                 try:
                     if verbose:
-                        LOG.info(_LI("Processing resource_properties_data "
-                                     "%(id)d..."), {'id': rpd.id})
+                        LOG.info("Processing resource_properties_data "
+                                 "%s...", rpd.id)
                     if encrypt:
                         result = crypt.encrypted_dict(rpd.data,
                                                       encryption_key)
@@ -1629,8 +1695,8 @@ def _db_encrypt_or_decrypt_resource_prop_data(
                                 'encrypted': encrypt})
                 except Exception as exc:
                     LOG.exception(
-                        _LE("Failed to %(crypt_action)s "
-                            "data of resource_properties_data %(id)d") %
+                        "Failed to %(crypt_action)s "
+                        "data of resource_properties_data %(id)d" %
                         {'id': rpd.id,
                          'crypt_action': _crypt_action(encrypt)})
                     excs.append(exc)
@@ -1638,8 +1704,8 @@ def _db_encrypt_or_decrypt_resource_prop_data(
                 finally:
                     if verbose:
                         LOG.info(
-                            _LI("Finished processing resource_properties_data"
-                                " %(id)d."), {'id': rpd.id})
+                            "Finished processing resource_properties_data"
+                            " %s.", rpd.id)
         next_batch = list(itertools.islice(rpd_batches, batch_size))
     return excs
 
@@ -1692,6 +1758,73 @@ def db_decrypt_parameters_and_properties(ctxt, encryption_key, batch_size=50,
     return excs
 
 
+def db_properties_data_migrate(ctxt, batch_size=50):
+    """Migrate properties data from legacy columns to new location in db.
+
+    :param ctxt: RPC context
+    :param batch_size: number of templates requested from db in each iteration.
+                       50 means that heat requests 50 templates, encrypt them
+                       and proceed with next 50 items.
+    """
+    session = ctxt.session
+
+    query = session.query(models.Resource).filter(and_(
+        models.Resource.properties_data.isnot(None),
+        models.Resource.rsrc_prop_data_id.is_(None)))
+    resource_batches = _get_batch(
+        session=session, ctxt=ctxt, query=query,
+        model=models.Resource, batch_size=batch_size)
+    next_batch = list(itertools.islice(resource_batches, batch_size))
+    while next_batch:
+        with session.begin():
+            for resource in next_batch:
+                try:
+                    encrypted = resource.properties_data_encrypted
+                    if encrypted is None:
+                        LOG.warning(
+                            'Unexpected: resource.encrypted is None for '
+                            'resource id %s for legacy '
+                            'resource.properties_data, assuming False.',
+                            resource.id)
+                        encrypted = False
+                    rsrc_prop_data = resource_prop_data_create(
+                        ctxt, {'encrypted': encrypted,
+                               'data': resource.properties_data})
+                    resource_update(ctxt, resource.id,
+                                    {'properties_data_encrypted': None,
+                                     'properties_data': None,
+                                     'rsrc_prop_data_id': rsrc_prop_data.id},
+                                    resource.atomic_key)
+                except Exception:
+                    LOG.exception('Failed to migrate properties_data for '
+                                  'resource %d', resource.id)
+                    continue
+        next_batch = list(itertools.islice(resource_batches, batch_size))
+
+    query = session.query(models.Event).filter(and_(
+        models.Event.resource_properties.isnot(None),
+        models.Event.rsrc_prop_data_id.is_(None)))
+    event_batches = _get_batch(
+        session=session, ctxt=ctxt, query=query,
+        model=models.Event, batch_size=batch_size)
+    next_batch = list(itertools.islice(event_batches, batch_size))
+    while next_batch:
+        with session.begin():
+            for event in next_batch:
+                try:
+                    prop_data = event.resource_properties
+                    rsrc_prop_data = resource_prop_data_create(
+                        ctxt, {'encrypted': False,
+                               'data': prop_data})
+                    event.update({'resource_properties': None,
+                                  'rsrc_prop_data_id': rsrc_prop_data.id})
+                except Exception:
+                    LOG.exception('Failed to migrate resource_properties '
+                                  'for event %d', event.id)
+                    continue
+        next_batch = list(itertools.islice(event_batches, batch_size))
+
+
 def _get_batch(session, ctxt, query, model, batch_size=50):
     last_batch_marker = None
     while True:
