@@ -22,8 +22,6 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 
 from heat.common import exception
-from heat.common.i18n import _LE
-from heat.common.i18n import _LI
 from heat.engine import resource
 from heat.engine import scheduler
 from heat.engine import stack as parser
@@ -50,15 +48,18 @@ class CheckResource(object):
                  engine_id,
                  rpc_client,
                  thread_group_mgr,
-                 msg_queue):
+                 msg_queue,
+                 input_data):
         self.engine_id = engine_id
         self._rpc_client = rpc_client
         self.thread_group_mgr = thread_group_mgr
         self.msg_queue = msg_queue
+        self.input_data = input_data
 
     def _try_steal_engine_lock(self, cnxt, resource_id):
         rs_obj = resource_objects.Resource.get_obj(cnxt,
-                                                   resource_id)
+                                                   resource_id,
+                                                   fields=('engine_id', ))
         if rs_obj.engine_id not in (None, self.engine_id):
             if not listener_client.EngineListenerClient(
                     rs_obj.engine_id).is_alive(cnxt):
@@ -68,7 +69,7 @@ class CheckResource(object):
         return False
 
     def _trigger_rollback(self, stack):
-        LOG.info(_LI("Triggering rollback of %(stack_name)s %(action)s "),
+        LOG.info("Triggering rollback of %(stack_name)s %(action)s ",
                  {'action': stack.action, 'stack_name': stack.name})
         stack.rollback()
 
@@ -113,9 +114,9 @@ class CheckResource(object):
                                           stack, self.msg_queue)
                 except resource.UpdateReplace:
                     new_res_id = rsrc.make_replacement(tmpl.id)
-                    LOG.info(_LI("Replacing resource with new id %s"),
+                    LOG.info("Replacing resource with new id %s",
                              new_res_id)
-                    rpc_data = sync_point.serialize_input_data(resource_data)
+                    rpc_data = sync_point.serialize_input_data(self.input_data)
                     self._rpc_client.check_resource(cnxt,
                                                     new_res_id,
                                                     current_traversal,
@@ -131,7 +132,7 @@ class CheckResource(object):
             return True
         except exception.UpdateInProgress:
             if self._try_steal_engine_lock(cnxt, rsrc.id):
-                rpc_data = sync_point.serialize_input_data(resource_data)
+                rpc_data = sync_point.serialize_input_data(self.input_data)
                 # set the resource state as failed
                 status_reason = ('Worker went down '
                                  'during resource %s' % rsrc.action)
@@ -177,7 +178,7 @@ class CheckResource(object):
             if (resource_id, True) in graph:
                 # not is_update evaluates to True below, which means update
                 key = (resource_id, not is_update)
-        LOG.info(_LI('Re-trigger resource: (%(key1)s, %(key2)s)'),
+        LOG.info('Re-trigger resource: (%(key1)s, %(key2)s)',
                  {'key1': key[0], 'key2': key[1]})
         predecessors = set(graph[key])
 
@@ -208,7 +209,7 @@ class CheckResource(object):
         def _get_input_data(req, fwd, input_forward_data=None):
             if fwd:
                 if input_forward_data is None:
-                    return construct_input_data(rsrc, stack)
+                    return rsrc.node_data().as_dict()
                 else:
                     # do not re-resolve attrs
                     return input_forward_data
@@ -232,7 +233,13 @@ class CheckResource(object):
                     cnxt, self._rpc_client, req, current_traversal,
                     set(graph[(req, fwd)]), graph_key, input_data, fwd,
                     stack.adopt_stack_data)
-
+            if is_update:
+                if input_forward_data is None:
+                    # we haven't resolved attribute data for the resource,
+                    # so clear any old attributes so they may be re-resolved
+                    rsrc.clear_stored_attributes()
+                else:
+                    rsrc.store_attributes()
             check_stack_complete(cnxt, stack, current_traversal,
                                  graph_key[0], deps, graph_key[1])
         except exception.EntityNotFound as e:
@@ -295,50 +302,16 @@ class CheckResource(object):
                                               stack, msg)
 
 
-def load_resource(cnxt, resource_id, resource_data, is_update):
-    if is_update:
-        cache_data = {in_data.get(
-            'name'): in_data for in_data in resource_data.values()
-            if in_data is not None}
-    else:
-        # no data to resolve in cleanup phase
-        cache_data = {}
-
+def load_resource(cnxt, resource_id, resource_data,
+                  current_traversal, is_update):
     try:
-        return resource.Resource.load(cnxt, resource_id,
-                                      is_update, cache_data)
+        return resource.Resource.load(cnxt, resource_id, current_traversal,
+                                      is_update, resource_data)
     except (exception.ResourceNotFound, exception.NotFound):
         # can be ignored
         return None, None, None
 
 
-def _resolve_attributes(dep_attrs, rsrc):
-    resolved_attributes = {}
-    for attr in dep_attrs:
-        try:
-            if isinstance(attr, six.string_types):
-                resolved_attributes[attr] = rsrc.get_attribute(attr)
-            else:
-                resolved_attributes[attr] = rsrc.get_attribute(*attr)
-        except exception.InvalidTemplateAttribute as ita:
-            LOG.info(six.text_type(ita))
-    return resolved_attributes
-
-
-def construct_input_data(rsrc, curr_stack):
-    dep_attrs = curr_stack.get_dep_attrs(
-        six.itervalues(curr_stack.resources),
-        rsrc.name)
-    input_data = {'id': rsrc.id,
-                  'name': rsrc.name,
-                  'reference_id': rsrc.get_reference_id(),
-                  'attrs': _resolve_attributes(dep_attrs, rsrc),
-                  'status': rsrc.status,
-                  'action': rsrc.action,
-                  'uuid': rsrc.uuid}
-    return input_data
-
-
 def check_stack_complete(cnxt, stack, current_traversal, sender_id, deps,
                          is_update):
     """Mark the stack complete if the update is complete.
@@ -383,7 +356,7 @@ def _check_for_message(msg_queue):
     if message == rpc_api.THREAD_CANCEL:
         raise CancelOperation
 
-    LOG.error(_LE('Unknown message "%s" received'), message)
+    LOG.error('Unknown message "%s" received', message)
 
 
 def check_resource_update(rsrc, template_id, resource_data, engine_id,
