@@ -24,15 +24,12 @@ from novaclient import exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
-from oslo_utils import uuidutils
 import six
 from six.moves.urllib import parse as urlparse
 import tenacity
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LI
-from heat.common.i18n import _LW
 from heat.engine.clients import client_plugin
 from heat.engine.clients import os as os_client
 from heat.engine import constraints
@@ -58,12 +55,16 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
 
     exceptions_module = exceptions
 
-    supported_versions = [
-        NOVA_API_VERSION, V2_2, V2_26
+    NOVA_API_VERSION = '2.1'
+
+    validate_versions = [
+        V2_2, V2_8, V2_10, V2_15, V2_26, V2_37, V2_42
     ] = [
-        '2.1', '2.2', '2.26'
+        '2.2', '2.8', '2.10', '2.15', '2.26', '2.37', '2.42'
     ]
 
+    supported_versions = [NOVA_API_VERSION] + validate_versions
+
     service_types = [COMPUTE] = ['compute']
 
     def _get_service_name(self):
@@ -89,7 +90,7 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
 
         client = nc.Client(version, **args)
         # NOTE: check for microversion availability
-        if version in [self.V2_2, self.V2_26]:
+        if version in self.validate_versions:
             try:
                 client.versions.get_current()
             except exceptions.NotAcceptable:
@@ -144,15 +145,15 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
         try:
             server = self.client().servers.get(server_id)
         except exceptions.OverLimit as exc:
-            LOG.warning(_LW("Received an OverLimit response when "
-                            "fetching server (%(id)s) : %(exception)s"),
+            LOG.warning("Received an OverLimit response when "
+                        "fetching server (%(id)s) : %(exception)s",
                         {'id': server_id,
                          'exception': exc})
         except exceptions.ClientException as exc:
             if ((getattr(exc, 'http_status', getattr(exc, 'code', None)) in
                  (500, 503))):
-                LOG.warning(_LW("Received the following exception when "
-                            "fetching server (%(id)s) : %(exception)s"),
+                LOG.warning("Received the following exception when "
+                            "fetching server (%(id)s) : %(exception)s",
                             {'id': server_id,
                              'exception': exc})
             else:
@@ -167,17 +168,17 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
         try:
             server.get()
         except exceptions.OverLimit as exc:
-            LOG.warning(_LW("Server %(name)s (%(id)s) received an OverLimit "
-                            "response during server.get(): %(exception)s"),
+            LOG.warning("Server %(name)s (%(id)s) received an OverLimit "
+                        "response during server.get(): %(exception)s",
                         {'name': server.name,
                          'id': server.id,
                          'exception': exc})
         except exceptions.ClientException as exc:
             if ((getattr(exc, 'http_status', getattr(exc, 'code', None)) in
                  (500, 503))):
-                LOG.warning(_LW('Server "%(name)s" (%(id)s) received the '
-                                'following exception during server.get(): '
-                                '%(exception)s'),
+                LOG.warning('Server "%(name)s" (%(id)s) received the '
+                            'following exception during server.get(): '
+                            '%(exception)s',
                             {'name': server.name,
                              'id': server.id,
                              'exception': exc})
@@ -568,7 +569,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         try:
             server = self.client().servers.get(server)
         except exceptions.NotFound as ex:
-            LOG.warning(_LW('Instance (%(server)s) not found: %(ex)s'),
+            LOG.warning('Instance (%(server)s) not found: %(ex)s',
                         {'server': server, 'ex': ex})
         else:
             for n in sorted(server.networks, reverse=True):
@@ -592,60 +593,43 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
 
         The actual console url is lazily resolved on access.
         """
+        nc = self.client
+        mks_version = self.V2_8
 
         class ConsoleUrls(collections.Mapping):
             def __init__(self, server):
-                self.console_methods = {
-                    'novnc': server.get_vnc_console,
-                    'xvpvnc': server.get_vnc_console,
-                    'spice-html5': server.get_spice_console,
-                    'rdp-html5': server.get_rdp_console,
-                    'serial': server.get_serial_console
-                }
+                self.console_method = server.get_console_url
+                self.support_console_types = ['novnc', 'xvpvnc',
+                                              'spice-html5', 'rdp-html5',
+                                              'serial', 'webmks']
 
             def __getitem__(self, key):
                 try:
-                    url = self.console_methods[key](key)['console']['url']
-                except exceptions.BadRequest as e:
-                    unavailable = 'Unavailable console type'
-                    if unavailable in e.message:
-                        url = e.message
+                    if key not in self.support_console_types:
+                        raise exceptions.UnsupportedConsoleType(key)
+                    if key == 'webmks':
+                        data = nc(mks_version).servers.get_console_url(
+                            server, key)
                     else:
-                        raise
+                        data = self.console_method(key)
+                    console_data = data.get(
+                        'remote_console', data.get('console'))
+                    url = console_data['url']
+                except exceptions.UnsupportedConsoleType as ex:
+                    url = ex.message
+                except Exception as e:
+                    url = _('Cannot get console url: %s') % six.text_type(e)
+
                 return url
 
             def __len__(self):
-                return len(self.console_methods)
+                return len(self.support_console_types)
 
             def __iter__(self):
-                return (key for key in self.console_methods)
+                return (key for key in self.support_console_types)
 
         return ConsoleUrls(server)
 
-    def get_net_id_by_label(self, label):
-        try:
-            net_id = self.client().networks.find(label=label).id
-        except exceptions.NotFound as ex:
-            LOG.debug('Nova network (%(net)s) not found: %(ex)s',
-                      {'net': label, 'ex': ex})
-            raise exception.EntityNotFound(entity='Nova network', name=label)
-        except exceptions.NoUniqueMatch as exc:
-            LOG.debug('Nova network (%(net)s) is not unique matched: %(exc)s',
-                      {'net': label, 'exc': exc})
-            raise exception.PhysicalResourceNameAmbiguity(name=label)
-        return net_id
-
-    def get_nova_network_id(self, net_identifier):
-        if uuidutils.is_uuid_like(net_identifier):
-            try:
-                net_id = self.client().networks.get(net_identifier).id
-            except exceptions.NotFound:
-                net_id = self.get_net_id_by_label(net_identifier)
-        else:
-            net_id = self.get_net_id_by_label(net_identifier)
-
-        return net_id
-
     def attach_volume(self, server_id, volume_id, device):
         try:
             va = self.client().volumes.create_server_volume(
@@ -687,12 +671,12 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
             self.client().volumes.get_server_volume(server_id, attach_id)
         except Exception as ex:
             self.ignore_not_found(ex)
-            LOG.info(_LI("Volume %(vol)s is detached from server %(srv)s"),
+            LOG.info("Volume %(vol)s is detached from server %(srv)s",
                      {'vol': attach_id, 'srv': server_id})
             return True
         else:
-            LOG.debug("Server %(srv)s still has attachment %(att)s." % {
-                'att': attach_id, 'srv': server_id})
+            LOG.debug("Server %(srv)s still has attachment %(att)s.",
+                      {'att': attach_id, 'srv': server_id})
             return False
 
     def interface_detach(self, server_id, port_id):
@@ -702,10 +686,15 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                 server.interface_detach(port_id)
                 return True
 
-    def interface_attach(self, server_id, port_id=None, net_id=None, fip=None):
+    def interface_attach(self, server_id, port_id=None, net_id=None, fip=None,
+                         security_groups=None):
         server = self.fetch_server(server_id)
         if server:
-            server.interface_attach(port_id, net_id, fip)
+            attachment = server.interface_attach(port_id, net_id, fip)
+            if not port_id and security_groups:
+                props = {'security_groups': security_groups}
+                self.clients.client('neutron').update_port(
+                    attachment.port_id, {'port': props})
             return True
         else:
             return False
@@ -731,6 +720,9 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         wait=tenacity.wait_fixed(0.5),
         retry=tenacity.retry_if_result(client_plugin.retry_if_result_is_false))
     def check_interface_attach(self, server_id, port_id):
+        if not port_id:
+            return True
+
         server = self.fetch_server(server_id)
         if server:
             interfaces = server.interface_list()
@@ -778,14 +770,6 @@ class FlavorConstraint(NovaBaseConstraint):
     resource_getter_name = 'find_flavor_by_name_or_id'
 
 
-class NetworkConstraint(NovaBaseConstraint):
-
-    expected_exceptions = (exception.EntityNotFound,
-                           exception.PhysicalResourceNameAmbiguity)
-
-    resource_getter_name = 'get_nova_network_id'
-
-
 class HostConstraint(NovaBaseConstraint):
 
     resource_getter_name = 'get_host'
