@@ -268,6 +268,11 @@ class HOTemplate20130523(template_common.CommonTemplate):
             self.t[self.RESOURCES] = {}
         self.t[self.RESOURCES][name] = definition.render_hot()
 
+    def add_output(self, definition):
+        if self.t.get(self.OUTPUTS) is None:
+            self.t[self.OUTPUTS] = {}
+        self.t[self.OUTPUTS][definition.name] = definition.render_hot()
+
 
 class HOTemplate20141016(HOTemplate20130523):
     functions = {
@@ -479,7 +484,7 @@ class HOTemplate20161014(HOTemplate20160408):
 
         self._parser_condition_functions = {}
         for n, f in six.iteritems(self.functions):
-            if not isinstance(f, hot_funcs.Removed):
+            if not f == hot_funcs.Removed:
                 self._parser_condition_functions[n] = function.Invalid
             else:
                 self._parser_condition_functions[n] = f
@@ -557,3 +562,66 @@ class HOTemplate20170224(HOTemplate20161014):
     }
 
     param_schema_class = parameters.HOTParamSchema20170224
+
+
+class HOTemplate20170901(HOTemplate20170224):
+    functions = {
+        'get_attr': hot_funcs.GetAttAllAttributes,
+        'get_file': hot_funcs.GetFile,
+        'get_param': hot_funcs.GetParam,
+        'get_resource': hot_funcs.GetResource,
+        'list_join': hot_funcs.JoinMultiple,
+        'repeat': hot_funcs.RepeatWithNestedLoop,
+        'resource_facade': hot_funcs.ResourceFacade,
+        'str_replace': hot_funcs.ReplaceJson,
+
+        # functions added in 2015-04-30
+        'digest': hot_funcs.Digest,
+
+        # functions added in 2015-10-15
+        'str_split': hot_funcs.StrSplit,
+
+        # functions added in 2016-04-08
+        'map_merge': hot_funcs.MapMerge,
+
+        # functions added in 2016-10-14
+        'yaql': hot_funcs.Yaql,
+        'map_replace': hot_funcs.MapReplace,
+        'if': hot_funcs.If,
+
+        # functions added in 2017-02-24
+        'filter': hot_funcs.Filter,
+        'str_replace_strict': hot_funcs.ReplaceJsonStrict,
+
+        # functions added in 2017-09-01
+        'make_url': hot_funcs.MakeURL,
+        'list_concat': hot_funcs.ListConcat,
+        'str_replace_vstrict': hot_funcs.ReplaceJsonVeryStrict,
+        'list_concat_unique': hot_funcs.ListConcatUnique,
+        'contains': hot_funcs.Contains,
+
+        # functions removed from 2015-10-15
+        'Fn::Select': hot_funcs.Removed,
+
+        # functions removed from 2014-10-16
+        'Fn::GetAZs': hot_funcs.Removed,
+        'Fn::Join': hot_funcs.Removed,
+        'Fn::Split': hot_funcs.Removed,
+        'Fn::Replace': hot_funcs.Removed,
+        'Fn::Base64': hot_funcs.Removed,
+        'Fn::MemberListToMap': hot_funcs.Removed,
+        'Fn::ResourceFacade': hot_funcs.Removed,
+        'Ref': hot_funcs.Removed,
+    }
+
+    condition_functions = {
+        'get_param': hot_funcs.GetParam,
+        'equals': hot_funcs.Equals,
+        'not': hot_funcs.Not,
+        'and': hot_funcs.And,
+        'or': hot_funcs.Or,
+
+        # functions added in 2017-09-01
+        'yaql': hot_funcs.Yaql,
+        'contains': hot_funcs.Contains
+    }
