@@ -11,6 +11,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import abc
 import collections
 import itertools
 
@@ -21,6 +22,7 @@ import six
 
 from heat.common import exception
 from heat.common.i18n import _
+from heat.common import param_utils
 from heat.engine import constraints as constr
 
 
@@ -163,8 +165,9 @@ class Schema(constr.Schema):
                                               'false')).lower() == 'true',
                    label=schema_dict.get(LABEL))
 
-    def validate_value(self, value, context=None):
-        super(Schema, self).validate_constraints(value, context)
+    def validate_value(self, value, context=None, template=None):
+        super(Schema, self).validate_constraints(value, context=context,
+                                                 template=template)
 
     def __getitem__(self, key):
         if key == self.TYPE:
@@ -201,7 +204,9 @@ class Parameter(object):
         else:
             raise ValueError(_('Invalid Parameter type "%s"') % schema.type)
 
-        return ParamClass(name, schema, value)
+        return super(Parameter, cls).__new__(ParamClass)
+
+    __slots__ = ('name', 'schema', 'user_value', 'user_default')
 
     def __init__(self, name, schema, value=None):
         """Initialise the parameter.
@@ -214,7 +219,7 @@ class Parameter(object):
         self.user_value = value
         self.user_default = None
 
-    def validate(self, validate_value=True, context=None):
+    def validate(self, validate_value=True, context=None, template=None):
         """Validates the parameter.
 
         This method validates if the parameter's schema is valid,
@@ -230,9 +235,9 @@ class Parameter(object):
                 return
 
             if self.user_value is not None:
-                self._validate(self.user_value, context)
+                self._validate(self.user_value, context, template)
             elif self.has_default():
-                self._validate(self.default(), context)
+                self._validate(self.default(), context, template)
             else:
                 raise exception.UserParameterMissing(key=self.name)
         except exception.StackValidationFailed as ex:
@@ -285,18 +290,24 @@ class Parameter(object):
     def set_default(self, value):
         self.user_default = value
 
+    @classmethod
+    def _value_as_text(cls, value):
+        return six.text_type(value)
+
     def __str__(self):
         """Return a string representation of the parameter."""
         value = self.value()
         if self.hidden():
             return six.text_type('******')
         else:
-            return six.text_type(value)
+            return self._value_as_text(value)
 
 
 class NumberParam(Parameter):
     """A template parameter of type "Number"."""
 
+    __slots__ = tuple()
+
     def __int__(self):
         """Return an integer representation of the parameter."""
         return int(super(NumberParam, self).value())
@@ -305,12 +316,12 @@ class NumberParam(Parameter):
         """Return a float representation of the parameter."""
         return float(super(NumberParam, self).value())
 
-    def _validate(self, val, context):
+    def _validate(self, val, context, template=None):
         try:
             Schema.str_to_num(val)
-        except ValueError as ex:
+        except (ValueError, TypeError) as ex:
             raise exception.StackValidationFailed(message=six.text_type(ex))
-        self.schema.validate_value(val, context)
+        self.schema.validate_value(val, context=context, template=template)
 
     def value(self):
         return Schema.str_to_num(super(NumberParam, self).value())
@@ -319,12 +330,14 @@ class NumberParam(Parameter):
 class BooleanParam(Parameter):
     """A template parameter of type "Boolean"."""
 
-    def _validate(self, val, context):
+    __slots__ = tuple()
+
+    def _validate(self, val, context, template=None):
         try:
             strutils.bool_from_string(val, strict=True)
         except ValueError as ex:
             raise exception.StackValidationFailed(message=six.text_type(ex))
-        self.schema.validate_value(val, context)
+        self.schema.validate_value(val, context=context, template=template)
 
     def value(self):
         if self.user_value is not None:
@@ -337,13 +350,20 @@ class BooleanParam(Parameter):
 class StringParam(Parameter):
     """A template parameter of type "String"."""
 
-    def _validate(self, val, context):
-        self.schema.validate_value(val, context)
+    __slots__ = tuple()
+
+    def _validate(self, val, context, template=None):
+        self.schema.validate_value(val, context=context, template=template)
+
+    def value(self):
+        return self.schema.to_schema_type(super(StringParam, self).value())
 
 
 class ParsedParameter(Parameter):
     """A template parameter with cached parsed value."""
 
+    __slots__ = ('parsed',)
+
     def __init__(self, name, schema, value=None):
         super(ParsedParameter, self).__init__(name, schema, value)
         self._update_parsed()
@@ -363,6 +383,8 @@ class ParsedParameter(Parameter):
 class CommaDelimitedListParam(ParsedParameter, collections.Sequence):
     """A template parameter of type "CommaDelimitedList"."""
 
+    __slots__ = ('parsed',)
+
     def __init__(self, name, schema, value=None):
         self.parsed = []
         super(CommaDelimitedListParam, self).__init__(name, schema, value)
@@ -372,10 +394,7 @@ class CommaDelimitedListParam(ParsedParameter, collections.Sequence):
         if isinstance(value, list):
             return [(six.text_type(x)) for x in value]
         try:
-            if value is not None:
-                if value == '':
-                    return []
-                return value.split(',')
+            return param_utils.delim_string_to_list(value)
         except (KeyError, AttributeError) as err:
             message = _('Value must be a comma-delimited list string: %s')
             raise ValueError(message % six.text_type(err))
@@ -395,22 +414,23 @@ class CommaDelimitedListParam(ParsedParameter, collections.Sequence):
         """Return an item from the list."""
         return self.parsed[index]
 
-    def __str__(self):
-        if self.hidden():
-            return super(CommaDelimitedListParam, self).__str__()
-        return ",".join(self.value())
+    @classmethod
+    def _value_as_text(cls, value):
+        return ",".join(value)
 
-    def _validate(self, val, context):
+    def _validate(self, val, context, template=None):
         try:
             parsed = self.parse(val)
         except ValueError as ex:
             raise exception.StackValidationFailed(message=six.text_type(ex))
-        self.schema.validate_value(parsed, context)
+        self.schema.validate_value(parsed, context=context, template=template)
 
 
 class JsonParam(ParsedParameter):
     """A template parameter who's value is map or list."""
 
+    __slots__ = ('parsed',)
+
     def __init__(self, name, schema, value=None):
         self.parsed = {}
         super(JsonParam, self).__init__(name, schema, value)
@@ -443,19 +463,19 @@ class JsonParam(ParsedParameter):
     def __len__(self):
         return len(self.parsed)
 
-    def __str__(self):
-        if self.hidden():
-            return super(JsonParam, self).__str__()
-        return encodeutils.safe_decode(jsonutils.dumps(self.value()))
+    @classmethod
+    def _value_as_text(cls, value):
+        return encodeutils.safe_decode(jsonutils.dumps(value))
 
-    def _validate(self, val, context):
+    def _validate(self, val, context, template=None):
         try:
             parsed = self.parse(val)
         except ValueError as ex:
             raise exception.StackValidationFailed(message=six.text_type(ex))
-        self.schema.validate_value(parsed, context)
+        self.schema.validate_value(parsed, context=context, template=template)
 
 
+@six.add_metaclass(abc.ABCMeta)
 class Parameters(collections.Mapping):
     """Parameters of a stack.
 
@@ -463,12 +483,6 @@ class Parameters(collections.Mapping):
     the stack's template.
     """
 
-    PSEUDO_PARAMETERS = (
-        PARAM_STACK_ID, PARAM_STACK_NAME, PARAM_REGION
-    ) = (
-        'AWS::StackId', 'AWS::StackName', 'AWS::Region'
-    )
-
     def __init__(self, stack_identifier, tmpl, user_params=None,
                  param_defaults=None):
         """Initialisation of the parameter.
@@ -498,9 +512,9 @@ class Parameters(collections.Mapping):
         self.non_pseudo_param_keys = [p for p in self.params if p not in
                                       self.PSEUDO_PARAMETERS]
 
-        for pd in six.iterkeys(param_defaults):
-            if pd in self.params:
-                self.params[pd].set_default(param_defaults[pd])
+        for pd_name, param_default in param_defaults.items():
+            if pd_name in self.params:
+                self.params[pd_name].set_default(param_default)
 
     def validate(self, validate_value=True, context=None):
         """Validates all parameters.
@@ -508,11 +522,10 @@ class Parameters(collections.Mapping):
         This method validates if all user-provided parameters are actually
         defined in the template, and if all parameters are valid.
         """
-        self._validate_tmpl_parameters()
         self._validate_user_parameters()
 
         for param in six.itervalues(self.params):
-            param.validate(validate_value, context)
+            param.validate(validate_value, context, self.tmpl)
 
     def __contains__(self, key):
         """Return whether the specified parameter exists."""
@@ -553,42 +566,9 @@ class Parameters(collections.Mapping):
             if param not in schemata:
                 raise exception.UnknownUserParameter(key=param)
 
-    def _validate_tmpl_parameters(self):
-        param = None
-        for key in six.iterkeys(self.tmpl.t):
-            if key == 'Parameters' or key == 'parameters':
-                param = key
-                break
-        if param is not None:
-            template_params = self.tmpl.t[key] or {}
-            for name, attrs in six.iteritems(template_params):
-                if not isinstance(attrs, dict):
-                    raise exception.InvalidTemplateParameter(key=name)
-
+    @abc.abstractmethod
     def _pseudo_parameters(self, stack_identifier):
-        stack_id = (stack_identifier.arn()
-                    if stack_identifier is not None else 'None')
-        stack_name = stack_identifier and stack_identifier.stack_name
-
-        yield Parameter(self.PARAM_STACK_ID,
-                        Schema(Schema.STRING, _('Stack ID'),
-                               default=str(stack_id)))
-        if stack_name:
-            yield Parameter(self.PARAM_STACK_NAME,
-                            Schema(Schema.STRING, _('Stack Name'),
-                                   default=stack_name))
-            yield Parameter(self.PARAM_REGION,
-                            Schema(Schema.STRING,
-                                   default='ap-southeast-1',
-                                   constraints=[
-                                       constr.AllowedValues(['us-east-1',
-                                                             'us-west-1',
-                                                             'us-west-2',
-                                                             'sa-east-1',
-                                                             'eu-west-1',
-                                                             'ap-southeast-1',
-                                                             'ap-northeast-1']
-                                                            )]))
+        pass
 
     def immutable_params_modified(self, new_parameters, input_params):
         # A parameter must have been present in the old stack for its
