@@ -18,11 +18,13 @@ import six
 
 from heat.common import exception
 from heat.common.i18n import _
+from heat.common import param_utils
 from heat.engine import constraints as constr
 from heat.engine import function
 from heat.engine.hot import parameters as hot_param
 from heat.engine import parameters
 from heat.engine import support
+from heat.engine import translation as trans
 
 SCHEMA_KEYS = (
     REQUIRED, IMPLEMENTED, DEFAULT, TYPE, SCHEMA,
@@ -71,6 +73,17 @@ class Schema(constr.Schema):
         # validate structural correctness of schema itself
         self.validate()
 
+    def validate(self, context=None):
+        super(Schema, self).validate()
+        # check that update_allowed and immutable
+        # do not contradict each other
+        if self.update_allowed and self.immutable:
+            msg = _("Options %(ua)s and %(im)s "
+                    "cannot both be True") % {
+                        'ua': UPDATE_ALLOWED,
+                        'im': IMMUTABLE}
+            raise exception.InvalidSchemaError(message=msg)
+
     @classmethod
     def from_legacy(cls, schema_dict):
         """Return a Property Schema object from a legacy schema dictionary."""
@@ -151,7 +164,8 @@ class Schema(constr.Schema):
         # a json parameter value is passed via a Map property, which requires
         # some coercion to pass strings or lists (which are both valid for
         # Json parameters but not for Map properties).
-        allow_conversion = param.type == param.MAP
+        allow_conversion = (param.type == param.MAP
+                            or param.type == param.LIST)
 
         # make update_allowed true by default on TemplateResources
         # as the template should deal with this.
@@ -200,10 +214,11 @@ def schemata(schema_dicts):
 
 class Property(object):
 
-    def __init__(self, schema, name=None, context=None):
+    def __init__(self, schema, name=None, context=None, path=None):
         self.schema = Schema.from_legacy(schema)
         self.name = name
         self.context = context
+        self.path = self.make_path(name, path)
 
     def required(self):
         return self.schema.required
@@ -229,6 +244,18 @@ class Property(object):
     def support_status(self):
         return self.schema.support_status
 
+    def make_path(self, name, path=None):
+        if path is None:
+            path = ''
+        if name is None:
+            name = ''
+
+        if isinstance(name, int) or name.isdigit():
+            name = str(name)
+
+        delim = '' if not path or path.endswith('.') else '.'
+        return delim.join([path, name])
+
     def _get_integer(self, value):
         if value is None:
             value = self.has_default() and self.default() or 0
@@ -254,13 +281,16 @@ class Property(object):
                 raise ValueError(_('Value must be a string'))
         return value
 
-    def _get_children(self, child_values, keys=None, validate=False):
+    def _get_children(self, child_values, keys=None, validate=False,
+                      translation=None):
         if self.schema.schema is not None:
             if keys is None:
                 keys = list(self.schema.schema)
             schemata = dict((k, self.schema.schema[k]) for k in keys)
             properties = Properties(schemata, dict(child_values),
-                                    context=self.context)
+                                    context=self.context,
+                                    parent_name=self.path,
+                                    translation=translation)
             if validate:
                 properties.validate()
 
@@ -268,7 +298,7 @@ class Property(object):
         else:
             return child_values
 
-    def _get_map(self, value, validate=False):
+    def _get_map(self, value, validate=False, translation=None):
         if value is None:
             value = self.default() if self.has_default() else {}
         if not isinstance(value, collections.Mapping):
@@ -283,31 +313,44 @@ class Property(object):
             raise TypeError(_('"%s" is not a map') % value)
 
         return dict(self._get_children(six.iteritems(value),
-                                       validate=validate))
+                                       validate=validate,
+                                       translation=translation))
 
-    def _get_list(self, value, validate=False):
+    def _get_list(self, value, validate=False, translation=None):
         if value is None:
             value = self.has_default() and self.default() or []
+        if self.schema.allow_conversion and isinstance(value,
+                                                       six.string_types):
+                value = param_utils.delim_string_to_list(value)
         if (not isinstance(value, collections.Sequence) or
                 isinstance(value, six.string_types)):
             raise TypeError(_('"%s" is not a list') % repr(value))
 
         return [v[1] for v in self._get_children(enumerate(value),
                                                  range(len(value)),
-                                                 validate)]
+                                                 validate=validate,
+                                                 translation=translation)]
 
     def _get_bool(self, value):
+        """Get value for boolean property.
+
+        Explicitly checking for bool, or string with lower value
+        "true" or "false", to avoid integer values.
+        """
         if value is None:
             value = self.has_default() and self.default() or False
         if isinstance(value, bool):
             return value
-        normalised = value.lower()
-        if normalised not in ['true', 'false']:
-            raise ValueError(_('"%s" is not a valid boolean') % normalised)
+        if isinstance(value, six.string_types):
+            normalised = value.lower()
+            if normalised not in ['true', 'false']:
+                raise ValueError(_('"%s" is not a valid boolean') % normalised)
+            return normalised == 'true'
 
-        return normalised == 'true'
+        raise TypeError(_('"%s" is not a valid boolean') % value)
 
-    def get_value(self, value, validate=False):
+    def get_value(self, value, validate=False, template=None,
+                  translation=None):
         """Get value from raw value and sanitize according to data type."""
 
         t = self.type()
@@ -318,14 +361,17 @@ class Property(object):
         elif t == Schema.NUMBER:
             _value = self._get_number(value)
         elif t == Schema.MAP:
-            _value = self._get_map(value, validate)
+            _value = self._get_map(value, validate, translation)
         elif t == Schema.LIST:
-            _value = self._get_list(value, validate)
+            _value = self._get_list(value, validate, translation)
         elif t == Schema.BOOLEAN:
             _value = self._get_bool(value)
+        elif t == Schema.ANY:
+            _value = value
 
         if validate:
-            self.schema.validate_constraints(_value, self.context)
+            self.schema.validate_constraints(_value, self.context,
+                                             template=template)
 
         return _value
 
@@ -333,17 +379,19 @@ class Property(object):
 class Properties(collections.Mapping):
 
     def __init__(self, schema, data, resolver=lambda d: d, parent_name=None,
-                 context=None, section=None):
-        self.props = dict((k, Property(s, k, context))
+                 context=None, section=None, translation=None):
+        self.props = dict((k, Property(s, k, context, path=parent_name))
                           for k, s in schema.items())
         self.resolve = resolver
         self.data = data
-        self.error_prefix = []
-        if parent_name is not None:
-            self.error_prefix.append(parent_name)
-        if section is not None:
-            self.error_prefix.append(section)
+        self.error_prefix = [section] if section is not None else []
+        self.parent_name = parent_name
         self.context = context
+        self.translation = (trans.Translation(properties=self)
+                            if translation is None else translation)
+
+    def update_translation(self, rules, client_resolve=True):
+        self.translation.set_rules(rules, client_resolve=client_resolve)
 
     @staticmethod
     def schema_from_params(params_snippet):
@@ -357,7 +405,7 @@ class Properties(collections.Mapping):
                         in params_snippet.items())
         return {}
 
-    def validate(self, with_value=True):
+    def validate(self, with_value=True, template=None):
         try:
             for key in self.data:
                 if key not in self.props:
@@ -365,19 +413,14 @@ class Properties(collections.Mapping):
                     raise exception.StackValidationFailed(message=msg)
 
             for (key, prop) in self.props.items():
-                # check that update_allowed and immutable
-                # do not contradict each other
-                if prop.update_allowed() and prop.immutable():
-                    msg = _("Property %(prop)s: %(ua)s and %(im)s "
-                            "cannot both be True") % {
-                                'prop': key,
-                                'ua': prop.schema.UPDATE_ALLOWED,
-                                'im': prop.schema.IMMUTABLE}
-                    raise exception.InvalidSchemaError(message=msg)
-
+                if (self.translation.is_deleted(prop.path) or
+                        self.translation.is_replaced(prop.path)):
+                    continue
                 if with_value:
                     try:
-                        self._get_property_value(key, validate=True)
+                        self._get_property_value(key,
+                                                 validate=True,
+                                                 template=template)
                     except exception.StackValidationFailed as ex:
                         path = [key]
                         path.extend(ex.path)
@@ -412,11 +455,14 @@ class Properties(collections.Mapping):
         if any(res.action == res.INIT for res in deps):
             return True
 
-    def get_user_value(self, key, validate=False):
+    def get_user_value(self, key, validate=False, template=None):
         if key not in self:
             raise KeyError(_('Invalid Property %s') % key)
 
         prop = self.props[key]
+        if (self.translation.is_deleted(prop.path) or
+                self.translation.is_replaced(prop.path)):
+            return
         if key in self.data:
             try:
                 unresolved_value = self.data[key]
@@ -425,7 +471,14 @@ class Properties(collections.Mapping):
                         validate = False
 
                 value = self.resolve(unresolved_value)
-                return prop.get_value(value, validate)
+
+                if self.translation.has_translation(prop.path):
+                    value = self.translation.translate(prop.path,
+                                                       value,
+                                                       self.data)
+
+                return prop.get_value(value, validate, template=template,
+                                      translation=self.translation)
             # Children can raise StackValidationFailed with unique path which
             # is necessary for further use in StackValidationFailed exception.
             # So we need to handle this exception in this method.
@@ -437,15 +490,24 @@ class Properties(collections.Mapping):
             except Exception as e:
                 raise ValueError(six.text_type(e))
 
-    def _get_property_value(self, key, validate=False):
+    def _get_property_value(self, key, validate=False, template=None):
         if key not in self:
             raise KeyError(_('Invalid Property %s') % key)
 
         prop = self.props[key]
-        if key in self.data:
-            return self.get_user_value(key, validate)
+        if not self.translation.is_deleted(prop.path) and key in self.data:
+            return self.get_user_value(key, validate, template=template)
+        elif self.translation.has_translation(prop.path):
+            value = self.translation.translate(prop.path, prop_data=self.data,
+                                               validate=validate,
+                                               template=template)
+            if value is not None or prop.has_default():
+                return prop.get_value(value)
+            elif prop.required():
+                raise ValueError(_('Property %s not assigned') % key)
         elif prop.has_default():
-            return prop.get_value(None, validate)
+            return prop.get_value(None, validate, template=template,
+                                  translation=self.translation)
         elif prop.required():
             raise ValueError(_('Property %s not assigned') % key)
 
