@@ -13,6 +13,7 @@
 
 import collections
 import copy
+import functools
 import itertools
 
 import six
@@ -24,6 +25,7 @@ from heat.common import timeutils
 from heat.engine import attributes
 from heat.engine import constraints
 from heat.engine import function
+from heat.engine import output
 from heat.engine import properties
 from heat.engine.resources import stack_resource
 from heat.engine import rsrc_defn
@@ -182,7 +184,7 @@ class ResourceGroup(stack_resource.StackResource):
                           "the 'refs' attribute. "
                           "Note this is destructive on update when specified; "
                           "even if the count is not being reduced, and once "
-                          "a resource name is removed, it's name is never "
+                          "a resource name is removed, its name is never "
                           "reused in subsequent updates."
                           ),
                         default=[]
@@ -278,9 +280,10 @@ class ResourceGroup(stack_resource.StackResource):
         if not self.get_size():
             return
 
-        test_tmpl = self._assemble_nested(["0"], include_all=True)
-        res_def = next(six.itervalues(
-            test_tmpl.resource_definitions(self.stack)))
+        first_name = next(self._resource_names(update_rsrc_data=False))
+        test_tmpl = self._assemble_nested([first_name],
+                                          include_all=True)
+        res_def = next(six.itervalues(test_tmpl.resource_definitions(None)))
         # make sure we can resolve the nested resource type
         self.stack.env.get_class_to_instantiate(res_def.resource_type)
 
@@ -293,8 +296,9 @@ class ResourceGroup(stack_resource.StackResource):
             nested_stack.strict_validate = False
             nested_stack.validate()
         except Exception as ex:
-            msg = _("Failed to validate: %s") % six.text_type(ex)
-            raise exception.StackValidationFailed(message=msg)
+            path = "%s<%s>" % (self.name, self.template_url)
+            raise exception.StackValidationFailed(
+                ex, path=[self.stack.t.RESOURCES, path])
 
     def _current_blacklist(self):
         db_rsrc_names = self.data().get('name_blacklist')
@@ -303,7 +307,7 @@ class ResourceGroup(stack_resource.StackResource):
         else:
             return []
 
-    def _name_blacklist(self):
+    def _name_blacklist(self, update_rsrc_data=True):
         """Resolve the remove_policies to names for removal."""
 
         nested = self.nested()
@@ -329,12 +333,12 @@ class ResourceGroup(stack_resource.StackResource):
                         rsrc_names.add(rsrc.name)
 
         # If the blacklist has changed, update the resource data
-        if rsrc_names != set(current_blacklist):
+        if update_rsrc_data and rsrc_names != set(current_blacklist):
             self.data_set('name_blacklist', ','.join(rsrc_names))
         return rsrc_names
 
-    def _resource_names(self, size=None):
-        name_blacklist = self._name_blacklist()
+    def _resource_names(self, size=None, update_rsrc_data=True):
+        name_blacklist = self._name_blacklist(update_rsrc_data)
         if size is None:
             size = self.get_size()
 
@@ -451,6 +455,31 @@ class ResourceGroup(stack_resource.StackResource):
         return [grouputils.get_rsrc_attr(self, key, False, n, *path)
                 for n in names]
 
+    def _nested_output_defns(self, resource_names, get_attr_fn):
+        for attr in self.referenced_attrs():
+            if isinstance(attr, six.string_types):
+                key, path = attr, []
+                output_name = attr
+            else:
+                key, path = attr[0], list(attr[1:])
+                output_name = ', '.join(attr)
+
+            if key.startswith("resource."):
+                keycomponents = key.split('.', 2)
+                res_name = keycomponents[1]
+                attr_name = keycomponents[2:]
+                if attr_name and (res_name in resource_names):
+                    value = get_attr_fn([res_name] + attr_name + path)
+                    yield output.OutputDefinition(output_name, value)
+
+            elif key == self.ATTR_ATTRIBUTES and path:
+                value = {r: get_attr_fn([r] + path) for r in resource_names}
+                yield output.OutputDefinition(output_name, value)
+
+            elif key not in self.ATTRIBUTES:
+                value = [get_attr_fn([r, key] + path) for r in resource_names]
+                yield output.OutputDefinition(output_name, value)
+
     def build_resource_definition(self, res_name, res_defn):
         res_def = copy.deepcopy(res_defn)
 
@@ -529,9 +558,17 @@ class ResourceGroup(stack_resource.StackResource):
         def_dict = self.get_resource_def(include_all)
         definitions = [(k, self.build_resource_definition(k, def_dict))
                        for k in names]
-        return scl_template.make_template(definitions,
+        tmpl = scl_template.make_template(definitions,
                                           version=template_version)
 
+        att_func = 'get_attr'
+        get_attr = functools.partial(tmpl.functions[att_func], None, att_func)
+        for odefn in self._nested_output_defns([k for k, d in definitions],
+                                               get_attr):
+            tmpl.add_output(odefn)
+
+        return tmpl
+
     def _assemble_for_rolling_update(self, total_capacity, max_updates,
                                      include_all=False,
                                      template_version=('heat_template_version',
