@@ -16,7 +16,6 @@ from oslo_serialization import jsonutils
 import six
 
 from heat.common.i18n import _
-from heat.common.i18n import _LW
 from heat.engine import attributes
 from heat.engine import constraints
 from heat.engine import properties
@@ -40,12 +39,16 @@ class Port(neutron.NeutronResource):
     was taken from the allocation pool for a specific subnet.
     """
 
+    entity = 'port'
+
     PROPERTIES = (
         NAME, NETWORK_ID, NETWORK, FIXED_IPS, SECURITY_GROUPS,
         REPLACEMENT_POLICY, DEVICE_ID, DEVICE_OWNER, DNS_NAME,
+        TAGS,
     ) = (
         'name', 'network_id', 'network', 'fixed_ips', 'security_groups',
         'replacement_policy', 'device_id', 'device_owner', 'dns_name',
+        'tags',
     )
 
     EXTRA_PROPERTIES = (
@@ -187,12 +190,17 @@ class Port(neutron.NeutronResource):
             ],
             update_allowed=True,
             support_status=support.SupportStatus(
-                status=support.DEPRECATED,
-                version='6.0.0',
-                message=_('Replacement policy used to work around flawed '
-                          'nova/neutron port interaction which has been '
-                          'fixed since Liberty.'),
-                previous_status=support.SupportStatus(version='2014.2'))
+                status=support.HIDDEN,
+                version='9.0.0',
+                previous_status=support.SupportStatus(
+                    status=support.DEPRECATED,
+                    version='6.0.0',
+                    message=_('Replacement policy used to work around flawed '
+                              'nova/neutron port interaction which has been '
+                              'fixed since Liberty.'),
+                    previous_status=support.SupportStatus(version='2014.2')
+                )
+            )
         ),
         DNS_NAME: properties.Schema(
             properties.Schema.STRING,
@@ -203,6 +211,13 @@ class Port(neutron.NeutronResource):
             ],
             support_status=support.SupportStatus(version='7.0.0'),
         ),
+        TAGS: properties.Schema(
+            properties.Schema.LIST,
+            _('The tags to be added to the port.'),
+            schema=properties.Schema(properties.Schema.STRING),
+            update_allowed=True,
+            support_status=support.SupportStatus(version='9.0.0')
+        ),
     }
 
     # NOTE(prazumovsky): properties_schema has been separated because some
@@ -416,6 +431,7 @@ class Port(neutron.NeutronResource):
         props['network_id'] = props.pop(self.NETWORK)
         self._prepare_port_properties(props)
         qos_policy = props.pop(self.QOS_POLICY, None)
+        tags = props.pop(self.TAGS, [])
         if qos_policy:
             props['qos_policy_id'] = self.client_plugin().get_qos_policy_id(
                 qos_policy)
@@ -423,6 +439,9 @@ class Port(neutron.NeutronResource):
         port = self.client().create_port({'port': props})['port']
         self.resource_id_set(port['id'])
 
+        if tags:
+            self.set_tags(tags)
+
     def _prepare_port_properties(self, props, prepare_for_update=False):
         if self.FIXED_IPS in props:
             fixed_ips = props[self.FIXED_IPS]
@@ -471,12 +490,19 @@ class Port(neutron.NeutronResource):
         if self.REPLACEMENT_POLICY in props:
             del(props[self.REPLACEMENT_POLICY])
 
-    def _show_resource(self):
-        return self.client().show_port(
-            self.resource_id)['port']
+    def _store_config_default_properties(self, attrs):
+        """A method for storing properties default values.
+
+        A method allows to store properties default values, which cannot be
+        defined in schema in case of specifying in config file.
+        """
+        super(Port, self)._store_config_default_properties(attrs)
+        if self.VNIC_TYPE in attrs:
+            self.data_set(self.VNIC_TYPE, attrs[self.VNIC_TYPE])
 
     def check_create_complete(self, *args):
         attributes = self._show_resource()
+        self._store_config_default_properties(attributes)
         return self.is_built(attributes)
 
     def handle_delete(self):
@@ -487,6 +513,20 @@ class Port(neutron.NeutronResource):
         else:
             return True
 
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        result = super(Port, self).parse_live_resource_data(
+            resource_properties, resource_data)
+        result[self.QOS_POLICY] = resource_data.get('qos_policy_id')
+        result.pop(self.MAC_ADDRESS)
+        fixed_ips = resource_data.get(self.FIXED_IPS) or []
+        if fixed_ips:
+            result.update({self.FIXED_IPS: []})
+            for fixed_ip in fixed_ips:
+                result[self.FIXED_IPS].append(
+                    {self.FIXED_IP_SUBNET: fixed_ip.get('subnet_id'),
+                     self.FIXED_IP_IP_ADDRESS: fixed_ip.get('ip_address')})
+        return result
+
     def _resolve_attribute(self, name):
         if self.resource_id is None:
             return
@@ -500,7 +540,7 @@ class Port(neutron.NeutronResource):
                         subnets.append(self.client().show_subnet(
                             subnet_id)['subnet'])
             except Exception as ex:
-                LOG.warning(_LW("Failed to fetch resource attributes: %s"), ex)
+                LOG.warning("Failed to fetch resource attributes: %s", ex)
                 return
             return subnets
         return super(Port, self)._resolve_attribute(name)
@@ -516,8 +556,13 @@ class Port(neutron.NeutronResource):
                 qos_policy = prop_diff.pop(self.QOS_POLICY)
                 prop_diff['qos_policy_id'] = self.client_plugin(
                     ).get_qos_policy_id(qos_policy) if qos_policy else None
+
+            if self.TAGS in prop_diff:
+                tags = prop_diff.pop(self.TAGS)
+                self.set_tags(tags)
+
             self._prepare_port_properties(prop_diff, prepare_for_update=True)
-            LOG.debug('updating port with %s' % prop_diff)
+            LOG.debug('updating port with %s', prop_diff)
             self.client().update_port(self.resource_id, {'port': prop_diff})
 
     def check_update_complete(self, *args):
@@ -529,34 +574,34 @@ class Port(neutron.NeutronResource):
         if self.resource_id is None:
             return
         # store port fixed_ips for restoring after failed update
-        # Ignore if the port does not exist in neutron (deleted)
-        with self.client_plugin().ignore_not_found:
-            fixed_ips = self._show_resource().get('fixed_ips', [])
-            self.data_set('port_fip', jsonutils.dumps(fixed_ips))
-            # reset fixed_ips for this port by setting fixed_ips to []
-            props = {'fixed_ips': []}
-            self.client().update_port(self.resource_id, {'port': props})
+        fixed_ips = self._show_resource().get('fixed_ips', [])
+        self.data_set('port_fip', jsonutils.dumps(fixed_ips))
+        # reset fixed_ips for this port by setting fixed_ips to []
+        props = {'fixed_ips': []}
+        self.client().update_port(self.resource_id, {'port': props})
 
     def restore_prev_rsrc(self, convergence=False):
         # In case of convergence, during rollback, the previous rsrc is
         # already selected and is being acted upon.
-        backup_stack = self.stack._backup_stack()
-        backup_res = backup_stack.resources.get(self.name)
-        prev_port = self if convergence else backup_res
-        fixed_ips = prev_port.data().get('port_fip', [])
-
-        props = {'fixed_ips': []}
         if convergence:
+            prev_port = self
             existing_port, rsrc_owning_stack, stack = resource.Resource.load(
-                prev_port.context, prev_port.replaced_by, True,
-                prev_port.stack.cache_data
+                prev_port.context, prev_port.replaced_by,
+                prev_port.stack.current_traversal, True,
+                prev_port.stack.defn._resource_data
             )
             existing_port_id = existing_port.resource_id
         else:
+            backup_stack = self.stack._backup_stack()
+            prev_port = backup_stack.resources.get(self.name)
             existing_port_id = self.resource_id
+
         if existing_port_id:
             # reset fixed_ips to [] for new resource
+            props = {'fixed_ips': []}
             self.client().update_port(existing_port_id, {'port': props})
+
+        fixed_ips = prev_port.data().get('port_fip', [])
         if fixed_ips and prev_port.resource_id:
             # restore ip for old port
             prev_port_props = {'fixed_ips': jsonutils.loads(fixed_ips)}
