@@ -54,7 +54,7 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         SCHEDULER_HINTS, METADATA, USER_DATA_FORMAT, USER_DATA,
         RESERVATION_ID, CONFIG_DRIVE, DISK_CONFIG, PERSONALITY,
         ADMIN_PASS, SOFTWARE_CONFIG_TRANSPORT, USER_DATA_UPDATE_POLICY,
-        TAGS
+        TAGS, DEPLOYMENT_SWIFT_DATA
     ) = (
         'name', 'image', 'block_device_mapping', 'block_device_mapping_v2',
         'flavor', 'flavor_update_policy', 'image_update_policy', 'key_name',
@@ -62,7 +62,7 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         'scheduler_hints', 'metadata', 'user_data_format', 'user_data',
         'reservation_id', 'config_drive', 'diskConfig', 'personality',
         'admin_pass', 'software_config_transport', 'user_data_update_policy',
-        'tags'
+        'tags', 'deployment_swift_data'
     )
 
     _BLOCK_DEVICE_MAPPING_KEYS = (
@@ -109,10 +109,12 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
 
     _NETWORK_KEYS = (
         NETWORK_UUID, NETWORK_ID, NETWORK_FIXED_IP, NETWORK_PORT,
-        NETWORK_SUBNET, NETWORK_PORT_EXTRA, NETWORK_FLOATING_IP
+        NETWORK_SUBNET, NETWORK_PORT_EXTRA, NETWORK_FLOATING_IP,
+        ALLOCATE_NETWORK, NIC_TAG,
     ) = (
         'uuid', 'network', 'fixed_ip', 'port',
-        'subnet', 'port_extra_properties', 'floating_ip'
+        'subnet', 'port_extra_properties', 'floating_ip',
+        'allocate_network', 'tag',
     )
 
     _SOFTWARE_CONFIG_FORMATS = (
@@ -127,16 +129,31 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         'POLL_SERVER_CFN', 'POLL_SERVER_HEAT', 'POLL_TEMP_URL', 'ZAQAR_MESSAGE'
     )
 
-    ATTRIBUTES = (
-        NAME_ATTR, ADDRESSES, NETWORKS_ATTR, FIRST_ADDRESS,
-        INSTANCE_NAME, ACCESSIPV4, ACCESSIPV6, CONSOLE_URLS, TAGS_ATTR
+    _ALLOCATE_TYPES = (
+        NETWORK_NONE, NETWORK_AUTO,
     ) = (
-        'name', 'addresses', 'networks', 'first_address',
-        'instance_name', 'accessIPv4', 'accessIPv6', 'console_urls', 'tags'
+        'none', 'auto',
     )
 
-    # valid image Status
-    IMAGE_STATUS_ACTIVE = 'active'
+    _DEPLOYMENT_SWIFT_DATA_KEYS = (
+        CONTAINER, OBJECT
+    ) = (
+        'container', 'object',
+    )
+
+    ATTRIBUTES = (
+        NAME_ATTR, ADDRESSES, NETWORKS_ATTR, FIRST_ADDRESS,
+        INSTANCE_NAME, ACCESSIPV4, ACCESSIPV6, CONSOLE_URLS, TAGS_ATTR,
+        OS_COLLECT_CONFIG
+    ) = (
+        'name', 'addresses', 'networks', 'first_address',
+        'instance_name', 'accessIPv4', 'accessIPv6', 'console_urls', 'tags',
+        'os_collect_config'
+    )
+
+    # Image Statuses
+    IMAGE_STATUSES = (IMAGE_ACTIVE, IMAGE_ERROR,
+                      IMAGE_DELETED) = ('active', 'error', 'deleted')
 
     properties_schema = {
         NAME: properties.Schema(
@@ -220,12 +237,16 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
                         properties.Schema.STRING,
                         _('The ID of the image to create a volume from.'),
                         support_status=support.SupportStatus(
-                            status=support.DEPRECATED,
-                            version='7.0.0',
+                            status=support.HIDDEN,
+                            version='9.0.0',
                             message=_('Use property %s.') %
                                     BLOCK_DEVICE_MAPPING_IMAGE,
                             previous_status=support.SupportStatus(
-                                version='5.0.0')
+                                status=support.DEPRECATED,
+                                version='7.0.0',
+                                previous_status=support.SupportStatus(
+                                    version='5.0.0')
+                            )
                         ),
                         constraints=[
                             constraints.CustomConstraint('glance.image')
@@ -404,6 +425,23 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
                             constraints.CustomConstraint('neutron.network')
                         ]
                     ),
+                    ALLOCATE_NETWORK: properties.Schema(
+                        properties.Schema.STRING,
+                        _('The special string values of network, '
+                          'auto: means either a network that is already '
+                          'available to the project will be used, or if one '
+                          'does not exist, will be automatically created for '
+                          'the project; none: means no networking will be '
+                          'allocated for the created server. Supported by '
+                          'Nova API since version "2.37". This property can '
+                          'not be used with other network keys.'),
+                        support_status=support.SupportStatus(version='9.0.0'),
+                        constraints=[
+                            constraints.AllowedValues(
+                                [NETWORK_NONE, NETWORK_AUTO])
+                        ],
+                        update_allowed=True,
+                    ),
                     NETWORK_FIXED_IP: properties.Schema(
                         properties.Schema.STRING,
                         _('Fixed IP address to specify for the port '
@@ -440,6 +478,12 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
                         properties.Schema.STRING,
                         _('ID of the floating IP to associate.'),
                         support_status=support.SupportStatus(version='6.0.0')
+                    ),
+                    NIC_TAG: properties.Schema(
+                        properties.Schema.STRING,
+                        _('Port tag. Heat ignores any update on this property '
+                          'as nova does not support it.'),
+                        support_status=support.SupportStatus(version='9.0.0')
                     )
                 },
             ),
@@ -503,7 +547,9 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         ),
         USER_DATA: properties.Schema(
             properties.Schema.STRING,
-            _('User data script to be executed by cloud-init.'),
+            _('User data script to be executed by cloud-init. Changes cause '
+              'replacement of the resource by default, but can be ignored '
+              'altogether by setting the `user_data_update_policy` property.'),
             default='',
             update_allowed=True
         ),
@@ -540,6 +586,37 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
             support_status=support.SupportStatus(version='8.0.0'),
             schema=properties.Schema(properties.Schema.STRING),
             update_allowed=True
+        ),
+        DEPLOYMENT_SWIFT_DATA: properties.Schema(
+            properties.Schema.MAP,
+            _('Swift container and object to use for storing deployment data '
+              'for the server resource. The parameter is a map value '
+              'with the keys "container" and "object", and the values '
+              'are the corresponding container and object names. The '
+              'software_config_transport parameter must be set to '
+              'POLL_TEMP_URL for swift to be used. If not specified, '
+              'and software_config_transport is set to POLL_TEMP_URL, a '
+              'container will be automatically created from the resource '
+              'name, and the object name will be a generated uuid.'),
+            support_status=support.SupportStatus(version='9.0.0'),
+            default={},
+            update_allowed=True,
+            schema={
+                CONTAINER: properties.Schema(
+                    properties.Schema.STRING,
+                    _('Name of the container.'),
+                    constraints=[
+                        constraints.Length(min=1)
+                    ]
+                ),
+                OBJECT: properties.Schema(
+                    properties.Schema.STRING,
+                    _('Name of the object.'),
+                    constraints=[
+                        constraints.Length(min=1)
+                    ]
+                )
+            }
         )
     }
 
@@ -603,7 +680,7 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
               "can be specified as parameter to the get_attr function, "
               "e.g. get_attr: [ <server>, console_urls, novnc ]. "
               "Currently supported types are "
-              "novnc, xvpvnc, spice-html5, rdp-html5, serial."),
+              "novnc, xvpvnc, spice-html5, rdp-html5, serial and webmks."),
             support_status=support.SupportStatus(version='2015.1'),
             type=attributes.Schema.MAP
         ),
@@ -611,7 +688,15 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
             _('Tags from the server. Supported since client version 2.26.'),
             support_status=support.SupportStatus(version='8.0.0'),
             type=attributes.Schema.LIST
-        )
+        ),
+        OS_COLLECT_CONFIG: attributes.Schema(
+            _('The os-collect-config configuration for the server\'s local '
+              'agent to be configured to connect to Heat to retrieve '
+              'deployment data.'),
+            support_status=support.SupportStatus(version='9.0.0'),
+            type=attributes.Schema.MAP,
+            cache_mode=attributes.Schema.CACHE_NONE
+        ),
     }
 
     default_client_name = 'nova'
@@ -648,38 +733,27 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
                                   self.BLOCK_DEVICE_MAPPING_IMAGE],
                 client_plugin=self.client_plugin('glance'),
                 finder='find_image_by_name_or_id'),
-        ]
-        if self.is_using_neutron():
-            rules.extend([
-                translation.TranslationRule(
-                    props,
-                    translation.TranslationRule.RESOLVE,
-                    translation_path=[self.NETWORKS, self.NETWORK_ID],
-                    client_plugin=self.client_plugin('neutron'),
-                    finder='find_resourceid_by_name_or_id',
-                    entity='network'),
-                translation.TranslationRule(
-                    props,
-                    translation.TranslationRule.RESOLVE,
-                    translation_path=[self.NETWORKS, self.NETWORK_SUBNET],
-                    client_plugin=self.client_plugin('neutron'),
-                    finder='find_resourceid_by_name_or_id',
-                    entity='subnet'),
-                translation.TranslationRule(
-                    props,
-                    translation.TranslationRule.RESOLVE,
-                    translation_path=[self.NETWORKS, self.NETWORK_PORT],
-                    client_plugin=self.client_plugin('neutron'),
-                    finder='find_resourceid_by_name_or_id',
-                    entity='port')])
-        else:
-            rules.extend([
-                translation.TranslationRule(
-                    props,
-                    translation.TranslationRule.RESOLVE,
-                    translation_path=[self.NETWORKS, self.NETWORK_ID],
-                    client_plugin=self.client_plugin('nova'),
-                    finder='get_nova_network_id')])
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                translation_path=[self.NETWORKS, self.NETWORK_ID],
+                client_plugin=self.client_plugin('neutron'),
+                finder='find_resourceid_by_name_or_id',
+                entity='network'),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                translation_path=[self.NETWORKS, self.NETWORK_SUBNET],
+                client_plugin=self.client_plugin('neutron'),
+                finder='find_resourceid_by_name_or_id',
+                entity='subnet'),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                translation_path=[self.NETWORKS, self.NETWORK_PORT],
+                client_plugin=self.client_plugin('neutron'),
+                finder='find_resourceid_by_name_or_id',
+                entity='port')]
         return rules
 
     def __init__(self, name, json_snippet, stack):
@@ -700,13 +774,11 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
             self.USER_DATA_FORMAT] == self.SOFTWARE_CONFIG
 
     def get_software_config(self, ud_content):
-        try:
+        with self.rpc_client().ignore_error_by_name('NotFound'):
             sc = self.rpc_client().show_software_config(
                 self.context, ud_content)
             return sc[rpc_api.SOFTWARE_CONFIG_CONFIG]
-        except Exception as ex:
-            self.rpc_client().ignore_error_named(ex, 'NotFound')
-            return ud_content
+        return ud_content
 
     def handle_create(self):
         security_groups = self.properties[self.SECURITY_GROUPS]
@@ -755,7 +827,19 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
 
         server = None
         try:
-            server = self.client().servers.create(
+            api_version = None
+            # if 'auto' or 'none' is specified, we get the string type
+            # nics after self._build_nics(), and the string network
+            # is supported since nova microversion 2.37
+            if isinstance(nics, six.string_types):
+                api_version = self.client_plugin().V2_37
+
+            if self._is_nic_tagged(self.properties[self.NETWORKS]):
+                api_version = self.client_plugin().V2_42
+
+            nc = self.client(version=api_version)
+
+            server = nc.servers.create(
                 name=self._server_name(),
                 image=image,
                 flavor=flavor,
@@ -787,9 +871,6 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
             if self.properties[self.TAGS]:
                 self._update_server_tags(self.properties[self.TAGS])
             self.store_external_ports()
-            # Addresses binds to server not immediately, so we need to wait
-            # until server is created and after that associate floating ip.
-            self.floating_ips_nova_associate()
         return check
 
     def _update_server_tags(self, tags):
@@ -797,15 +878,6 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         self.client(version=self.client_plugin().V2_26
                     ).servers.set_tags(server, tags)
 
-    def floating_ips_nova_associate(self):
-        # If there is no neutron used, floating_ip still unassociated,
-        # so need associate it with nova.
-        if not self.is_using_neutron():
-            for net in self.properties.get(self.NETWORKS) or []:
-                if net.get(self.NETWORK_FLOATING_IP):
-                    self._floating_ip_nova_associate(
-                        net.get(self.NETWORK_FLOATING_IP))
-
     def handle_check(self):
         server = self.client().servers.get(self.resource_id)
         status = self.client_plugin().get_status(server)
@@ -857,12 +929,17 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         reality_nets = self._add_port_for_address(server,
                                                   extend_networks=False)
         reality_net_ids = {}
+        client_plugin = self.client_plugin('neutron')
         for net_key in reality_nets:
             try:
-                net_id = self.client_plugin().get_net_id_by_label(net_key)
-            except (exception.EntityNotFound,
-                    exception.PhysicalResourceNameAmbiguity):
-                net_id = None
+                net_id = client_plugin.find_resourceid_by_name_or_id('network',
+                                                                     net_key)
+            except Exception as ex:
+                if (client_plugin.is_not_found(ex) or
+                        client_plugin.is_no_unique(ex)):
+                    net_id = None
+                else:
+                    raise
             if net_id:
                 reality_net_ids[net_id] = reality_nets.get(net_key)
 
@@ -1023,12 +1100,17 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         This method is used only for resolving attributes.
         """
         nets = copy.deepcopy(networks)
+        client_plugin = self.client_plugin('neutron')
         for key in list(nets.keys()):
             try:
-                net_id = self.client_plugin().get_net_id_by_label(key)
-            except (exception.EntityNotFound,
-                    exception.PhysicalResourceNameAmbiguity):
-                net_id = None
+                net_id = client_plugin.find_resourceid_by_name_or_id('network',
+                                                                     key)
+            except Exception as ex:
+                if (client_plugin.is_not_found(ex) or
+                        client_plugin.is_no_unique(ex)):
+                    net_id = None
+                else:
+                    raise
             if net_id:
                 nets[net_id] = nets[key]
         return nets
@@ -1039,6 +1121,8 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         if name == self.FIRST_ADDRESS:
             return self.client_plugin().server_to_ipaddress(
                 self.resource_id) or ''
+        if name == self.OS_COLLECT_CONFIG:
+            return self.metadata_get().get('os-collect-config', {})
         if name == self.NAME_ATTR:
             return self._server_name()
         try:
@@ -1150,16 +1234,16 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
             updaters.append(
                 progress.ServerUpdateProgress(
                     self.resource_id, 'interface_detach',
-                    complete=True,
-                    handler_extra={'args': (port,)})
+                    handler_extra={'args': (port,)},
+                    checker_extra={'args': (port,)})
             )
 
         for args in add_nets:
             updaters.append(
                 progress.ServerUpdateProgress(
                     self.resource_id, 'interface_attach',
-                    complete=True,
-                    handler_extra={'kwargs': args})
+                    handler_extra={'kwargs': args},
+                    checker_extra={'args': (args['port_id'],)})
             )
 
         return updaters
@@ -1342,10 +1426,10 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
                 return
             raise
         else:
-            if image_obj.status.lower() != self.IMAGE_STATUS_ACTIVE:
+            if image_obj.status.lower() != self.IMAGE_ACTIVE:
                 msg = _('Image status is required to be %(cstatus)s not '
                         '%(wstatus)s.') % {
-                    'cstatus': self.IMAGE_STATUS_ACTIVE,
+                    'cstatus': self.IMAGE_ACTIVE,
                     'wstatus': image_obj.status}
                 raise exception.StackValidationFailed(message=msg)
 
@@ -1384,16 +1468,51 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         if image:
             self._validate_image_flavor(image, flavor)
 
-        # network properties 'uuid' and 'network' shouldn't be used
-        # both at once for all networks
         networks = self.properties[self.NETWORKS] or []
-        # record if any networks include explicit ports
-        networks_with_port = False
         for network in networks:
-            networks_with_port = (networks_with_port or
-                                  network.get(self.NETWORK_PORT) is not None)
             self._validate_network(network)
 
+        has_str_net = self._str_network(networks) is not None
+        if has_str_net:
+            if len(networks) != 1:
+                msg = _('Property "%s" can not be specified if '
+                        'multiple network interfaces set for '
+                        'server.') % self.ALLOCATE_NETWORK
+                raise exception.StackValidationFailed(message=msg)
+            # Check if str_network is allowed to use
+            try:
+                self.client(
+                    version=self.client_plugin().V2_37)
+            except exception.InvalidServiceVersion as ex:
+                msg = (_('Cannot use "%(prop)s" property - compute service '
+                         'does not support the required api '
+                         'microversion: %(ex)s')
+                       % {'prop': self.ALLOCATE_NETWORK,
+                          'ex': six.text_type(ex)})
+                raise exception.StackValidationFailed(message=msg)
+
+        # record if any networks include explicit ports
+        has_port = any(n[self.NETWORK_PORT] is not None for n in networks)
+        # if 'security_groups' present for the server and explicit 'port'
+        # in one or more entries in 'networks', raise validation error
+        if has_port and self.properties[self.SECURITY_GROUPS]:
+            raise exception.ResourcePropertyConflict(
+                self.SECURITY_GROUPS,
+                "/".join([self.NETWORKS, self.NETWORK_PORT]))
+
+        # Check if nic tag is allowed to use
+        if self._is_nic_tagged(networks=networks):
+            try:
+                self.client(
+                    version=self.client_plugin().V2_42)
+            except exception.InvalidServiceVersion as ex:
+                msg = (_('Cannot use "%(tag)s" property in networks - '
+                         'nova does not support it: %(error)s')) % {
+                    'tag': self.NIC_TAG,
+                    'error': six.text_type(ex)
+                }
+                raise exception.StackValidationFailed(message=msg)
+
         # Check if tags is allowed to use
         if self.properties[self.TAGS]:
             try:
@@ -1410,13 +1529,6 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         if metadata or personality:
             limits = self.client_plugin().absolute_limits()
 
-        # if 'security_groups' present for the server and explicit 'port'
-        # in one or more entries in 'networks', raise validation error
-        if networks_with_port and self.properties[self.SECURITY_GROUPS]:
-            raise exception.ResourcePropertyConflict(
-                self.SECURITY_GROUPS,
-                "/".join([self.NETWORKS, self.NETWORK_PORT]))
-
         # verify that the number of metadata entries is not greater
         # than the maximum number allowed in the provider's absolute
         # limits
@@ -1458,8 +1570,6 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         if self.resource_id is None:
             return
 
-        self._floating_ips_disassociate()
-
         try:
             self.client().servers.delete(self.resource_id)
         except Exception as e:
@@ -1483,12 +1593,12 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
     def check_delete_complete(self, prg):
         if not prg:
             return True
-
         if not prg.image_complete:
-            image = self.client().images.get(prg.image_id)
-            if image.status in ('DELETED', 'ERROR'):
+            image = self.client_plugin('glance').get_image(prg.image_id)
+            if image.status.lower() in (self.IMAGE_ERROR,
+                                        self.IMAGE_DELETED):
                 raise exception.Error(image.status)
-            elif image.status == 'ACTIVE':
+            elif image.status.lower() == self.IMAGE_ACTIVE:
                 prg.image_complete = True
                 if not self._delete():
                     return True
@@ -1520,7 +1630,7 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
             # if the server has been suspended successful,
             # no need to suspend again
             if self.client_plugin().get_status(server) != 'SUSPENDED':
-                LOG.debug('suspending server %s' % self.resource_id)
+                LOG.debug('suspending server %s', self.resource_id)
                 server.suspend()
             return server.id
 
@@ -1530,8 +1640,8 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         if not server:
             return False
         status = cp.get_status(server)
-        LOG.debug('%(name)s check_suspend_complete status = %(status)s'
-                  % {'name': self.name, 'status': status})
+        LOG.debug('%(name)s check_suspend_complete status = %(status)s',
+                  {'name': self.name, 'status': status})
         if status in list(cp.deferred_server_statuses + ['ACTIVE']):
             return status == 'SUSPENDED'
         else:
@@ -1563,7 +1673,7 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
             # if the server has been resumed successful,
             # no need to resume again
             if self.client_plugin().get_status(server) != 'ACTIVE':
-                LOG.debug('resuming server %s' % self.resource_id)
+                LOG.debug('resuming server %s', self.resource_id)
                 server.resume()
             return server.id
 
@@ -1577,10 +1687,10 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
         return image_id
 
     def check_snapshot_complete(self, image_id):
-        image = self.client().images.get(image_id)
-        if image.status == 'ACTIVE':
+        image = self.client_plugin('glance').get_image(image_id)
+        if image.status.lower() == self.IMAGE_ACTIVE:
             return True
-        elif image.status == 'ERROR' or image.status == 'DELETED':
+        elif image.status.lower() in (self.IMAGE_ERROR, self.IMAGE_DELETED):
             raise exception.Error(image.status)
 
         return False
@@ -1588,12 +1698,18 @@ class Server(server_base.BaseServer, sh.SchedulerHintsMixin,
     def handle_delete_snapshot(self, snapshot):
         image_id = snapshot['resource_data'].get('snapshot_image_id')
         with self.client_plugin().ignore_not_found:
-            self.client().images.delete(image_id)
+            self.client('glance').images.delete(image_id)
 
     def handle_restore(self, defn, restore_data):
         image_id = restore_data['resource_data']['snapshot_image_id']
         props = dict((k, v) for k, v in self.properties.data.items()
                      if v is not None)
+        for key in [self.BLOCK_DEVICE_MAPPING, self.BLOCK_DEVICE_MAPPING_V2,
+                    self.NETWORKS]:
+            if props.get(key) is not None:
+                props[key] = list(dict((k, v) for k, v in prop.items()
+                                       if v is not None)
+                                  for prop in props[key])
         props[self.IMAGE] = image_id
         return defn.freeze(properties=props)
 
