@@ -12,7 +12,6 @@
 #    under the License.
 
 import json
-import warnings
 import weakref
 
 from oslo_config import cfg
@@ -23,8 +22,6 @@ import six
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LI
-from heat.common.i18n import _LW
 from heat.common import identifier
 from heat.common import template_format
 from heat.engine import attributes
@@ -79,10 +76,20 @@ class StackResource(resource.Resource):
         except AssertionError:
             raise
         except Exception as ex:
+            path = "%s<%s>" % (self.name, self.template_url)
             raise exception.StackValidationFailed(
-                error=_("Failed to validate"),
-                path=[self.stack.t.get_section_name('resources'), self.name],
-                message=six.text_type(ex))
+                ex, path=[self.stack.t.RESOURCES, path])
+
+    @property
+    def template_url(self):
+        """Template url for the stack resource.
+
+        When stack resource is a TemplateResource, it's the template
+        location. For group resources like ResourceGroup where the
+        template is constructed dynamically, it's just a placeholder.
+        """
+
+        return "nested_stack"
 
     def _outputs_to_attribs(self, json_snippet):
         outputs = json_snippet.get('Outputs')
@@ -183,7 +190,7 @@ class StackResource(resource.Resource):
             params = self.child_params()
         except NotImplementedError:
             class_name = reflection.get_class_name(self, fully_qualified=False)
-            LOG.warning(_LW("Preview of '%s' not yet implemented"), class_name)
+            LOG.warning("Preview of '%s' not yet implemented", class_name)
             return self
 
         name = "%s-%s" % (self.stack.name, self.name)
@@ -338,12 +345,6 @@ class StackResource(resource.Resource):
                 'files': parsed_template.files,
             }
 
-    def raise_local_exception(self, ex):
-        warnings.warn('raise_local_exception() is deprecated. Use the '
-                      'translate_remote_exceptions context manager instead.',
-                      DeprecationWarning)
-        return self.translate_remote_exceptions(ex)
-
     @excutils.exception_filter
     def translate_remote_exceptions(self, ex):
         if (isinstance(ex, exception.ActionInProgress) and
@@ -460,7 +461,7 @@ class StackResource(resource.Resource):
 
         if self.stack.action == self.stack.ROLLBACK:
             if self._try_rollback():
-                LOG.info(_LI('Triggered nested stack %s rollback'),
+                LOG.info('Triggered nested stack %s rollback',
                          self.physical_resource_name())
                 return {'target_action': self.stack.ROLLBACK}
 
@@ -496,7 +497,8 @@ class StackResource(resource.Resource):
 
         kwargs.update({
             'stack_identity': dict(self.nested_identifier()),
-            'args': {rpc_api.PARAM_TIMEOUT: timeout_mins}
+            'args': {rpc_api.PARAM_TIMEOUT: timeout_mins,
+                     rpc_api.PARAM_CONVERGE: self.converge}
         })
         with self.translate_remote_exceptions:
             try:
@@ -537,14 +539,12 @@ class StackResource(resource.Resource):
         if stack_identity is None:
             return
 
-        try:
+        with self.rpc_client().ignore_error_by_name('EntityNotFound'):
             if self.abandon_in_progress:
                 self.rpc_client().abandon_stack(self.context, stack_identity)
             else:
                 self.rpc_client().delete_stack(self.context, stack_identity,
                                                cast=False)
-        except Exception as ex:
-            self.rpc_client().ignore_error_named(ex, 'EntityNotFound')
 
     def handle_delete(self):
         return self.delete_nested()
