@@ -23,7 +23,6 @@ from six.moves.urllib import parse as urlparse
 from heat.common import crypt
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LI
 from heat.db.sqlalchemy import api as db_api
 from heat.engine import api
 from heat.engine import resource
@@ -101,9 +100,11 @@ class SoftwareConfigService(object):
     def _push_metadata_software_deployments(
             self, cnxt, server_id, stack_user_project_id):
         rs = db_api.resource_get_by_physical_resource_id(cnxt, server_id)
-        if not rs or rs.action == resource.Resource.DELETE:
+        if not rs:
             return
         cnxt.session.refresh(rs)
+        if rs.action == resource.Resource.DELETE:
+            return
         deployments = self.metadata_software_deployments(cnxt, server_id)
         md = rs.rsrc_metadata or {}
         md['deployments'] = deployments
@@ -140,7 +141,7 @@ class SoftwareConfigService(object):
         except Exception as ex:
             # ignore not-found, in case swift is not consistent yet
             if swift_plugin.is_not_found(ex):
-                LOG.info(_LI('Signal object not found: %(c)s %(o)s'), {
+                LOG.info('Signal object not found: %(c)s %(o)s', {
                     'c': container, 'o': object_name})
                 return sd
             raise
@@ -162,8 +163,8 @@ class SoftwareConfigService(object):
         except Exception as ex:
             # ignore not-found, in case swift is not consistent yet
             if swift_plugin.is_not_found(ex):
-                LOG.info(_LI(
-                    'Signal object not found: %(c)s %(o)s'), {
+                LOG.info(
+                    'Signal object not found: %(c)s %(o)s', {
                         'c': container, 'o': object_name})
                 return sd
             raise
