@@ -13,10 +13,13 @@
 # limitations under the License.
 
 import ast
-from oslo_log import log as logging
+import eventlet
+import random
 import six
 
-from heat.common.i18n import _
+from oslo_log import log as logging
+
+from heat.common import exception
 from heat.objects import sync_point as sync_point_object
 
 LOG = logging.getLogger(__name__)
@@ -49,7 +52,7 @@ def get(context, entity_id, traversal_id, is_update):
                                                         is_update)
     if sync_point is None:
         key = (entity_id, traversal_id, is_update)
-        raise SyncPointNotFound(key)
+        raise exception.EntityNotFound(entity='Sync Point', name=key)
 
     return sync_point
 
@@ -70,7 +73,7 @@ def update_input_data(context, entity_id, current_traversal,
     return rows_updated
 
 
-def _str_pack_tuple(t):
+def str_pack_tuple(t):
     return u'tuple:' + str(t)
 
 
@@ -94,7 +97,7 @@ def _serialize(d):
     d2 = {}
     for k, v in d.items():
         if isinstance(k, tuple):
-            k = _str_pack_tuple(k)
+            k = str_pack_tuple(k)
         if isinstance(v, dict):
             v = _serialize(v)
         d2[k] = v
@@ -118,14 +121,19 @@ def sync(cnxt, entity_id, current_traversal, is_update, propagate,
     rows_updated = None
     sync_point = None
     input_data = None
+    nconflicts = max(0, len(predecessors) - 2)
+    # limit to 10 seconds
+    max_wt = min(nconflicts * 0.01, 10)
     while not rows_updated:
-        # TODO(sirushtim): Add a conf option to add no. of retries
         sync_point = get(cnxt, entity_id, current_traversal, is_update)
         input_data = deserialize_input_data(sync_point.input_data)
         input_data.update(new_data)
         rows_updated = update_input_data(
             cnxt, entity_id, current_traversal, is_update,
             sync_point.atomic_key, serialize_input_data(input_data))
+        # don't aggressively spin; induce some sleep
+        if not rows_updated:
+            eventlet.sleep(random.uniform(0, max_wt))
 
     waiting = predecessors - set(input_data)
     key = make_key(entity_id, current_traversal, is_update)
@@ -136,10 +144,3 @@ def sync(cnxt, entity_id, current_traversal, is_update, propagate,
         LOG.debug('[%s] Ready %s: Got %s',
                   key, entity_id, _dump_list(input_data))
         propagate(entity_id, serialize_input_data(input_data))
-
-
-class SyncPointNotFound(Exception):
-    """Raised when resource update requires replacement."""
-    def __init__(self, sync_point):
-        msg = _("Sync Point %s not found") % (sync_point, )
-        super(Exception, self).__init__(six.text_type(msg))
