@@ -19,8 +19,6 @@ from oslo_utils import timeutils
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LI
-from heat.common.i18n import _LW
 from heat.engine import stack
 from heat.engine import timestamp
 from heat.objects import stack as stack_object
@@ -85,9 +83,8 @@ class WatchRule(object):
                 watch = watch_rule_objects.WatchRule.get_by_name(context,
                                                                  watch_name)
             except Exception as ex:
-                LOG.warning(_LW('WatchRule.load (%(watch_name)s) db error '
-                                '%(ex)s'), {'watch_name': watch_name,
-                                            'ex': ex})
+                LOG.warning('WatchRule.load (%(watch_name)s) db error %(ex)s',
+                            {'watch_name': watch_name, 'ex': ex})
         if watch is None:
             raise exception.EntityNotFound(entity='Watch Rule',
                                            name=watch_name)
@@ -114,7 +111,7 @@ class WatchRule(object):
             'stack_id': self.stack_id
         }
 
-        if not self.id:
+        if self.id is None:
             wr = watch_rule_objects.WatchRule.create(self.context, wr_values)
             self.id = wr.id
         else:
@@ -123,7 +120,7 @@ class WatchRule(object):
 
     def destroy(self):
         """Delete the watchrule from the database."""
-        if self.id:
+        if self.id is not None:
             watch_rule_objects.WatchRule.delete(self.context, self.id)
 
     def do_data_cmp(self, data, threshold):
@@ -218,7 +215,7 @@ class WatchRule(object):
         data = 0
         for d in self.watch_data:
             if d.created_at < self.now - self.timeperiod:
-                LOG.debug('ignoring %s' % str(d.data))
+                LOG.debug('ignoring %s', str(d.data))
                 continue
             data = data + float(d.data[self.rule['MetricName']]['Value'])
 
@@ -255,25 +252,24 @@ class WatchRule(object):
         return actions
 
     def rule_actions(self, new_state):
-        LOG.info(_LI('WATCH: stack:%(stack)s, watch_name:%(watch_name)s, '
-                     'new_state:%(new_state)s'), {'stack': self.stack_id,
-                                                  'watch_name': self.name,
-                                                  'new_state': new_state})
+        LOG.info('WATCH: stack:%(stack)s, watch_name:%(watch_name)s, '
+                 'new_state:%(new_state)s', {'stack': self.stack_id,
+                                             'watch_name': self.name,
+                                             'new_state': new_state})
         actions = []
         if self.ACTION_MAP[new_state] not in self.rule:
-            LOG.info(_LI('no action for new state %s'), new_state)
+            LOG.info('no action for new state %s', new_state)
         else:
             s = stack_object.Stack.get_by_id(
                 self.context,
-                self.stack_id,
-                eager_load=True)
+                self.stack_id)
             stk = stack.Stack.load(self.context, stack=s)
             if (stk.action != stk.DELETE
                     and stk.status == stk.COMPLETE):
                 for refid in self.rule[self.ACTION_MAP[new_state]]:
                     actions.append(stk.resource_by_refid(refid).signal)
             else:
-                LOG.warning(_LW("Could not process watch state %s for stack"),
+                LOG.warning("Could not process watch state %s for stack",
                             new_state)
         return actions
 
@@ -293,7 +289,7 @@ class WatchRule(object):
                 dims = dims[0]
             sample['resource_metadata'] = dims
             sample['resource_id'] = dims.get('InstanceId')
-            LOG.debug('new sample:%(k)s data:%(sample)s' % {
+            LOG.debug('new sample:%(k)s data:%(sample)s', {
                       'k': k, 'sample': sample})
             clients.client('ceilometer').samples.create(**sample)
 
@@ -306,8 +302,8 @@ class WatchRule(object):
             return
 
         if self.state == self.SUSPENDED:
-            LOG.debug('Ignoring metric data for %s, SUSPENDED state'
-                      % self.name)
+            LOG.debug('Ignoring metric data for %s, SUSPENDED state',
+                      self.name)
             return []
 
         if self.rule['MetricName'] not in data:
@@ -356,9 +352,9 @@ class WatchRule(object):
                           % {'self_state': self.state, 'name': self.name,
                              'state': state})
             else:
-                LOG.warning(_LW("Unable to override state %(state)s for "
-                                "watch %(name)s"), {'state': self.state,
-                                                    'name': self.name})
+                LOG.warning("Unable to override state %(state)s for "
+                            "watch %(name)s", {'state': self.state,
+                                               'name': self.name})
         return actions
 
 
