@@ -23,12 +23,10 @@ from oslo_utils import uuidutils
 from osprofiler import profiler
 
 from heat.common import context
-from heat.common.i18n import _LE
-from heat.common.i18n import _LI
-from heat.common.i18n import _LW
 from heat.common import messaging as rpc_messaging
 from heat.db.sqlalchemy import api as db_api
 from heat.engine import check_resource
+from heat.engine import node_data
 from heat.engine import stack as parser
 from heat.engine import sync_point
 from heat.objects import stack as stack_objects
@@ -63,7 +61,7 @@ class WorkerService(object):
     or expect replies from these messages.
     """
 
-    RPC_API_VERSION = '1.3'
+    RPC_API_VERSION = '1.4'
 
     def __init__(self,
                  host,
@@ -85,7 +83,7 @@ class WorkerService(object):
             server=self.engine_id,
             topic=self.topic)
         self.target = target
-        LOG.info(_LI("Starting %(topic)s (%(version)s) in engine %(engine)s."),
+        LOG.info("Starting %(topic)s (%(version)s) in engine %(engine)s.",
                  {'topic': self.topic,
                   'version': self.RPC_API_VERSION,
                   'engine': self.engine_id})
@@ -97,13 +95,13 @@ class WorkerService(object):
         if self._rpc_server is None:
             return
         # Stop rpc connection at first for preventing new requests
-        LOG.info(_LI("Stopping %(topic)s in engine %(engine)s."),
+        LOG.info("Stopping %(topic)s in engine %(engine)s.",
                  {'topic': self.topic, 'engine': self.engine_id})
         try:
             self._rpc_server.stop()
             self._rpc_server.wait()
         except Exception as e:
-            LOG.error(_LE("%(topic)s is failed to stop, %(exc)s"),
+            LOG.error("%(topic)s is failed to stop, %(exc)s",
                       {'topic': self.topic, 'exc': e})
 
     def stop_traversal(self, stack):
@@ -135,18 +133,17 @@ class WorkerService(object):
         cancelled = _cancel_workers(stack, self.thread_group_mgr,
                                     self.engine_id, self._rpc_client)
         if not cancelled:
-            LOG.error(_LE("Failed to stop all workers of stack %(name)s "
-                          ", stack cancel not complete"),
-                      {'name': stack.name})
+            LOG.error("Failed to stop all workers of stack %s, "
+                      "stack cancel not complete", stack.name)
             return False
 
-        LOG.info(_LI('[%(name)s(%(id)s)] Stopped all active workers for stack '
-                     '%(action)s'),
+        LOG.info('[%(name)s(%(id)s)] Stopped all active workers for stack '
+                 '%(action)s',
                  {'name': stack.name, 'id': stack.id, 'action': stack.action})
 
         return True
 
-    def _retrigger_replaced(self, is_update, rsrc, stack, msg_queue):
+    def _retrigger_replaced(self, is_update, rsrc, stack, check_resource):
         graph = stack.convergence_dependencies.graph()
         key = (rsrc.id, is_update)
         if key not in graph and rsrc.replaces is not None:
@@ -158,39 +155,41 @@ class WorkerService(object):
             # The old resource might be in the graph (a rollback case);
             # just re-trigger it.
             key = (rsrc.replaces, is_update)
-            cr = check_resource.CheckResource(self.engine_id, self._rpc_client,
-                                              self.thread_group_mgr, msg_queue)
-            cr.retrigger_check_resource(stack.context, is_update, key[0],
-                                        stack)
+            check_resource.retrigger_check_resource(stack.context, is_update,
+                                                    key[0], stack)
 
     @context.request_context
     @log_exceptions
     def check_resource(self, cnxt, resource_id, current_traversal, data,
-                       is_update, adopt_stack_data):
+                       is_update, adopt_stack_data, converge=False):
         """Process a node in the dependency graph.
 
         The node may be associated with either an update or a cleanup of its
         associated resource.
         """
-        resource_data = dict(sync_point.deserialize_input_data(data))
+        in_data = sync_point.deserialize_input_data(data)
+        resource_data = node_data.load_resources_data(in_data if is_update
+                                                      else {})
         rsrc, rsrc_owning_stack, stack = check_resource.load_resource(
-            cnxt, resource_id, resource_data, is_update)
+            cnxt, resource_id, resource_data, current_traversal, is_update)
 
         if rsrc is None:
             return
 
+        rsrc.converge = converge
+
         msg_queue = eventlet.queue.LightQueue()
         try:
             self.thread_group_mgr.add_msg_queue(stack.id, msg_queue)
+            cr = check_resource.CheckResource(self.engine_id,
+                                              self._rpc_client,
+                                              self.thread_group_mgr,
+                                              msg_queue, in_data)
             if current_traversal != stack.current_traversal:
                 LOG.debug('[%s] Traversal cancelled; re-trigerring.',
                           current_traversal)
-                self._retrigger_replaced(is_update, rsrc, stack, msg_queue)
+                self._retrigger_replaced(is_update, rsrc, stack, cr)
             else:
-                cr = check_resource.CheckResource(self.engine_id,
-                                                  self._rpc_client,
-                                                  self.thread_group_mgr,
-                                                  msg_queue)
                 cr.check(cnxt, resource_id, current_traversal, resource_data,
                          is_update, adopt_stack_data, rsrc, stack)
         finally:
@@ -212,18 +211,17 @@ def _stop_traversal(stack):
     old_trvsl = stack.current_traversal
     updated = _update_current_traversal(stack)
     if not updated:
-        LOG.warning(_LW("Failed to update stack %(name)s with new "
-                        "traversal, aborting stack cancel"),
-                    {'name': stack.name})
+        LOG.warning("Failed to update stack %(name)s with new "
+                    "traversal, aborting stack cancel", stack.name)
         return
 
     reason = 'Stack %(action)s cancelled' % {'action': stack.action}
     updated = stack.state_set(stack.action, stack.FAILED, reason)
     if not updated:
-        LOG.warning(_LW("Failed to update stack %(name)s status "
-                        "to %(action)s_%(state)s"),
+        LOG.warning("Failed to update stack %(name)s status "
+                    "to %(action)s_%(state)s",
                     {'name': stack.name, 'action': stack.action,
-                    'state': stack.FAILED})
+                     'state': stack.FAILED})
         return
 
     sync_point.delete_all(stack.context, stack.id, old_trvsl)
