@@ -23,7 +23,6 @@ import oslo_i18n as i18n
 from oslo_log import log as logging
 
 from heat.common import config
-from heat.common.i18n import _LI
 from heat.common import messaging
 from heat.common import profiler
 from heat import version
@@ -45,7 +44,7 @@ def init_application():
 
     port = cfg.CONF.heat_api_cloudwatch.bind_port
     host = cfg.CONF.heat_api_cloudwatch.bind_host
-    LOG.info(_LI('Starting Heat CloudWatch API on %(host)s:%(port)s'),
+    LOG.info('Starting Heat CloudWatch API on %(host)s:%(port)s',
              {'host': host, 'port': port})
     profiler.setup('heat-api-cloudwatch', host)
 
