@@ -14,6 +14,7 @@
 
 """Event object."""
 
+from oslo_log import log as logging
 from oslo_versionedobjects import base
 from oslo_versionedobjects import fields
 
@@ -22,6 +23,8 @@ from heat.db.sqlalchemy import api as db_api
 from heat.objects import base as heat_base
 from heat.objects import resource_properties_data as rpd
 
+LOG = logging.getLogger(__name__)
+
 
 class Event(
         heat_base.HeatObject,
@@ -37,42 +40,51 @@ class Event(
         'physical_resource_id': fields.StringField(nullable=True),
         'resource_status_reason': fields.StringField(nullable=True),
         'resource_type': fields.StringField(nullable=True),
-        'rsrc_prop_data': fields.ObjectField(
-            rpd.ResourcePropertiesData),
+        'rsrc_prop_data_id': fields.ObjectField(
+            fields.IntegerField(nullable=True)),
         'created_at': fields.DateTimeField(read_only=True),
         'updated_at': fields.DateTimeField(nullable=True),
     }
 
     @staticmethod
     def _from_db_object(context, event, db_event):
+        event._resource_properties = None
         for field in event.fields:
+            if field == 'resource_status_reason':
+                # this works whether db_event is a dict or db ref
+                event[field] = db_event['_resource_status_reason']
+            else:
                 event[field] = db_event[field]
-        if db_event['rsrc_prop_data']:
-            event['rsrc_prop_data'] = \
-                rpd.ResourcePropertiesData._from_db_object(
-                    rpd.ResourcePropertiesData(context), context,
-                    db_event['rsrc_prop_data'])
-            event._resource_properties = event['rsrc_prop_data'].data
-        else:
+        if db_event['rsrc_prop_data_id'] is None:
             event._resource_properties = db_event['resource_properties'] or {}
+        else:
+            if hasattr(db_event, '__dict__'):
+                rpd_obj = db_event.__dict__.get('rsrc_prop_data')
+            elif hasattr(db_event, 'rsrc_prop_data'):
+                rpd_obj = db_event['rsrc_prop_data']
+            else:
+                rpd_obj = None
+            if rpd_obj is not None:
+                # Object is already eager loaded
+                rpd_obj = (
+                    rpd.ResourcePropertiesData._from_db_object(
+                        rpd.ResourcePropertiesData(),
+                        context,
+                        rpd_obj))
+                event._resource_properties = rpd_obj.data
         event._context = context
         event.obj_reset_changes()
         return event
 
     @property
     def resource_properties(self):
+        if self._resource_properties is None:
+            LOG.info('rsrp_prop_data lazy load')
+            rpd_obj = rpd.ResourcePropertiesData.get_by_id(
+                self._context, self.rsrc_prop_data_id)
+            self._resource_properties = rpd_obj.data or {}
         return self._resource_properties
 
-    @classmethod
-    def get_by_id(cls, context, event_id):
-        db_event = db_api.event_get(context, event_id)
-        return cls._from_db_object(context, cls(context), db_event)
-
-    @classmethod
-    def get_all(cls, context):
-        return [cls._from_db_object(context, cls(), db_event)
-                for db_event in db_api.event_get_all(context)]
-
     @classmethod
     def get_all_by_tenant(cls, context, **kwargs):
         return [cls._from_db_object(context, cls(), db_event)
@@ -92,8 +104,11 @@ class Event(
 
     @classmethod
     def create(cls, context, values):
-        return cls._from_db_object(context, cls(),
-                                   db_api.event_create(context, values))
+        # Using dict() allows us to be done with the sqlalchemy/model
+        # layer in one call, rather than hitting that layer for every
+        # field in _from_db_object().
+        return cls._from_db_object(context, cls(context=context),
+                                   dict(db_api.event_create(context, values)))
 
     def identifier(self, stack_identifier):
         """Return a unique identifier for the event."""
