@@ -20,14 +20,14 @@ PARAM_KEYS = (
     PARAM_CLEAR_PARAMETERS, PARAM_GLOBAL_TENANT, PARAM_LIMIT,
     PARAM_NESTED_DEPTH, PARAM_TAGS, PARAM_SHOW_HIDDEN, PARAM_TAGS_ANY,
     PARAM_NOT_TAGS, PARAM_NOT_TAGS_ANY, TEMPLATE_TYPE, PARAM_WITH_DETAIL,
-    RESOLVE_OUTPUTS, PARAM_IGNORE_ERRORS
+    RESOLVE_OUTPUTS, PARAM_IGNORE_ERRORS, PARAM_CONVERGE
 ) = (
     'timeout_mins', 'disable_rollback', 'adopt_stack_data',
     'show_deleted', 'show_nested', 'existing',
     'clear_parameters', 'global_tenant', 'limit',
     'nested_depth', 'tags', 'show_hidden', 'tags_any',
     'not_tags', 'not_tags_any', 'template_type', 'with_detail',
-    'resolve_outputs', 'ignore_errors'
+    'resolve_outputs', 'ignore_errors', 'converge'
 )
 
 STACK_KEYS = (
@@ -191,12 +191,13 @@ VALIDATE_PARAM_KEYS = (
     PARAM_TYPE, PARAM_DEFAULT, PARAM_NO_ECHO,
     PARAM_ALLOWED_VALUES, PARAM_ALLOWED_PATTERN, PARAM_MAX_LENGTH,
     PARAM_MIN_LENGTH, PARAM_MAX_VALUE, PARAM_MIN_VALUE,
+    PARAM_STEP, PARAM_OFFSET,
     PARAM_DESCRIPTION, PARAM_CONSTRAINT_DESCRIPTION, PARAM_LABEL,
     PARAM_CUSTOM_CONSTRAINT, PARAM_VALUE
 ) = (
     'Type', 'Default', 'NoEcho',
     'AllowedValues', 'AllowedPattern', 'MaxLength',
-    'MinLength', 'MaxValue', 'MinValue',
+    'MinLength', 'MaxValue', 'MinValue', 'Step', 'Offset',
     'Description', 'ConstraintDescription', 'Label',
     'CustomConstraint', 'Value'
 )
@@ -217,7 +218,8 @@ SOFTWARE_CONFIG_KEYS = (
     SOFTWARE_CONFIG_INPUTS,
     SOFTWARE_CONFIG_OUTPUTS,
     SOFTWARE_CONFIG_OPTIONS,
-    SOFTWARE_CONFIG_CREATION_TIME
+    SOFTWARE_CONFIG_CREATION_TIME,
+    SOFTWARE_CONFIG_PROJECT
 ) = (
     'id',
     'name',
@@ -226,7 +228,8 @@ SOFTWARE_CONFIG_KEYS = (
     'inputs',
     'outputs',
     'options',
-    'creation_time'
+    'creation_time',
+    'project'
 )
 
 SOFTWARE_DEPLOYMENT_KEYS = (
