@@ -62,9 +62,6 @@ class CfnStackControllerTest(common.HeatTestCase):
         self.assertRaises(
             exception.HeatInvalidActionError, self.controller.default, None)
 
-    def tearDown(self):
-        super(CfnStackControllerTest, self).tearDown()
-
     def _dummy_GET_request(self, params=None):
         # Mangle the params dict into a query string
         params = params or {}
@@ -163,13 +160,13 @@ class CfnStackControllerTest(common.HeatTestCase):
                        u'StackStatus': u'CREATE_COMPLETE'}]}}}
         self.assertEqual(expected, result)
         default_args = {'limit': None, 'sort_keys': None, 'marker': None,
-                        'sort_dir': None, 'filters': None, 'tenant_safe': True,
+                        'sort_dir': None, 'filters': None,
                         'show_deleted': False, 'show_nested': False,
                         'show_hidden': False, 'tags': None,
                         'tags_any': None, 'not_tags': None,
                         'not_tags_any': None}
         mock_call.assert_called_once_with(
-            dummy_req.context, ('list_stacks', default_args), version='1.8')
+            dummy_req.context, ('list_stacks', default_args), version='1.33')
 
     @mock.patch.object(rpc_client.EngineClient, 'call')
     def test_list_rmt_aterr(self, mock_call):
@@ -185,7 +182,7 @@ class CfnStackControllerTest(common.HeatTestCase):
         result = self.controller.list(dummy_req)
         self.assertIsInstance(result, exception.HeatInvalidParameterValueError)
         mock_call.assert_called_once_with(
-            dummy_req.context, ('list_stacks', mock.ANY), version='1.8')
+            dummy_req.context, ('list_stacks', mock.ANY), version='1.33')
 
     @mock.patch.object(rpc_client.EngineClient, 'call')
     def test_list_rmt_interr(self, mock_call):
@@ -201,7 +198,7 @@ class CfnStackControllerTest(common.HeatTestCase):
         result = self.controller.list(dummy_req)
         self.assertIsInstance(result, exception.HeatInternalFailureError)
         mock_call.assert_called_once_with(
-            dummy_req.context, ('list_stacks', mock.ANY), version='1.8')
+            dummy_req.context, ('list_stacks', mock.ANY), version='1.33')
 
     def test_describe_last_updated_time(self):
         params = {'Action': 'DescribeStacks'}
@@ -535,8 +532,9 @@ class CfnStackControllerTest(common.HeatTestCase):
               'nested_depth': 0,
               'user_creds_id': None,
               'parent_resource_name': None,
-              'stack_user_project_id': None}),
-            version='1.23'
+              'stack_user_project_id': None,
+              'template_id': None}),
+            version='1.29'
         ).AndRaise(failure)
 
     def _stub_rpc_create_stack_call_success(self, stack_name, engine_parms,
@@ -564,8 +562,9 @@ class CfnStackControllerTest(common.HeatTestCase):
               'nested_depth': 0,
               'user_creds_id': None,
               'parent_resource_name': None,
-              'stack_user_project_id': None}),
-            version='1.23'
+              'stack_user_project_id': None,
+              'template_id': None}),
+            version='1.29'
         ).AndReturn(engine_resp)
 
         self.m.ReplayAll()
@@ -874,8 +873,9 @@ class CfnStackControllerTest(common.HeatTestCase):
               'params': engine_parms,
               'files': {},
               'environment_files': None,
-              'args': engine_args}),
-            version='1.23'
+              'args': engine_args,
+              'template_id': None}),
+            version='1.29'
         ).AndReturn(identity)
 
         self.m.ReplayAll()
@@ -1028,31 +1028,6 @@ class CfnStackControllerTest(common.HeatTestCase):
         result = self.controller.get_template(dummy_req)
         self.assertIsInstance(result, exception.HeatInvalidParameterValueError)
 
-    def test_get_template_err_none(self):
-        stack_name = "wordpress"
-        identity = dict(identifier.HeatIdentifier('t', stack_name, '6'))
-        params = {'Action': 'GetTemplate', 'StackName': stack_name}
-        dummy_req = self._dummy_GET_request(params)
-        self._stub_enforce(dummy_req, 'GetTemplate')
-
-        # Stub out the RPC call to the engine to return None
-        # this test the "no such stack" error path
-        engine_resp = None
-
-        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
-        rpc_client.EngineClient.call(
-            dummy_req.context, ('identify_stack', {'stack_name': stack_name})
-        ).AndReturn(identity)
-        rpc_client.EngineClient.call(
-            dummy_req.context, ('get_template', {'stack_identity': identity})
-        ).AndReturn(engine_resp)
-
-        self.m.ReplayAll()
-
-        result = self.controller.get_template(dummy_req)
-
-        self.assertIsInstance(result, exception.HeatInvalidParameterValueError)
-
     def test_validate_err_no_template(self):
         # Format a dummy request with a missing template field
         params = {'Action': 'ValidateTemplate'}
@@ -1213,7 +1188,7 @@ class CfnStackControllerTest(common.HeatTestCase):
                         u'resource_properties': {u'UserData': u'blah'},
                         u'resource_type': u'AWS::EC2::Instance'}]
 
-        kwargs = {'stack_identity': identity,
+        kwargs = {'stack_identity': identity, 'nested_depth': None,
                   'limit': None, 'sort_keys': None, 'marker': None,
                   'sort_dir': None, 'filters': None}
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
@@ -1221,7 +1196,9 @@ class CfnStackControllerTest(common.HeatTestCase):
             dummy_req.context, ('identify_stack', {'stack_name': stack_name})
         ).AndReturn(identity)
         rpc_client.EngineClient.call(
-            dummy_req.context, ('list_events', kwargs)
+            dummy_req.context,
+            ('list_events', kwargs),
+            version='1.31'
         ).AndReturn(engine_resp)
 
         self.m.ReplayAll()
@@ -1259,7 +1236,9 @@ class CfnStackControllerTest(common.HeatTestCase):
             dummy_req.context, ('identify_stack', {'stack_name': stack_name})
         ).AndReturn(identity)
         rpc_client.EngineClient.call(
-            dummy_req.context, ('list_events', {'stack_identity': identity})
+            dummy_req.context,
+            ('list_events', {'stack_identity': identity}),
+            version='1.31'
         ).AndRaise(Exception())
 
         self.m.ReplayAll()
