@@ -12,7 +12,6 @@
 #    under the License.
 
 import os
-import six
 
 from oslo_config import fixture as config_fixture
 
@@ -51,9 +50,6 @@ class WatchControllerTest(common.HeatTestCase):
                                                        'deny_stack_user.json')
         self.addCleanup(self.m.VerifyAll)
 
-    def tearDown(self):
-        super(WatchControllerTest, self).tearDown()
-
     def _dummy_GET_request(self, params=None):
         # Mangle the params dict into a query string
         params = params or {}
@@ -500,7 +496,7 @@ class WatchControllerTest(common.HeatTestCase):
                      'ALARM': rpc_api.WATCH_STATE_ALARM,
                      'INSUFFICIENT_DATA': rpc_api.WATCH_STATE_NODATA}
 
-        for state in six.iterkeys(state_map):
+        for state in state_map:
             params = {u'StateValue': state,
                       u'StateReason': u'',
                       u'AlarmName': u'HttpFailureAlarm',
