@@ -19,6 +19,7 @@ import six
 
 from heat.common import exception
 from heat.common import template_format
+from heat.engine import node_data
 from heat.engine import resource
 from heat.engine import scheduler
 from heat.tests.autoscaling import inline_templates
@@ -157,15 +158,15 @@ class TestAutoScalingPolicy(common.HeatTestCase):
 
     def test_scaling_policy_refid_convg_cache_data(self):
         t = template_format.parse(as_template)
-        cache_data = {'my-policy': {
+        cache_data = {'my-policy': node_data.NodeData.from_dict({
             'uuid': mock.ANY,
             'id': mock.ANY,
             'action': 'CREATE',
             'status': 'COMPLETE',
             'reference_id': 'convg_xyz'
-        }}
+        })}
         stack = utils.parse_stack(t, cache_data=cache_data)
-        rsrc = stack['my-policy']
+        rsrc = stack.defn['my-policy']
         self.assertEqual('convg_xyz', rsrc.FnGetRefId())
 
 
@@ -299,7 +300,7 @@ class ScalingPolicyAttrTest(common.HeatTestCase):
         self.assertEqual('heat', base[2])
         self.assertEqual('test_tenant_id', base[4])
 
-        res = base[5].split('%2F')
+        res = base[5].split('/')
         self.assertEqual('stacks', res[0])
         self.assertEqual(self.stack_name, res[1])
         self.assertEqual('resources', res[3])
@@ -317,7 +318,7 @@ class ScalingPolicyAttrTest(common.HeatTestCase):
         self.m.StubOutWithMock(self.stack.clients.client_plugin('heat'),
                                'get_heat_url')
         self.stack.clients.client_plugin('heat').get_heat_url().AndReturn(
-            'http://server.test:8000/v1')
+            'http://server.test:8000/v1/')
         self.m.ReplayAll()
         self.assertEqual(
             'http://server.test:8000/v1/test_tenant_id/stacks/'
@@ -325,3 +326,16 @@ class ScalingPolicyAttrTest(common.HeatTestCase):
                 self.stack.name, self.stack.id),
             self.policy.FnGetAtt('signal_url'))
         self.m.VerifyAll()
+
+    def test_signal_attribute_with_prefix(self):
+        self.m.StubOutWithMock(self.stack.clients.client_plugin('heat'),
+                               'get_heat_url')
+        self.stack.clients.client_plugin('heat').get_heat_url().AndReturn(
+            'http://server.test/heat-api/v1/1234')
+        self.m.ReplayAll()
+        self.assertEqual(
+            'http://server.test/heat-api/v1/test_tenant_id/stacks/'
+            '%s/%s/resources/my-policy/signal' % (
+                self.stack.name, self.stack.id),
+            self.policy.FnGetAtt('signal_url'))
+        self.m.VerifyAll()
