@@ -17,9 +17,11 @@ from oslo_config import cfg
 from heat.common import exception
 from heat.common import short_id
 from heat.common import template_format
+from heat.engine import node_data
 from heat.engine.resources.aws.iam import user
 from heat.engine.resources.openstack.heat import access_policy as ap
 from heat.engine import scheduler
+from heat.engine import stk_defn
 from heat.objects import resource_data as resource_data_object
 from heat.tests import common
 from heat.tests import fakes
@@ -282,15 +284,15 @@ class UserTest(common.HeatTestCase):
 
     def test_user_refid_convg_cache_data(self):
         t = template_format.parse(user_template)
-        cache_data = {'CfnUser': {
+        cache_data = {'CfnUser': node_data.NodeData.from_dict({
             'uuid': mock.ANY,
             'id': mock.ANY,
             'action': 'CREATE',
             'status': 'COMPLETE',
             'reference_id': 'convg_xyz'
-        }}
+        })}
         stack = utils.parse_stack(t, cache_data=cache_data)
-        rsrc = stack['CfnUser']
+        rsrc = stack.defn['CfnUser']
         self.assertEqual('convg_xyz', rsrc.FnGetRefId())
 
 
@@ -315,13 +317,12 @@ class AccessKeyTest(common.HeatTestCase):
         self.assertIsNone(rsrc.validate())
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
+        stk_defn.update_resource_data(stack.defn, resource_name,
+                                      rsrc.node_data())
         return rsrc
 
     def create_access_key(self, t, stack, resource_name):
-        resource_defns = stack.t.resource_definitions(stack)
-        rsrc = user.AccessKey(resource_name,
-                              resource_defns[resource_name],
-                              stack)
+        rsrc = stack[resource_name]
         self.assertIsNone(rsrc.validate())
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
