@@ -481,9 +481,9 @@ class VolumeTest(vt_base.BaseVolumeTest):
         stack = utils.parse_stack(self.t, stack_name=stack_name)
         rsrc = self.create_volume(self.t, stack, 'DataVolume')
         deleter = scheduler.TaskRunner(rsrc.destroy)
-        self.assertRaisesRegexp(exception.ResourceFailure,
-                                ".*ResourceInError.*error_deleting.*delete",
-                                deleter)
+        self.assertRaisesRegex(exception.ResourceFailure,
+                               ".*ResourceInError.*error_deleting.*delete",
+                               deleter)
 
         self.m.VerifyAll()
 
@@ -517,6 +517,7 @@ class VolumeTest(vt_base.BaseVolumeTest):
     def test_volume_check(self):
         stack = utils.parse_stack(self.t, stack_name='volume_check')
         res = stack['DataVolume']
+        res.state_set(res.CREATE, res.COMPLETE)
         fake_volume = vt_base.FakeVolume('available')
         cinder = mock.Mock()
         cinder.volumes.get.return_value = fake_volume
@@ -533,6 +534,7 @@ class VolumeTest(vt_base.BaseVolumeTest):
     def test_volume_check_not_available(self):
         stack = utils.parse_stack(self.t, stack_name='volume_check_na')
         res = stack['DataVolume']
+        res.state_set(res.CREATE, res.COMPLETE)
         cinder = mock.Mock()
         fake_volume = vt_base.FakeVolume('foobar')
         cinder.volumes.get.return_value = fake_volume
@@ -546,6 +548,7 @@ class VolumeTest(vt_base.BaseVolumeTest):
     def test_volume_check_fail(self):
         stack = utils.parse_stack(self.t, stack_name='volume_check_fail')
         res = stack['DataVolume']
+        res.state_set(res.CREATE, res.COMPLETE)
         cinder = mock.Mock()
         cinder.volumes.get.side_effect = Exception('boom')
         self.patchobject(res, 'client', return_value=cinder)
@@ -736,3 +739,14 @@ class VolumeTest(vt_base.BaseVolumeTest):
                       six.text_type(ex))
         self.assertEqual((rsrc.UPDATE, rsrc.FAILED), rsrc.state)
         self.m.VerifyAll()
+
+    def test_vaildate_deletion_policy(self):
+        cfg.CONF.set_override('backups_enabled', False, group='volumes')
+        stack_name = 'test_volume_validate_deletion_policy'
+        self.t['Resources']['DataVolume']['DeletionPolicy'] = 'Snapshot'
+        stack = utils.parse_stack(self.t, stack_name=stack_name)
+        rsrc = self.get_volume(self.t, stack, 'DataVolume')
+        self.assertRaisesRegex(
+            exception.StackValidationFailed,
+            'volume backup service is not enabled',
+            rsrc.validate)
