@@ -22,6 +22,7 @@ from keystoneauth1.identity import generic
 from manilaclient import exceptions as manila_exc
 import mock
 from neutronclient.common import exceptions as neutron_exc
+from openstack import exceptions
 from oslo_config import cfg
 from saharaclient.api import base as sahara_base
 import six
@@ -37,21 +38,18 @@ from heat.engine.clients import client_plugin
 from heat.tests import common
 from heat.tests import fakes
 from heat.tests.openstack.nova import fakes as fakes_nova
-from heat.tests import utils
 
 
 class ClientsTest(common.HeatTestCase):
 
     def test_bad_cloud_backend(self):
         con = mock.Mock()
-        cfg.CONF.set_override('cloud_backend', 'some.weird.object',
-                              enforce_type=True)
+        cfg.CONF.set_override('cloud_backend', 'some.weird.object')
         exc = self.assertRaises(exception.Invalid, clients.Clients, con)
         self.assertIn('Invalid cloud_backend setting in heat.conf detected',
                       six.text_type(exc))
 
-        cfg.CONF.set_override('cloud_backend', 'heat.engine.clients.Clients',
-                              enforce_type=True)
+        cfg.CONF.set_override('cloud_backend', 'heat.engine.clients.Clients')
         exc = self.assertRaises(exception.Invalid, clients.Clients, con)
         self.assertIn('Invalid cloud_backend setting in heat.conf detected',
                       six.text_type(exc))
@@ -102,7 +100,7 @@ class ClientsTest(common.HeatTestCase):
 
     def test_clients_get_heat_cfn_metadata_url_conf(self):
         cfg.CONF.set_override('heat_metadata_server_url',
-                              'http://server.test:123', enforce_type=True)
+                              'http://server.test:123')
         obj = self._client_cfn_url()
         self.assertEqual("http://server.test:123/v1/",
                          obj.get_cfn_metadata_server_url())
@@ -130,14 +128,13 @@ class ClientsTest(common.HeatTestCase):
         con.tenant_id = "b363706f891f48019483f8bd6503c54b"
         con.auth_token = None
         con.auth_plugin = fakes.FakeAuth(auth_token='anewtoken')
-        self.stub_keystoneclient(context=con)
         c = clients.Clients(con)
         con.clients = c
 
         obj = c.client_plugin('heat')
         obj.url_for = mock.Mock(name="url_for")
         obj.url_for.return_value = "url_from_keystone"
-        self.assertEqual('anewtoken', c.client('keystone').auth_token)
+        self.assertEqual('url_from_keystone', obj.get_heat_url())
 
     @mock.patch.object(heatclient, 'Client')
     def test_clients_heat_cached(self, mock_call):
@@ -183,11 +180,11 @@ class ClientPluginTest(common.HeatTestCase):
         plugin = FooClientsPlugin(con)
 
         cfg.CONF.set_override('ca_file', '/tmp/bar',
-                              group='clients_heat', enforce_type=True)
+                              group='clients_heat')
         cfg.CONF.set_override('ca_file', '/tmp/foo',
-                              group='clients', enforce_type=True)
+                              group='clients')
         cfg.CONF.set_override('endpoint_type', 'internalURL',
-                              group='clients', enforce_type=True)
+                              group='clients')
 
         # check heat group
         self.assertEqual('/tmp/bar',
@@ -297,42 +294,6 @@ class ClientPluginTest(common.HeatTestCase):
 
         self.assertRaises(TypeError, client_plugin.ClientPlugin, c)
 
-    def test_create_client_on_token_expiration(self):
-        cfg.CONF.set_override('reauthentication_auth_method', 'trusts',
-                              enforce_type=True)
-        con = utils.dummy_context()
-        auth_ref = mock.Mock()
-        self.patchobject(con.auth_plugin, 'get_auth_ref',
-                         return_value=auth_ref)
-        auth_ref.will_expire_soon.return_value = False
-        plugin = FooClientsPlugin(con)
-        plugin._create = mock.Mock()
-        plugin.client()
-        self.assertEqual(1, plugin._create.call_count)
-        plugin.client()
-        self.assertEqual(1, plugin._create.call_count)
-        auth_ref.will_expire_soon.return_value = True
-        plugin.client()
-        self.assertEqual(2, plugin._create.call_count)
-
-    def test_create_client_on_invalidate(self):
-        cfg.CONF.set_override('reauthentication_auth_method', 'trusts',
-                              enforce_type=True)
-        con = utils.dummy_context()
-        auth_ref = mock.Mock()
-        self.patchobject(con.auth_plugin, 'get_auth_ref',
-                         return_value=auth_ref)
-        auth_ref.will_expire_soon.return_value = False
-        plugin = FooClientsPlugin(con)
-        plugin._create = mock.Mock()
-        plugin.client()
-        self.assertEqual(1, plugin._create.call_count)
-        plugin.client()
-        self.assertEqual(1, plugin._create.call_count)
-        plugin.invalidate()
-        plugin.client()
-        self.assertEqual(2, plugin._create.call_count)
-
 
 class TestClientPluginsInitialise(common.HeatTestCase):
 
@@ -368,20 +329,6 @@ class TestClientPluginsInitialise(common.HeatTestCase):
             self.assertGreaterEqual(len(plugin.service_types), 1,
                                     'service_types is not defined for plugin')
 
-    @mock.patch.object(client_plugin.ClientPlugin, 'invalidate')
-    def test_invalidate_all_clients(self, mock_invalidate):
-        plugin_types = clients._mgr.names()
-        con = mock.Mock()
-        c = clients.Clients(con)
-        con.clients = c
-        for plugin_name in plugin_types:
-            plugin = c.client_plugin(plugin_name)
-            self.assertIsNotNone(plugin)
-        c.invalidate_plugins()
-        # while client plugin is initialized and while client is invoked
-        # its being invalidated, so the count will be doubled
-        self.assertEqual(len(plugin_types) * 2, mock_invalidate.call_count)
-
 
 class TestIsNotFound(common.HeatTestCase):
 
@@ -692,6 +639,15 @@ class TestIsNotFound(common.HeatTestCase):
             plugin='nova',
             exception=lambda: fakes_nova.fake_exception(409),
         )),
+        ('openstack_not_found', dict(
+            is_not_found=True,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=False,
+            is_unprocessable_entity=False,
+            plugin='openstack',
+            exception=lambda: exceptions.ResourceNotFound,
+        )),
         ('swift_not_found', dict(
             is_not_found=True,
             is_over_limit=False,
