@@ -46,12 +46,11 @@ class ResourceTypeTest(common.HeatTestCase):
         self.assertEqual(set(['OS::Designate::Domain',
                               'OS::Designate::Record',
                               'OS::Heat::HARestarter',
-                              'OS::Neutron::HealthMonitor',
-                              'OS::Neutron::LoadBalancer',
-                              'OS::Neutron::Pool',
-                              'OS::Neutron::PoolMember',
+                              'OS::Magnum::Bay',
+                              'OS::Magnum::BayModel',
                               'OS::Glance::Image',
-                              'OS::Aodh::CombinationAlarm']),
+                              'OS::Nova::FloatingIP',
+                              'OS::Nova::FloatingIPAssociation']),
                          set(resources))
 
     @mock.patch.object(res.Resource, 'is_service_available')
@@ -81,14 +80,14 @@ class ResourceTypeTest(common.HeatTestCase):
         description = ("Heat Template Resource for Designate Domain.\n\n"
                        "Designate provides DNS-as-a-Service services for "
                        "OpenStack. So, domain\nis a realm with an "
-                       "identification string, unique in DNS.\n")
+                       "identification string, unique in DNS.")
         self.assertIn({'resource_type': 'OS::Designate::Domain',
                        'description': description}, resources)
         self.assertIn({'resource_type': 'AWS::RDS::DBInstance',
                        'description': 'Builtin AWS::RDS::DBInstance'},
                       resources)
         self.assertIn({'resource_type': 'AWS::EC2::Instance',
-                       'description': 'No description given'},
+                       'description': 'No description available'},
                       resources)
 
     def test_resource_schema(self):
@@ -122,7 +121,7 @@ class ResourceTypeTest(common.HeatTestCase):
                 'message': None,
                 'previous_status': None
             },
-            'description': 'No description given'
+            'description': 'No description available'
         }
 
         schema = self.eng.resource_schema(self.ctx, type_name=type_name,
