@@ -285,6 +285,7 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
         self.assertEqual(u'False', rsrc.FnGetAtt('bootable'))
         self.assertEqual(u'False', rsrc.FnGetAtt('encrypted'))
         self.assertEqual(u'[]', rsrc.FnGetAtt('attachments'))
+        self.assertEqual([], rsrc.FnGetAtt('attachments_list'))
         self.assertEqual('False', rsrc.FnGetAtt('multiattach'))
         error = self.assertRaises(exception.InvalidTemplateAttribute,
                                   rsrc.FnGetAtt, 'unknown')
@@ -1038,8 +1039,7 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
 
     def test_cinder_create_with_stack_scheduler_hints(self):
         fv = vt_base.FakeVolume('creating')
-        sh.cfg.CONF.set_override('stack_scheduler_hints', True,
-                                 enforce_type=True)
+        sh.cfg.CONF.set_override('stack_scheduler_hints', True)
 
         stack_name = 'test_cvolume_stack_scheduler_hints_stack'
         t = template_format.parse(single_cinder_volume_template)
@@ -1089,17 +1089,16 @@ class CinderVolumeTest(vt_base.BaseVolumeTest):
     def test_cinder_create_with_image_and_imageRef(self):
         stack_name = 'test_create_with_image_and_imageRef'
         combinations = {'imageRef': 'image-456', 'image': 'image-123'}
-        err_msg = ("Property error: resources.volume2.properties: Cannot "
-                   "define the following properties at the same time: "
-                   "['image', 'imageRef'].")
+        err_msg = ("Cannot define the following properties at the same time: "
+                   "image, imageRef")
         self.stub_ImageConstraint_validate()
         stack = utils.parse_stack(self.t, stack_name=stack_name)
         vp = stack.t['Resources']['volume2']['Properties']
         vp.pop('size')
         vp.update(combinations)
-        ex = self.assertRaises(exception.StackValidationFailed,
-                               stack.get, 'volume2')
-        self.assertEqual(err_msg, six.text_type(ex))
+        rsrc = stack.get('volume2')
+        ex = self.assertRaises(exception.StackValidationFailed, rsrc.validate)
+        self.assertIn(err_msg, six.text_type(ex))
 
     def test_cinder_create_with_image_and_size(self):
         stack_name = 'test_create_with_image_and_size'
