@@ -21,10 +21,11 @@ from heat.common import grouputils
 from heat.common import short_id
 from heat.common import template_format
 from heat.engine.clients.os import neutron
+from heat.engine import resource
 from heat.engine.resources.openstack.heat import instance_group as instgrp
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
-from heat.engine import stack as parser
+from heat.engine import stk_defn
 from heat.tests.autoscaling import inline_templates
 from heat.tests import common
 from heat.tests import utils
@@ -209,7 +210,7 @@ class TestLaunchConfig(common.HeatTestCase):
                                                       metadata)
         # Changing metadata in the second update triggers UpdateReplace
         updater = scheduler.TaskRunner(rsrc.update, update_snippet)
-        self.assertRaises(exception.UpdateReplace, updater)
+        self.assertRaises(resource.UpdateReplace, updater)
 
 
 class LoadbalancerReloadTest(common.HeatTestCase):
@@ -319,21 +320,12 @@ class LoadbalancerReloadTest(common.HeatTestCase):
         properties = t['Resources']['ElasticLoadBalancer']['Properties']
         properties['AvailabilityZones'] = {'Fn::GetAZs': ''}
 
-        self.patchobject(parser.Stack, 'get_availability_zones',
+        self.patchobject(stk_defn.StackDefinition, 'get_availability_zones',
                          return_value=['abc', 'xyz'])
 
         mock_members = self.patchobject(grouputils, 'get_member_refids')
         mock_members.return_value = ['aaaabbbbcccc']
 
-        # Check that the Fn::GetAZs is correctly resolved
-        expected = {u'Properties': {'Instances': ['aaaabbbbcccc'],
-                                    u'Listeners': [{u'InstancePort': u'80',
-                                                    u'LoadBalancerPort': u'80',
-                                                    u'Protocol': u'HTTP'}],
-                                    u'AvailabilityZones': ['abc', 'xyz']},
-                    u'DeletionPolicy': 'Delete',
-                    u'Metadata': {}}
-
         stack = utils.parse_stack(t, params=inline_templates.as_params)
         lb = stack['ElasticLoadBalancer']
         lb.state_set(lb.CREATE, lb.COMPLETE)
@@ -341,7 +333,7 @@ class LoadbalancerReloadTest(common.HeatTestCase):
         group = stack['WebServerGroup']
         group._lb_reload()
         lb.handle_update.assert_called_once_with(
-            mock.ANY, expected,
+            mock.ANY, mock.ANY,
             {'Instances': ['aaaabbbbcccc']})
 
 
@@ -350,12 +342,12 @@ class InstanceGroupWithNestedStack(common.HeatTestCase):
         super(InstanceGroupWithNestedStack, self).setUp()
         t = template_format.parse(inline_templates.as_template)
         self.stack = utils.parse_stack(t, params=inline_templates.as_params)
-        lc = self.create_launch_config(t, self.stack)
-        lcid = lc.FnGetRefId()
+        self.create_launch_config(t, self.stack)
+        wsg_props = self.stack['WebServerGroup'].t._properties
         self.defn = rsrc_defn.ResourceDefinition(
             'asg', 'OS::Heat::InstanceGroup',
             {'Size': 2, 'AvailabilityZones': ['zoneb'],
-             'LaunchConfigurationName': lcid})
+             'LaunchConfigurationName': wsg_props['LaunchConfigurationName']})
         self.group = instgrp.InstanceGroup('asg', self.defn, self.stack)
 
         self.group._lb_reload = mock.Mock()
