@@ -19,10 +19,12 @@ import six
 from heat.common import exception
 from heat.common import grouputils
 from heat.common import template_format
+from heat.engine.clients.os import glance
+from heat.engine.clients.os import nova
+from heat.engine import node_data
 from heat.engine.resources.openstack.heat import resource_group
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
-from heat.engine import stack as stackm
 from heat.tests import common
 from heat.tests import utils
 
@@ -113,12 +115,30 @@ template_attr = {
     }
 }
 
+template_server = {
+    "heat_template_version": "2013-05-23",
+    "resources": {
+        "group1": {
+            "type": "OS::Heat::ResourceGroup",
+            "properties": {
+                "count": 2,
+                "resource_def": {
+                    "type": "OS::Nova::Server",
+                    "properties": {
+                        "image": "image%index%",
+                        "flavor": "flavor%index%"
+                    }
+                }
+            }
+        }
+    }
+}
+
 
 class ResourceGroupTest(common.HeatTestCase):
 
     def setUp(self):
         common.HeatTestCase.setUp(self)
-        self.m.StubOutWithMock(stackm.Stack, 'validate')
 
     def test_assemble_nested(self):
         """Tests nested stack creation based on props.
@@ -444,6 +464,30 @@ class ResourceGroupTest(common.HeatTestCase):
         resg = stack.resources['group1']
         self.assertIsNone(resg.validate())
 
+    def test_validate_with_blacklist(self):
+        templ = copy.deepcopy(template_server)
+        self.mock_flavor = mock.Mock(ram=4, disk=4)
+        self.mock_active_image = mock.Mock(min_ram=1, min_disk=1,
+                                           status='active')
+        self.mock_inactive_image = mock.Mock(min_ram=1, min_disk=1,
+                                             status='inactive')
+
+        def get_image(image_identifier):
+            if image_identifier == 'image0':
+                return self.mock_inactive_image
+            else:
+                return self.mock_active_image
+
+        self.patchobject(glance.GlanceClientPlugin, 'get_image',
+                         side_effect=get_image)
+        self.patchobject(nova.NovaClientPlugin, 'get_flavor',
+                         return_value=self.mock_flavor)
+        props = templ["resources"]["group1"]["properties"]
+        props["removal_policies"] = [{"resource_list": ["0"]}]
+        stack = utils.parse_stack(templ)
+        resg = stack.resources['group1']
+        self.assertIsNone(resg.validate())
+
     def test_invalid_res_type(self):
         """Test that error raised for unknown resource type."""
         tmp = copy.deepcopy(template)
@@ -832,16 +876,16 @@ class ResourceGroupAttrTest(common.HeatTestCase):
         self.assertEqual(['0', '1'], rsrc.FnGetAtt(rsrc.REFS))
 
     def test_get_attribute_convg(self):
-        cache_data = {'group1': {
+        cache_data = {'group1': node_data.NodeData.from_dict({
             'uuid': mock.ANY,
             'id': mock.ANY,
             'action': 'CREATE',
             'status': 'COMPLETE',
             'attrs': {'refs': ['rsrc1', 'rsrc2']}
-        }}
+        })}
         stack = utils.parse_stack(template, cache_data=cache_data)
-        rsrc = stack['group1']
-        self.assertEqual(['rsrc1', 'rsrc2'], rsrc.FnGetAtt(rsrc.REFS))
+        rsrc = stack.defn['group1']
+        self.assertEqual(['rsrc1', 'rsrc2'], rsrc.FnGetAtt('refs'))
 
     def test_get_attribute_blacklist(self):
         resg = self._create_dummy_stack()
