@@ -11,6 +11,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import contextlib
 import copy
 import re
 import uuid
@@ -26,6 +27,7 @@ from heat.common import template_format
 from heat.engine.clients.os import nova
 from heat.engine.clients.os import swift
 from heat.engine.clients.os import zaqar
+from heat.engine import node_data
 from heat.engine import resource
 from heat.engine.resources.openstack.heat import software_deployment as sd
 from heat.engine import rsrc_defn
@@ -203,6 +205,15 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         self.rpc_client = mock.MagicMock()
         self.deployment._rpc_client = self.rpc_client
 
+        @contextlib.contextmanager
+        def exc_filter(*args):
+            try:
+                yield
+            except exc.NotFound:
+                pass
+
+        self.rpc_client.ignore_error_by_name.side_effect = exc_filter
+
     def test_validate(self):
         template = dict(self.template_with_server)
         props = template['Resources']['server']['Properties']
@@ -674,6 +685,29 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         self.assertEqual(
             'Deployment to server failed: something wrong', six.text_type(err))
 
+    def test_handle_create_cancel(self):
+        self._create_stack(self.template)
+        mock_sd = self.mock_deployment()
+        self.rpc_client.show_software_deployment.return_value = mock_sd
+        self.deployment.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
+
+        # status in_progress
+        mock_sd['status'] = self.deployment.IN_PROGRESS
+        self.deployment.handle_create_cancel(None)
+        self.assertEqual(
+            'FAILED',
+            self.rpc_client.update_software_deployment.call_args[1]['status'])
+
+        # status failed
+        mock_sd['status'] = self.deployment.FAILED
+        self.deployment.handle_create_cancel(None)
+
+        # deployment not created
+        mock_sd = None
+        self.deployment.handle_create_cancel(None)
+        self.assertEqual(1,
+                         self.rpc_client.update_software_deployment.call_count)
+
     def test_handle_delete(self):
         self._create_stack(self.template)
         mock_sd = self.mock_deployment()
@@ -1085,15 +1119,16 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         self.assertEqual(0, self.deployment.FnGetAtt('deploy_status_code'))
 
     def test_fn_get_att_convg(self):
-        cache_data = {'deployment_mysql': {
+        cache_data = {'deployment_mysql': node_data.NodeData.from_dict({
             'uuid': mock.ANY,
             'id': mock.ANY,
             'action': 'CREATE',
             'status': 'COMPLETE',
             'attrs': {'foo': 'bar'}
-        }}
+        })}
         self._create_stack(self.template, cache_data=cache_data)
-        self.assertEqual('bar', self.deployment.FnGetAtt('foo'))
+        self.assertEqual('bar',
+                         self.stack.defn[self.deployment.name].FnGetAtt('foo'))
 
     def test_fn_get_att_error(self):
         self._create_stack(self.template)
@@ -1187,7 +1222,11 @@ class SoftwareDeploymentTest(common.HeatTestCase):
             }],
             'outputs': [],
         }
-        self.rpc_client.show_software_config.return_value = config
+
+        def show_sw_config(*args):
+            return config.copy()
+
+        self.rpc_client.show_software_config.side_effect = show_sw_config
         mock_sd = self.mock_deployment()
 
         self.rpc_client.show_software_deployment.return_value = mock_sd
