@@ -21,6 +21,7 @@ from oslo_serialization import jsonutils
 from heat.common import exception
 from heat.common import template_format
 from heat.engine.clients.os import mistral as client
+from heat.engine import node_data
 from heat.engine import resource
 from heat.engine.resources.openstack.mistral import workflow
 from heat.engine.resources import signal_responder
@@ -130,7 +131,8 @@ resources:
            - wait_instance
        - name: on_error
          action: std.echo output="output"
-
+       - name: external_workflow
+         workflow: external_workflow_name
 """
 
 workflow_updating_request_body_property = """
@@ -338,7 +340,7 @@ resources:
               count: 15
             wait_before: 5
             wait_after: 5
-            pause_before: 5
+            pause_before: true
             timeout: 42
             concurrency: 5
 """
@@ -781,13 +783,12 @@ class TestMistralWorkflow(common.HeatTestCase):
 
         rsrc_defns = stack.t.resource_definitions(stack)['workflow']
 
+        workflow_rsrc = workflow.Workflow('workflow', rsrc_defns, stack)
         ex = self.assertRaises(exception.StackValidationFailed,
-                               workflow.Workflow, 'workflow', rsrc_defns,
-                               stack)
-        error_msg = ("Property error: resources.workflow.properties: Cannot "
-                     "define the following properties at the same time: "
-                     "['retry', 'policies'].")
-        self.assertEqual(error_msg, six.text_type(ex))
+                               workflow_rsrc.validate)
+        error_msg = ("Cannot define the following properties at "
+                     "the same time: tasks.retry, tasks.policies.retry")
+        self.assertIn(error_msg, six.text_type(ex))
 
     def validate_json_inputs(self, actual_input, expected_input):
         actual_json_input = jsonutils.loads(actual_input)
@@ -820,7 +821,7 @@ class TestMistralWorkflow(common.HeatTestCase):
 
         return [FakeWorkflow('create_vm')]
 
-    def test_mistal_workflow_refid(self):
+    def test_mistral_workflow_refid(self):
         tmpl = template_format.parse(workflow_template)
         stack = utils.parse_stack(tmpl, stack_name='test')
         rsrc = stack['workflow']
@@ -829,18 +830,18 @@ class TestMistralWorkflow(common.HeatTestCase):
         rsrc.action = 'CREATE'
         self.assertEqual('test-workflow-owevpzgiqw66', rsrc.FnGetRefId())
 
-    def test_mistal_workflow_refid_convergence_cache_data(self):
+    def test_mistral_workflow_refid_convergence_cache_data(self):
         tmpl = template_format.parse(workflow_template)
-        cache_data = {'workflow': {
+        cache_data = {'workflow': node_data.NodeData.from_dict({
             'uuid': mock.ANY,
             'id': mock.ANY,
             'action': 'CREATE',
             'status': 'COMPLETE',
             'reference_id': 'convg_xyz'
-        }}
+        })}
         stack = utils.parse_stack(tmpl, stack_name='test',
                                   cache_data=cache_data)
-        rsrc = stack['workflow']
+        rsrc = stack.defn['workflow']
         self.assertEqual('convg_xyz', rsrc.FnGetRefId())
 
     def test_policies_translation_successful(self):
@@ -849,11 +850,12 @@ class TestMistralWorkflow(common.HeatTestCase):
         rsrc_defns = stack.t.resource_definitions(stack)['workflow']
         wf = workflow.Workflow('workflow', rsrc_defns, stack)
 
-        self.assertEqual([{'name': 'check_dat_thing',
-                           'action': 'nova.servers_list',
-                           'retry': {'delay': 5, 'count': 15},
-                           'wait_before': 5,
-                           'wait_after': 5,
-                           'pause_before': 5,
-                           'timeout': 42,
-                           'concurrency': 5}], wf.properties.data['tasks'])
+        result = {k: v for k, v in wf.properties['tasks'][0].items() if v}
+        self.assertEqual({'name': 'check_dat_thing',
+                          'action': 'nova.servers_list',
+                          'retry': {'delay': 5, 'count': 15},
+                          'wait_before': 5,
+                          'wait_after': 5,
+                          'pause_before': True,
+                          'timeout': 42,
+                          'concurrency': 5}, result)
