@@ -11,11 +11,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
 from neutronclient.common import exceptions as qe
 import six
 
 from heat.common import exception
+from heat.engine import attributes
 from heat.engine import properties
 from heat.engine.resources.openstack.neutron import net
 from heat.engine.resources.openstack.neutron import neutron as nr
@@ -83,17 +83,22 @@ class NeutronTest(common.HeatTestCase):
             def is_service_available(cls, context):
                 return (True, None)
 
-        empty_tmpl = {'heat_template_version': '2016-10-14'}
+        empty_tmpl = {'heat_template_version': 'ocata'}
         tmpl = template.Template(empty_tmpl)
         stack_name = 'dummystack'
         self.dummy_stack = stack.Stack(utils.dummy_context(), stack_name, tmpl)
+        self.dummy_stack.store()
 
         tmpl = rsrc_defn.ResourceDefinition('test_res', 'Foo')
-        self.dummy_stack.has_cache_data = mock.Mock(return_value=False)
         return SomeNeutronResource('aresource', tmpl, self.dummy_stack)
 
     def test_resolve_attribute(self):
         res = self._get_some_neutron_resource()
+        res.attributes_schema.update(
+            {'attr2': attributes.Schema(type=attributes.Schema.STRING)})
+        res.attributes = attributes.Attributes(res.name,
+                                               res.attributes_schema,
+                                               res._resolve_all_attributes)
         side_effect = [{'attr1': 'val1', 'attr2': 'val2'},
                        {'attr1': 'val1', 'attr2': 'val2'},
                        {'attr1': 'val1', 'attr2': 'val2'},
@@ -102,15 +107,52 @@ class NeutronTest(common.HeatTestCase):
         res.resource_id = 'resource_id'
         self.assertEqual({'attr1': 'val1', 'attr2': 'val2'},
                          res.FnGetAtt('show'))
-        self.assertEqual('val2', res._resolve_all_attributes('attr2'))
+        self.assertEqual('val2', res.attributes['attr2'])
         self.assertRaises(KeyError, res._resolve_all_attributes, 'attr3')
-        self.assertIsNone(res._resolve_all_attributes('attr2'))
+        self.assertIsNone(res._resolve_all_attributes('attr1'))
 
         res.resource_id = None
-        # use local cached object
-        self.assertEqual({'attr1': 'val1', 'attr2': 'val2'},
-                         res.FnGetAtt('show'))
-        # reset cache, so resolver should be used again
-        # and return None due to resource_id is None
-        res.attributes.reset_resolved_values()
+        # use local cached object for non-show attribute
+        self.assertEqual('val2',
+                         res.FnGetAtt('attr2'))
+        # but the 'show' attribute is never cached
         self.assertIsNone(res.FnGetAtt('show'))
+
+        # remove 'attr2' from res.attributes cache
+        res.attributes.reset_resolved_values()
+        # _resolve_attribute (in NeutronResource class) returns None
+        # due to no resource_id
+        self.assertIsNone(res.FnGetAtt('attr2'))
+
+    def test_needs_replace_failed(self):
+        res = self._get_some_neutron_resource()
+        res.state_set(res.CREATE, res.FAILED)
+        side_effect = [
+            {'attr1': 'val1', 'status': 'ACTIVE'},
+            {'attr1': 'val1', 'status': 'ERROR'},
+            {'attr1': 'val1', 'attr2': 'val2'},
+            qe.NotFound]
+        mock_show_resource = self.patchobject(res, '_show_resource',
+                                              side_effect=side_effect)
+        # needs replace because res not created yet
+        res.resource_id = None
+        self.assertTrue(res.needs_replace_failed())
+        self.assertEqual(0, mock_show_resource.call_count)
+
+        # no need to replace because res is ACTIVE underlying
+        res.resource_id = 'I am a resource'
+        self.assertFalse(res.needs_replace_failed())
+        self.assertEqual(1, mock_show_resource.call_count)
+
+        # needs replace because res is ERROR underlying
+        self.assertTrue(res.needs_replace_failed())
+        self.assertEqual(2, mock_show_resource.call_count)
+
+        # no need to replace because res exists and no status
+        # to check
+        self.assertFalse(res.needs_replace_failed())
+        self.assertEqual(3, mock_show_resource.call_count)
+
+        # needs replace because res can not be found
+        self.assertTrue(res.needs_replace_failed())
+        self.assertEqual(4, mock_show_resource.call_count)
