@@ -24,9 +24,11 @@ from heat.common import template_format
 from heat.common import timeutils
 from heat.engine.clients.os import neutron
 from heat.engine.hot import functions as hot_funcs
+from heat.engine import node_data
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
 from heat.engine import stack as parser
+from heat.engine import stk_defn
 from heat.engine import template as tmpl
 from heat.tests import common
 from heat.tests import utils
@@ -287,14 +289,14 @@ class NeutronFloatingIPTest(common.HeatTestCase):
         template = tmpl.Template(t)
         stack = parser.Stack(utils.dummy_context(), 'test', template,
                              cache_data={
-                                 'floating_ip': {
+                                 'floating_ip': node_data.NodeData.from_dict({
                                      'uuid': mock.ANY,
                                      'id': mock.ANY,
                                      'action': 'CREATE',
                                      'status': 'COMPLETE',
-                                     'reference_id': 'abc'}})
+                                     'reference_id': 'abc'})})
 
-        rsrc = stack['floating_ip']
+        rsrc = stack.defn['floating_ip']
         self.assertEqual('abc', rsrc.FnGetRefId())
 
     def test_floatip_association_port(self):
@@ -426,14 +428,17 @@ class NeutronFloatingIPTest(common.HeatTestCase):
         fip = stack['floating_ip']
         scheduler.TaskRunner(fip.create)()
         self.assertEqual((fip.CREATE, fip.COMPLETE), fip.state)
+        stk_defn.update_resource_data(stack.defn, fip.name, fip.node_data())
 
         p = stack['port_floating']
         scheduler.TaskRunner(p.create)()
         self.assertEqual((p.CREATE, p.COMPLETE), p.state)
+        stk_defn.update_resource_data(stack.defn, p.name, p.node_data())
 
         fipa = stack['floating_ip_assoc']
         scheduler.TaskRunner(fipa.create)()
         self.assertEqual((fipa.CREATE, fipa.COMPLETE), fipa.state)
+        stk_defn.update_resource_data(stack.defn, fipa.name, fipa.node_data())
         self.assertIsNotNone(fipa.id)
         self.assertEqual(fipa.id, fipa.resource_id)
 
@@ -444,7 +449,7 @@ class NeutronFloatingIPTest(common.HeatTestCase):
         update_port_id = '2146dfbf-ba77-4083-8e86-d052f671ece5'
         props['port_id'] = update_port_id
         update_snippet = rsrc_defn.ResourceDefinition(fipa.name, fipa.type(),
-                                                      stack.t.parse(stack,
+                                                      stack.t.parse(stack.defn,
                                                                     props))
 
         scheduler.TaskRunner(fipa.update, update_snippet)()
@@ -587,6 +592,40 @@ class NeutronFloatingIPTest(common.HeatTestCase):
 
         self.m.VerifyAll()
 
+    def test_floatingip_create_specify_subnet(self):
+        neutronV20.find_resourceid_by_name_or_id(
+            mox.IsA(neutronclient.Client),
+            'network',
+            'abcd1234',
+            cmd_resource=None,
+        ).MultipleTimes().AndReturn('abcd1234')
+        neutronV20.find_resourceid_by_name_or_id(
+            mox.IsA(neutronclient.Client),
+            'subnet',
+            'sub1234',
+            cmd_resource=None,
+        ).MultipleTimes().AndReturn('sub1234')
+        self.stub_NetworkConstraint_validate()
+        neutronclient.Client.create_floatingip({
+            'floatingip': {'floating_network_id': u'abcd1234',
+                           'subnet_id': u'sub1234'}
+        }).AndReturn({'floatingip': {
+            'status': 'ACTIVE',
+            'id': 'fc68ea2c-b60b-4b4f-bd82-94ec81110766',
+            'floating_ip_address': '172.24.4.98'
+        }})
+
+        self.m.ReplayAll()
+        t = template_format.parse(neutron_floating_template)
+        props = t['resources']['floating_ip']['properties']
+        props['floating_subnet'] = 'sub1234'
+        stack = utils.parse_stack(t)
+        fip = stack['floating_ip']
+        scheduler.TaskRunner(fip.create)()
+        self.assertEqual((fip.CREATE, fip.COMPLETE), fip.state)
+
+        self.m.VerifyAll()
+
     def test_floatip_port(self):
         t = template_format.parse(neutron_floating_no_assoc_template)
         t['resources']['port_floating']['properties']['network'] = "xyz1234"
@@ -691,26 +730,29 @@ class NeutronFloatingIPTest(common.HeatTestCase):
         p = stack['port_floating']
         scheduler.TaskRunner(p.create)()
         self.assertEqual((p.CREATE, p.COMPLETE), p.state)
+        stk_defn.update_resource_data(stack.defn, p.name, p.node_data())
 
         fip = stack['floating_ip']
         scheduler.TaskRunner(fip.create)()
         self.assertEqual((fip.CREATE, fip.COMPLETE), fip.state)
+        stk_defn.update_resource_data(stack.defn, fip.name, fip.node_data())
 
         # test update FloatingIp with port_id
         props = copy.deepcopy(fip.properties.data)
         update_port_id = '2146dfbf-ba77-4083-8e86-d052f671ece5'
         props['port_id'] = update_port_id
         update_snippet = rsrc_defn.ResourceDefinition(fip.name, fip.type(),
-                                                      stack.t.parse(stack,
+                                                      stack.t.parse(stack.defn,
                                                                     props))
         scheduler.TaskRunner(fip.update, update_snippet)()
         self.assertEqual((fip.UPDATE, fip.COMPLETE), fip.state)
+        stk_defn.update_resource_data(stack.defn, fip.name, fip.node_data())
 
         # test update FloatingIp with None port_id
         props = copy.deepcopy(fip.properties.data)
         del(props['port_id'])
         update_snippet = rsrc_defn.ResourceDefinition(fip.name, fip.type(),
-                                                      stack.t.parse(stack,
+                                                      stack.t.parse(stack.defn,
                                                                     props))
         scheduler.TaskRunner(fip.update, update_snippet)()
         self.assertEqual((fip.UPDATE, fip.COMPLETE), fip.state)
