@@ -83,6 +83,13 @@ class NeutronPortTest(common.HeatTestCase):
         self.find_mock = self.patchobject(
             neutronV20, 'find_resourceid_by_name_or_id')
 
+    def test_missing_network(self):
+        t = template_format.parse(neutron_port_template)
+        t['resources']['port']['properties'] = {}
+        stack = utils.parse_stack(t)
+        port = stack['port']
+        self.assertRaises(exception.StackValidationFailed, port.validate)
+
     def test_missing_subnet_id(self):
         t = template_format.parse(neutron_port_template)
         t['resources']['port']['properties']['fixed_ips'][0].pop('subnet')
@@ -281,6 +288,30 @@ class NeutronPortTest(common.HeatTestCase):
 
         }}
 
+    def test_create_with_tags(self):
+        t = template_format.parse(neutron_port_template)
+        t['resources']['port']['properties']['tags'] = ['tag1', 'tag2']
+        stack = utils.parse_stack(t)
+
+        port_prop = {
+            'network_id': u'net_or_sub',
+            'fixed_ips': [
+                {'subnet_id': u'net_or_sub', 'ip_address': u'10.0.3.21'}
+            ],
+            'name': utils.PhysName(stack.name, 'port'),
+            'admin_state_up': True,
+            'device_owner': u'network:dhcp'}
+
+        set_tag_mock = self.patchobject(neutronclient.Client, 'replace_tag')
+        self._mock_create_with_props()
+
+        port = stack['port']
+        scheduler.TaskRunner(port.create)()
+        self.assertEqual((port.CREATE, port.COMPLETE), port.state)
+        self.create_mock.assert_called_once_with({'port': port_prop})
+        set_tag_mock.assert_called_with('ports', port.resource_id,
+                                        {'tags': ['tag1', 'tag2']})
+
     def test_security_groups(self):
         t = template_format.parse(neutron_port_template)
         t['resources']['port']['properties']['security_groups'] = [
@@ -429,7 +460,7 @@ class NeutronPortTest(common.HeatTestCase):
         create_kwargs = props.copy()
         create_kwargs['admin_state_up'] = True
 
-        self.find_mock.side_effect = [net1] * 7 + [net2] * 2
+        self.find_mock.side_effect = [net1] * 8 + [net2] * 2 + [net1]
         self.create_mock.return_value = {'port': {
             "status": "ACTIVE",
             "id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766"
@@ -457,23 +488,23 @@ class NeutronPortTest(common.HeatTestCase):
         self.assertEqual((port.CREATE, port.COMPLETE), port.state)
         self.create_mock.assert_called_once_with({'port': create_kwargs})
 
-        # Switch from network_id=ID to network=ID (no replace and no upate)
+        # Switch from network_id=ID to network=ID (no replace)
         new_props = props.copy()
         new_props['network'] = new_props.pop('network_id')
         update_snippet = rsrc_defn.ResourceDefinition(port.name, port.type(),
                                                       new_props)
 
         scheduler.TaskRunner(port.update, update_snippet)()
-        self.assertEqual((port.CREATE, port.COMPLETE), port.state)
+        self.assertEqual((port.UPDATE, port.COMPLETE), port.state)
         self.assertEqual(0, self.update_mock.call_count)
 
-        # Switch from network=ID to network=NAME (no replace and no update)
+        # Switch from network=ID to network=NAME (no replace)
         new_props['network'] = 'net1234'
         update_snippet = rsrc_defn.ResourceDefinition(port.name, port.type(),
                                                       new_props)
 
         scheduler.TaskRunner(port.update, update_snippet)()
-        self.assertEqual((port.CREATE, port.COMPLETE), port.state)
+        self.assertEqual((port.UPDATE, port.COMPLETE), port.state)
         self.assertEqual(0, self.update_mock.call_count)
 
         # Switch to a different network (replace)
@@ -482,7 +513,7 @@ class NeutronPortTest(common.HeatTestCase):
                                                       new_props)
         updater = scheduler.TaskRunner(port.update, update_snippet)
         self.assertRaises(resource.UpdateReplace, updater)
-        self.assertEqual(9, self.find_mock.call_count)
+        self.assertEqual(11, self.find_mock.call_count)
 
     def test_get_port_attributes(self):
         t = template_format.parse(neutron_port_template)
@@ -605,25 +636,6 @@ class NeutronPortTest(common.HeatTestCase):
         self.assertFalse(port.data_set.called)
         self.assertFalse(n_client.update_port.called)
 
-    def test_prepare_for_replace_port_not_found(self):
-        t = template_format.parse(neutron_port_template)
-        stack = utils.parse_stack(t)
-        port = stack['port']
-        port.resource_id = 'test_res_id'
-        port._show_resource = mock.Mock(side_effect=qe.NotFound)
-        port.data_set = mock.Mock()
-        n_client = mock.Mock()
-        port.client = mock.Mock(return_value=n_client)
-
-        # execute prepare_for_replace
-        port.prepare_for_replace()
-
-        # check, if the port is not found, do nothing in
-        # prepare_for_replace()
-        self.assertTrue(port._show_resource.called)
-        self.assertFalse(port.data_set.called)
-        self.assertFalse(n_client.update_port.called)
-
     def test_prepare_for_replace_port(self):
         t = template_format.parse(neutron_port_template)
         stack = utils.parse_stack(t)
@@ -720,6 +732,68 @@ class NeutronPortTest(common.HeatTestCase):
             mock.call(existing_rsrc.resource_id, expected_existing_props),
             mock.call(prev_rsrc.resource_id, expected_prev_props)])
 
+    def test_port_get_live_state(self):
+        t = template_format.parse(neutron_port_template)
+        t['resources']['port']['properties']['value_specs'] = {
+            'binding:vif_type': 'test'}
+
+        stack = utils.parse_stack(t)
+
+        port = stack['port']
+
+        resp = {'port': {
+            'status': 'DOWN',
+            'binding:host_id': '',
+            'name': 'flip-port-xjbal77qope3',
+            'allowed_address_pairs': [],
+            'admin_state_up': True,
+            'network_id': 'd6859535-efef-4184-b236-e5fcae856e0f',
+            'dns_name': '',
+            'extra_dhcp_opts': [],
+            'mac_address': 'fa:16:3e:fe:64:79',
+            'qos_policy_id': 'some',
+            'dns_assignment': [],
+            'binding:vif_details': {},
+            'binding:vif_type': 'unbound',
+            'device_owner': '',
+            'tenant_id': '30f466e3d14b4251853899f9c26e2b66',
+            'binding:profile': {},
+            'port_security_enabled': True,
+            'binding:vnic_type': 'normal',
+            'fixed_ips': [
+                {'subnet_id': '02d9608f-8f30-4611-ad02-69855c82457f',
+                 'ip_address': '10.0.3.4'}],
+            'id': '829bf5c1-b59c-40ad-80e3-ea15a93879f3',
+            'security_groups': ['c276247f-50fd-4289-862a-80fb81a55de1'],
+            'device_id': ''}
+        }
+        port.client().show_port = mock.MagicMock(return_value=resp)
+        port.resource_id = '1234'
+        port._data = {}
+        port.data_set = mock.Mock()
+
+        reality = port.get_live_state(port.properties)
+        expected = {
+            'name': 'flip-port-xjbal77qope3',
+            'allowed_address_pairs': [],
+            'admin_state_up': True,
+            'device_owner': '',
+            'port_security_enabled': True,
+            'binding:vnic_type': 'normal',
+            'fixed_ips': [
+                {'subnet': '02d9608f-8f30-4611-ad02-69855c82457f',
+                 'ip_address': '10.0.3.4'}],
+            'security_groups': ['c276247f-50fd-4289-862a-80fb81a55de1'],
+            'device_id': '',
+            'dns_name': '',
+            'qos_policy': 'some',
+            'value_specs': {'binding:vif_type': 'unbound'}
+        }
+
+        self.assertEqual(set(expected.keys()), set(reality.keys()))
+        for key in expected:
+            self.assertEqual(expected[key], reality[key])
+
 
 class UpdatePortTest(common.HeatTestCase):
     scenarios = [
@@ -813,6 +887,7 @@ class UpdatePortTest(common.HeatTestCase):
         }
         self.patchobject(neutronclient.Client, 'list_security_groups',
                          return_value=fake_groups_list)
+        set_tag_mock = self.patchobject(neutronclient.Client, 'replace_tag')
 
         props = {'network_id': u'net1234',
                  'name': str(utils.PhysName(stack.name, 'port')),
@@ -822,6 +897,7 @@ class UpdatePortTest(common.HeatTestCase):
         update_props = props.copy()
         update_props['security_groups'] = self.secgrp
         update_props['value_specs'] = self.value_specs
+        update_props['tags'] = ['test_tag']
         if self.fixed_ips:
             update_props['fixed_ips'] = self.fixed_ips
         update_props['allowed_address_pairs'] = self.addr_pair
@@ -840,6 +916,8 @@ class UpdatePortTest(common.HeatTestCase):
             for value_spec in six.iteritems(value_specs):
                 update_dict[value_spec[0]] = value_spec[1]
 
+        tags = update_dict.pop('tags')
+
         # create port
         port = stack['port']
         self.assertIsNone(scheduler.TaskRunner(port.handle_create)())
@@ -852,7 +930,8 @@ class UpdatePortTest(common.HeatTestCase):
                                                update_props)())
 
         update_port.assset_called_once_with(update_dict)
-
+        set_tag_mock.assert_called_with('ports', port.resource_id,
+                                        {'tags': tags})
         # check, that update does not cause of Update Replace
         create_snippet = rsrc_defn.ResourceDefinition(port.name, port.type(),
                                                       props)
