@@ -12,6 +12,7 @@
 #    under the License.
 
 import copy
+import mock
 from neutronclient.common import exceptions as qe
 from neutronclient.neutron import v2_0 as neutronV20
 from neutronclient.v2_0 import client as neutronclient
@@ -20,11 +21,14 @@ import six
 from heat.common import exception
 from heat.common import template_format
 from heat.engine.clients.os import neutron
+from heat.engine.clients.os import openstacksdk
 from heat.engine.hot import functions as hot_funcs
+from heat.engine import node_data
 from heat.engine import resource
 from heat.engine.resources.openstack.neutron import subnet
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
+from heat.engine import stk_defn
 from heat.tests import common
 from heat.tests import utils
 
@@ -101,12 +105,20 @@ class NeutronSubnetTest(common.HeatTestCase):
 
     def setUp(self):
         super(NeutronSubnetTest, self).setUp()
-        self.m.StubOutWithMock(neutronclient.Client, 'create_subnet')
-        self.m.StubOutWithMock(neutronclient.Client, 'delete_subnet')
-        self.m.StubOutWithMock(neutronclient.Client, 'show_subnet')
-        self.m.StubOutWithMock(neutronclient.Client, 'update_subnet')
+        self.create_mock = self.patchobject(neutronclient.Client,
+                                            'create_subnet')
+        self.delete_mock = self.patchobject(neutronclient.Client,
+                                            'delete_subnet')
+        self.show_mock = self.patchobject(neutronclient.Client,
+                                          'show_subnet')
+        self.update_mock = self.patchobject(neutronclient.Client,
+                                            'update_subnet')
+
         self.patchobject(neutron.NeutronClientPlugin, 'has_extension',
                          return_value=True)
+        self.patchobject(openstacksdk.OpenStackSDKPlugin,
+                         'find_network_segment',
+                         return_value='fc68ea2c-b60b-4b4f-bd82-94ec81110766')
         self.patchobject(neutronV20, 'find_resourceid_by_name_or_id',
                          return_value='fc68ea2c-b60b-4b4f-bd82-94ec81110766')
 
@@ -116,6 +128,47 @@ class NeutronSubnetTest(common.HeatTestCase):
                              stack)
         return rsrc
 
+    def _setup_mock(self, stack_name=None, use_deprecated_templ=False,
+                    tags=None):
+        if use_deprecated_templ:
+            t = template_format.parse(neutron_template_deprecated)
+        else:
+            t = template_format.parse(neutron_template)
+        if tags:
+            t['resources']['sub_net']['properties']['tags'] = tags
+        stack = utils.parse_stack(t, stack_name=stack_name)
+        sn = {
+            "subnet": {
+                "name": "name",
+                "network_id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766",
+                "tenant_id": "c1210485b2424d48804aad5d39c61b8f",
+                "allocation_pools": [
+                    {"start": "10.0.3.20", "end": "10.0.3.150"}],
+                "gateway_ip": "10.0.3.1",
+                'host_routes': [
+                    {'destination': u'10.0.4.0/24', 'nexthop': u'10.0.3.20'}],
+                "ip_version": 4,
+                "cidr": "10.0.3.0/24",
+                "dns_nameservers": ["8.8.8.8"],
+                "id": "91e47a57-7508-46fe-afc9-fc454e8580e1",
+                "enable_dhcp": True,
+            }
+        }
+        self.create_mock.return_value = sn
+        self.show_mock.side_effect = [
+            qe.NeutronClientException(status_code=404),
+            sn,
+            sn,
+            qe.NeutronClientException(status_code=404)
+        ]
+
+        self.delete_mock.side_effect = [
+            None,
+            qe.NeutronClientException(status_code=404)
+        ]
+
+        return t, stack
+
     def test_subnet(self):
         update_props = {'subnet': {
             'dns_nameservers': ['8.8.8.8', '192.168.1.254'],
@@ -124,19 +177,37 @@ class NeutronSubnetTest(common.HeatTestCase):
             'host_routes': [{'destination': '192.168.1.0/24',
                              'nexthop': '194.168.1.2'}],
             'gateway_ip': '10.0.3.105',
+            'tags': ['tag2', 'tag3'],
             'allocation_pools': [
                 {'start': '10.0.3.20', 'end': '10.0.3.100'},
                 {'start': '10.0.3.110', 'end': '10.0.3.200'}]}}
 
-        t, stack = self._test_subnet(u_props=update_props)
-        neutronclient.Client.update_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1', update_props)
+        t, stack = self._setup_mock(tags=['tag1', 'tag2'])
+        create_props = {'subnet': {
+            'name': utils.PhysName(stack.name, 'test_subnet'),
+            'network_id': 'fc68ea2c-b60b-4b4f-bd82-94ec81110766',
+            'dns_nameservers': [u'8.8.8.8'],
+            'allocation_pools': [
+                {'start': u'10.0.3.20', 'end': u'10.0.3.150'}],
+            'host_routes': [
+                {'destination': u'10.0.4.0/24', 'nexthop': u'10.0.3.20'}],
+            'ip_version': 4,
+            'cidr': u'10.0.3.0/24',
+            'tenant_id': 'c1210485b2424d48804aad5d39c61b8f',
+            'enable_dhcp': True}}
+
         self.patchobject(stack['net'], 'FnGetRefId',
                          return_value='fc68ea2c-b60b-4b4f-bd82-94ec81110766')
+        set_tag_mock = self.patchobject(neutronclient.Client, 'replace_tag')
         rsrc = self.create_subnet(t, stack, 'sub_net')
-        self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
+        self.create_mock.assert_called_once_with(create_props)
+        set_tag_mock.assert_called_once_with(
+            'subnets',
+            rsrc.resource_id,
+            {'tags': ['tag1', 'tag2']}
+        )
         rsrc.validate()
         ref_id = rsrc.FnGetRefId()
         self.assertEqual('91e47a57-7508-46fe-afc9-fc454e8580e1', ref_id)
@@ -153,10 +224,20 @@ class NeutronSubnetTest(common.HeatTestCase):
         update_snippet = rsrc_defn.ResourceDefinition(rsrc.name, rsrc.type(),
                                                       update_props['subnet'])
         rsrc.handle_update(update_snippet, {}, update_props['subnet'])
-
+        self.update_mock.assert_called_once_with(
+            '91e47a57-7508-46fe-afc9-fc454e8580e1',
+            update_props)
+        set_tag_mock.assert_called_with(
+            'subnets',
+            rsrc.resource_id,
+            {'tags': ['tag2', 'tag3']}
+        )
         # with name None
         del update_props['subnet']['name']
         rsrc.handle_update(update_snippet, {}, update_props['subnet'])
+        self.update_mock.assert_called_with(
+            '91e47a57-7508-46fe-afc9-fc454e8580e1',
+            update_props)
 
         # with no prop_diff
         rsrc.handle_update(update_snippet, {}, {})
@@ -164,7 +245,6 @@ class NeutronSubnetTest(common.HeatTestCase):
         self.assertIsNone(scheduler.TaskRunner(rsrc.delete)())
         rsrc.state_set(rsrc.CREATE, rsrc.COMPLETE, 'to delete again')
         self.assertIsNone(scheduler.TaskRunner(rsrc.delete)())
-        self.m.VerifyAll()
 
     def test_update_subnet_with_value_specs(self):
         update_props = {'subnet': {
@@ -177,11 +257,10 @@ class NeutronSubnetTest(common.HeatTestCase):
         update_props_merged['subnet']['enable_dhcp'] = True
         del update_props_merged['subnet']['value_specs']
 
-        t, stack = self._test_subnet(u_props=update_props_merged)
+        t, stack = self._setup_mock()
         self.patchobject(stack['net'], 'FnGetRefId',
                          return_value='fc68ea2c-b60b-4b4f-bd82-94ec81110766')
         rsrc = self.create_subnet(t, stack, 'sub_net')
-        self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
         rsrc.validate()
@@ -195,11 +274,13 @@ class NeutronSubnetTest(common.HeatTestCase):
         update_snippet = rsrc_defn.ResourceDefinition(rsrc.name, rsrc.type(),
                                                       update_props['subnet'])
         rsrc.handle_update(update_snippet, {}, update_props['subnet'])
-
+        self.update_mock.assert_called_once_with(
+            '91e47a57-7508-46fe-afc9-fc454e8580e1',
+            update_props
+        )
         self.assertIsNone(scheduler.TaskRunner(rsrc.delete)())
         rsrc.state_set(rsrc.CREATE, rsrc.COMPLETE, 'to delete again')
         self.assertIsNone(scheduler.TaskRunner(rsrc.delete)())
-        self.m.VerifyAll()
 
     def test_update_subnet_with_no_name(self):
         stack_name = utils.random_name()
@@ -209,11 +290,10 @@ class NeutronSubnetTest(common.HeatTestCase):
         update_props_name = {'subnet': {
             'name': utils.PhysName(stack_name, 'test_subnet'),
         }}
-        t, stack = self._test_subnet(stack_name, u_props=update_props_name)
+        t, stack = self._setup_mock(stack_name)
         self.patchobject(stack['net'], 'FnGetRefId',
                          return_value='fc68ea2c-b60b-4b4f-bd82-94ec81110766')
         rsrc = self.create_subnet(t, stack, 'sub_net')
-        self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
         rsrc.validate()
@@ -227,11 +307,14 @@ class NeutronSubnetTest(common.HeatTestCase):
         update_snippet = rsrc_defn.ResourceDefinition(rsrc.name, rsrc.type(),
                                                       update_props['subnet'])
         rsrc.handle_update(update_snippet, {}, update_props['subnet'])
+        self.update_mock.assert_called_once_with(
+            '91e47a57-7508-46fe-afc9-fc454e8580e1',
+            update_props_name
+        )
 
         self.assertIsNone(scheduler.TaskRunner(rsrc.delete)())
         rsrc.state_set(rsrc.CREATE, rsrc.COMPLETE, 'to delete again')
         self.assertIsNone(scheduler.TaskRunner(rsrc.delete)())
-        self.m.VerifyAll()
 
     def test_subnet_with_subnetpool(self):
         subnet_dict = {
@@ -252,32 +335,9 @@ class NeutronSubnetTest(common.HeatTestCase):
                 "tenant_id": "c1210485b2424d48804aad5d39c61b8f"
             }
         }
-        neutronclient.Client.create_subnet({
-            'subnet': {
-                'network_id': 'fc68ea2c-b60b-4b4f-bd82-94ec81110766',
-                'name': 'mysubnet',
-                'subnetpool_id': 'fc68ea2c-b60b-4b4f-bd82-94ec81110766',
-                'prefixlen': 24,
-                'dns_nameservers': [u'8.8.8.8'],
-                'allocation_pools': [
-                    {'start': u'10.0.3.20', 'end': u'10.0.3.150'}],
-                'host_routes': [
-                    {'destination': u'10.0.4.0/24', 'nexthop': u'10.0.3.20'}],
-                'ip_version': 4,
-                'enable_dhcp': True,
-                'tenant_id': 'c1210485b2424d48804aad5d39c61b8f'
-            }
-        }).AndReturn(subnet_dict)
-
-        neutronclient.Client.delete_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1'
-        ).AndReturn(None)
-
-        neutronclient.Client.show_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1'
-        ).AndRaise(qe.NeutronClientException(status_code=404))
-
-        self.m.ReplayAll()
+        self.create_mock.return_value = subnet_dict
+        self.show_mock.side_effect = [
+            qe.NeutronClientException(status_code=404)]
         t = template_format.parse(neutron_template)
         del t['resources']['sub_net']['properties']['cidr']
         t['resources']['sub_net']['properties'][
@@ -295,14 +355,52 @@ class NeutronSubnetTest(common.HeatTestCase):
         ref_id = rsrc.FnGetRefId()
         self.assertEqual('91e47a57-7508-46fe-afc9-fc454e8580e1', ref_id)
         scheduler.TaskRunner(rsrc.delete)()
-        self.m.VerifyAll()
 
+    def test_subnet_with_segment(self):
+        subnet_dict = {
+            "subnet": {
+                "allocation_pools": [
+                    {"start": "10.0.3.20", "end": "10.0.3.150"}],
+                "host_routes": [
+                    {"destination": "10.0.4.0/24", "nexthop": "10.0.3.20"}],
+                "segment_id": 'fc68ea2c-b60b-4b4f-bd82-94ec81110766',
+                "prefixlen": 24,
+                "dns_nameservers": ["8.8.8.8"],
+                "enable_dhcp": True,
+                "gateway_ip": "10.0.3.1",
+                "id": "91e47a57-7508-46fe-afc9-fc454e8580e1",
+                "ip_version": 4,
+                "name": "name",
+                "network_id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766",
+                "tenant_id": "c1210485b2424d48804aad5d39c61b8f"
+            }
+        }
+        self.create_mock.return_value = subnet_dict
+        self.show_mock.side_effect = [
+            qe.NeutronClientException(status_code=404)]
+        t = template_format.parse(neutron_template)
+        del t['resources']['sub_net']['properties']['cidr']
+        t['resources']['sub_net']['properties'][
+            'segment'] = 'fc68ea2c-b60b-4b4f-bd82-94ec81110766'
+        t['resources']['sub_net']['properties'][
+            'prefixlen'] = 24
+        t['resources']['sub_net']['properties'][
+            'name'] = 'mysubnet'
+        stack = utils.parse_stack(t)
+        self.patchobject(stack['net'], 'FnGetRefId',
+                         return_value='fc68ea2c-b60b-4b4f-bd82-94ec81110766')
+        rsrc = self.create_subnet(t, stack, 'sub_net')
+        scheduler.TaskRunner(rsrc.create)()
+        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
+        ref_id = rsrc.FnGetRefId()
+        self.assertEqual('91e47a57-7508-46fe-afc9-fc454e8580e1', ref_id)
+        scheduler.TaskRunner(rsrc.delete)()
+
     def test_subnet_deprecated(self):
-        t, stack = self._test_subnet(resolve_neutron=False)
+        t, stack = self._setup_mock(use_deprecated_templ=True)
         self.patchobject(stack['net'], 'FnGetRefId',
                          return_value='fc68ea2c-b60b-4b4f-bd82-94ec81110766')
         rsrc = self.create_subnet(t, stack, 'sub_net')
-        self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
         rsrc.validate()
@@ -320,119 +418,12 @@ class NeutronSubnetTest(common.HeatTestCase):
         self.assertIsNone(scheduler.TaskRunner(rsrc.delete)())
         rsrc.state_set(rsrc.CREATE, rsrc.COMPLETE, 'to delete again')
         self.assertIsNone(scheduler.TaskRunner(rsrc.delete)())
-        self.m.VerifyAll()
-
-    def _test_subnet(self, stack_name=None,
-                     resolve_neutron=True, u_props=None):
-        default_update_props = {'subnet': {
-            'dns_nameservers': ['8.8.8.8', '192.168.1.254'],
-            'name': 'mysubnet',
-            'enable_dhcp': True,
-            'host_routes': [{'destination': '192.168.1.0/24',
-                             'nexthop': '194.168.1.2'}]}}
-        update_props = u_props if u_props else default_update_props
-
-        if resolve_neutron:
-            t = template_format.parse(neutron_template)
-            # Update script
-            neutronclient.Client.update_subnet(
-                '91e47a57-7508-46fe-afc9-fc454e8580e1', update_props)
-
-        else:
-            t = template_format.parse(neutron_template_deprecated)
-        stack = utils.parse_stack(t, stack_name=stack_name)
-
-        neutronclient.Client.create_subnet({
-            'subnet': {
-                'name': utils.PhysName(stack.name, 'test_subnet'),
-                'network_id': 'fc68ea2c-b60b-4b4f-bd82-94ec81110766',
-                'dns_nameservers': [u'8.8.8.8'],
-                'allocation_pools': [
-                    {'start': u'10.0.3.20', 'end': u'10.0.3.150'}],
-                'host_routes': [
-                    {'destination': u'10.0.4.0/24', 'nexthop': u'10.0.3.20'}],
-                'ip_version': 4,
-                'cidr': u'10.0.3.0/24',
-                'tenant_id': 'c1210485b2424d48804aad5d39c61b8f',
-                'enable_dhcp': True
-            }
-        }).AndReturn({
-            "subnet": {
-                "allocation_pools": [
-                    {"start": "10.0.3.20", "end": "10.0.3.150"}],
-                "cidr": "10.0.3.0/24",
-                "dns_nameservers": ["8.8.8.8"],
-                "enable_dhcp": True,
-                "gateway_ip": "10.0.3.1",
-                "host_routes": [
-                    {"destination": "10.0.4.0/24", "nexthop": "10.0.3.20"}],
-                "id": "91e47a57-7508-46fe-afc9-fc454e8580e1",
-                "ip_version": 4,
-                "name": "name",
-                "network_id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766",
-                "tenant_id": "c1210485b2424d48804aad5d39c61b8f"
-            }
-        })
-        neutronclient.Client.show_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1').AndRaise(
-                qe.NeutronClientException(status_code=404))
-        sn = {
-            "subnet": {
-                "name": "name",
-                "network_id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766",
-                "tenant_id": "c1210485b2424d48804aad5d39c61b8f",
-                "allocation_pools": [
-                    {"start": "10.0.3.20", "end": "10.0.3.150"}],
-                "gateway_ip": "10.0.3.1",
-                'host_routes': [
-                    {'destination': u'10.0.4.0/24', 'nexthop': u'10.0.3.20'}],
-                "ip_version": 4,
-                "cidr": "10.0.3.0/24",
-                "dns_nameservers": ["8.8.8.8"],
-                "id": "91e47a57-7508-46fe-afc9-fc454e8580e1",
-                "enable_dhcp": True,
-            }
-        }
-        neutronclient.Client.show_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1').AndReturn(sn)
-        neutronclient.Client.show_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1').AndReturn(sn)
-
-        # Delete script
-        neutronclient.Client.delete_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1'
-        ).AndReturn(None)
-
-        neutronclient.Client.show_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1'
-        ).AndRaise(qe.NeutronClientException(status_code=404))
-
-        neutronclient.Client.delete_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1'
-        ).AndRaise(qe.NeutronClientException(status_code=404))
-
-        return t, stack
 
     def test_subnet_disable_dhcp(self):
         t = template_format.parse(neutron_template)
         t['resources']['sub_net']['properties']['enable_dhcp'] = 'False'
         stack = utils.parse_stack(t)
-
-        neutronclient.Client.create_subnet({
-            'subnet': {
-                'name': utils.PhysName(stack.name, 'test_subnet'),
-                'network_id': u'fc68ea2c-b60b-4b4f-bd82-94ec81110766',
-                'dns_nameservers': [u'8.8.8.8'],
-                'allocation_pools': [
-                    {'start': u'10.0.3.20', 'end': u'10.0.3.150'}],
-                'host_routes': [
-                    {'destination': u'10.0.4.0/24', 'nexthop': u'10.0.3.20'}],
-                'ip_version': 4,
-                'enable_dhcp': False,
-                'cidr': u'10.0.3.0/24',
-                'tenant_id': 'c1210485b2424d48804aad5d39c61b8f'
-            }
-        }).AndReturn({
+        subnet_info = {
             "subnet": {
                 "allocation_pools": [
                     {"start": "10.0.3.20", "end": "10.0.3.150"}],
@@ -448,42 +439,18 @@ class NeutronSubnetTest(common.HeatTestCase):
                 "network_id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766",
                 "tenant_id": "c1210485b2424d48804aad5d39c61b8f"
             }
-        })
+        }
+        self.create_mock.return_value = subnet_info
 
-        neutronclient.Client.show_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1').AndReturn({
-                "subnet": {
-                    "name": "name",
-                    "network_id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766",
-                    "tenant_id": "c1210485b2424d48804aad5d39c61b8f",
-                    "allocation_pools": [
-                        {"start": "10.0.3.20", "end": "10.0.3.150"}],
-                    "host_routes": [
-                        {"destination": "10.0.4.0/24",
-                         "nexthop": "10.0.3.20"}],
-                    "gateway_ip": "10.0.3.1",
-                    "ip_version": 4,
-                    "cidr": "10.0.3.0/24",
-                    "dns_nameservers": ["8.8.8.8"],
-                    "id": "91e47a57-7508-46fe-afc9-fc454e8580e1",
-                    "enable_dhcp": False,
-                }
-            })
+        self.show_mock.side_effect = [
+            subnet_info,
+            qe.NeutronClientException(status_code=404)
+        ]
 
-        neutronclient.Client.delete_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1'
-        ).AndReturn(None)
-
-        neutronclient.Client.show_subnet(
-            '91e47a57-7508-46fe-afc9-fc454e8580e1'
-        ).AndRaise(qe.NeutronClientException(status_code=404))
-
-        self.m.ReplayAll()
         self.patchobject(stack['net'], 'FnGetRefId',
                          return_value='fc68ea2c-b60b-4b4f-bd82-94ec81110766')
         rsrc = self.create_subnet(t, stack, 'sub_net')
 
-        self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
         rsrc.validate()
@@ -492,7 +459,6 @@ class NeutronSubnetTest(common.HeatTestCase):
         self.assertEqual('91e47a57-7508-46fe-afc9-fc454e8580e1', ref_id)
         self.assertIs(False, rsrc.FnGetAtt('enable_dhcp'))
         scheduler.TaskRunner(rsrc.delete)()
-        self.m.VerifyAll()
 
     def test_null_gateway_ip(self):
         p = {}
@@ -546,8 +512,7 @@ class NeutronSubnetTest(common.HeatTestCase):
         props['cidr'] = 'fdfa:6a50:d22b::/64'
         props['dns_nameservers'] = ['2001:4860:4860::8844']
         stack = utils.parse_stack(t)
-
-        neutronclient.Client.create_subnet({
+        create_info = {
             'subnet': {
                 'name': utils.PhysName(stack.name, 'test_subnet'),
                 'network_id': 'fc68ea2c-b60b-4b4f-bd82-94ec81110766',
@@ -559,33 +524,19 @@ class NeutronSubnetTest(common.HeatTestCase):
                 'ipv6_address_mode': 'slaac',
                 'ipv6_ra_mode': 'slaac'
             }
-        }).AndReturn({
-            "subnet": {
-                "allocation_pools": [
-                    {"start": "fdfa:6a50:d22b::2",
-                     "end": "fdfa:6a50:d22b:0:ffff:ffff:ffff:fffe"}],
-                "cidr": "fd00:1::/64",
-                "enable_dhcp": True,
-                "gateway_ip": "fdfa:6a50:d22b::1",
-                "id": "91e47a57-7508-46fe-afc9-fc454e8580e1",
-                "ip_version": 6,
-                "name": "name",
-                "network_id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766",
-                "tenant_id": "c1210485b2424d48804aad5d39c61b8f",
-                'ipv6_address_mode': 'slaac',
-                'ipv6_ra_mode': 'slaac'
-            }
-        })
+        }
+        subnet_info = copy.deepcopy(create_info)
+        subnet_info['subnet']['id'] = "91e47a57-7508-46fe-afc9-fc454e8580e1"
+        self.create_mock.return_value = subnet_info
 
-        self.m.ReplayAll()
         self.patchobject(stack['net'], 'FnGetRefId',
                          return_value='fc68ea2c-b60b-4b4f-bd82-94ec81110766')
         rsrc = self.create_subnet(t, stack, 'sub_net')
 
         scheduler.TaskRunner(rsrc.create)()
+        self.create_mock.assert_called_once_with(create_info)
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
         rsrc.validate()
-        self.m.VerifyAll()
 
     def test_host_routes_validate_destination(self):
         t = template_format.parse(neutron_template)
@@ -650,7 +601,6 @@ class NeutronSubnetTest(common.HeatTestCase):
         msg = ("Cannot define the following properties at the same time: "
                "subnetpool, cidr.")
         self.assertEqual(msg, six.text_type(ex))
-        self.m.VerifyAll()
 
     def test_validate_none_subnetpool_cidr(self):
         t = template_format.parse(neutron_template)
@@ -735,11 +685,79 @@ class NeutronSubnetTest(common.HeatTestCase):
         """
         t = template_format.parse(template)
         stack = utils.parse_stack(t)
-        self.patchobject(stack['net'], 'FnGetRefId',
-                         return_value='fc68ea2c-b60b-4b4f-bd82-94ec81110766')
+        rsrc = stack['subnet']
+        nd = {'reference_id': 'fc68ea2c-b60b-4b4f-bd82-94ec81110766'}
+        stk_defn.update_resource_data(stack.defn, 'net',
+                                      node_data.NodeData.from_dict(nd))
+        self.create_mock.return_value = {
+            "subnet": {
+                "id": "91e47a57-7508-46fe-afc9-fc454e8580e1",
+                "ip_version": 4,
+                "name": "name",
+                "network_id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766",
+                "tenant_id": "c1210485b2424d48804aad5d39c61b8f"
+            }
+        }
+        stack.create()
+
+        self.assertEqual(hot_funcs.GetResource(stack.defn, 'get_resource',
+                                               'net'),
+                         rsrc.properties.get('network'))
+        self.assertIsNone(rsrc.properties.get('network_id'))
+
+    def test_subnet_get_live_state(self):
+        template = """
+        heat_template_version: 2015-04-30
+        resources:
+          net:
+            type: OS::Neutron::Net
+            properties:
+              name: test
+          subnet:
+            type: OS::Neutron::Subnet
+            properties:
+              network_id: { get_resource: net }
+              cidr: 10.0.0.0/25
+              value_specs:
+                test_value_spec: value_spec_value
+        """
+        t = template_format.parse(template)
+        stack = utils.parse_stack(t)
         rsrc = stack['subnet']
         stack.create()
 
-        self.assertEqual(hot_funcs.GetResource(stack, 'get_resource', 'net'),
-                         rsrc.properties.get('network'))
-        self.assertIsNone(rsrc.properties.get('network_id'))
+        subnet_resp = {'subnet': {
+            'name': 'subnet-subnet-la5usdgifhrd',
+            'enable_dhcp': True,
+            'network_id': 'dffd43b3-6206-4402-87e6-8a16ddf3bd68',
+            'tenant_id': '30f466e3d14b4251853899f9c26e2b66',
+            'dns_nameservers': [],
+            'ipv6_ra_mode': None,
+            'allocation_pools': [{'start': '10.0.0.2', 'end': '10.0.0.126'}],
+            'gateway_ip': '10.0.0.1',
+            'ipv6_address_mode': None,
+            'ip_version': 4,
+            'host_routes': [],
+            'prefixlen': None,
+            'cidr': '10.0.0.0/25',
+            'id': 'b255342b-31b7-4674-8ea4-a144bca658b0',
+            'subnetpool_id': None,
+            'test_value_spec': 'value_spec_value'}
+        }
+        rsrc.client().show_subnet = mock.MagicMock(return_value=subnet_resp)
+        rsrc.resource_id = '1234'
+
+        reality = rsrc.get_live_state(rsrc.properties)
+        expected = {
+            'name': 'subnet-subnet-la5usdgifhrd',
+            'enable_dhcp': True,
+            'dns_nameservers': [],
+            'allocation_pools': [{'start': '10.0.0.2', 'end': '10.0.0.126'}],
+            'gateway_ip': '10.0.0.1',
+            'host_routes': [],
+            'value_specs': {'test_value_spec': 'value_spec_value'}
+        }
+
+        self.assertEqual(set(expected.keys()), set(reality.keys()))
+        for key in expected:
+            self.assertEqual(expected[key], reality[key])
