@@ -134,18 +134,18 @@ class VPNServiceTest(common.HeatTestCase):
     def create_vpnservice(self, resolve_neutron=True, resolve_router=True):
         self.stub_SubnetConstraint_validate()
         self.stub_RouterConstraint_validate()
-        neutronV20.find_resourceid_by_name_or_id(
-            mox.IsA(neutronclient.Client),
-            'subnet',
-            'sub123',
-            cmd_resource=None,
-        ).AndReturn('sub123')
         neutronV20.find_resourceid_by_name_or_id(
             mox.IsA(neutronclient.Client),
             'router',
             'rou123',
             cmd_resource=None,
-        ).AndReturn('rou123')
+        ).MultipleTimes().AndReturn('rou123')
+        neutronV20.find_resourceid_by_name_or_id(
+            mox.IsA(neutronclient.Client),
+            'subnet',
+            'sub123',
+            cmd_resource=None,
+        ).MultipleTimes().AndReturn('sub123')
         if resolve_neutron:
             snippet = template_format.parse(vpnservice_template)
         else:
@@ -454,8 +454,9 @@ class IPsecSiteConnectionTest(common.HeatTestCase):
             'con123', {'ipsec_site_connection': {'admin_state_up': False}})
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
-        update_template = copy.deepcopy(rsrc.t)
-        update_template['Properties']['admin_state_up'] = False
+        props = dict(rsrc.properties)
+        props['admin_state_up'] = False
+        update_template = rsrc.t.freeze(properties=props)
         scheduler.TaskRunner(rsrc.update, update_template)()
         self.m.VerifyAll()
 
@@ -593,13 +594,19 @@ class IKEPolicyTest(common.HeatTestCase):
 
     def test_update(self):
         rsrc = self.create_ikepolicy()
-        neutronclient.Client.update_ikepolicy('ike123',
-                                              {'ikepolicy': {
-                                                  'name': 'New IKEPolicy'}})
+        update_body = {
+            'ikepolicy': {
+                'name': 'New IKEPolicy',
+                'auth_algorithm': 'sha512'
+            }
+        }
+        neutronclient.Client.update_ikepolicy('ike123', update_body)
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
-        update_template = copy.deepcopy(rsrc.t)
-        update_template['Properties']['name'] = 'New IKEPolicy'
+        props = dict(rsrc.properties)
+        props['name'] = 'New IKEPolicy'
+        props['auth_algorithm'] = 'sha512'
+        update_template = rsrc.t.freeze(properties=props)
         scheduler.TaskRunner(rsrc.update, update_template)()
         self.m.VerifyAll()
 
@@ -743,6 +750,8 @@ class IPsecPolicyTest(common.HeatTestCase):
         self.m.ReplayAll()
         scheduler.TaskRunner(rsrc.create)()
         update_template = copy.deepcopy(rsrc.t)
-        update_template['Properties']['name'] = 'New IPsecPolicy'
+        props = dict(rsrc.properties)
+        props['name'] = 'New IPsecPolicy'
+        update_template = rsrc.t.freeze(properties=props)
         scheduler.TaskRunner(rsrc.update, update_template)()
         self.m.VerifyAll()
