@@ -26,6 +26,7 @@ from heat.db.sqlalchemy import models
 from heat.engine import api
 from heat.engine.cfn import parameters as cfn_param
 from heat.engine import event
+from heat.engine import parent_rsrc
 from heat.engine import stack as parser
 from heat.engine import template
 from heat.objects import event as event_object
@@ -58,13 +59,15 @@ class FormatTest(common.HeatTestCase):
 
     def _dummy_event(self, res_properties=None):
         resource = self.stack['generic1']
+        ev_uuid = 'abc123yc-9f88-404d-a85b-531529456xyz'
         ev = event.Event(self.context, self.stack, 'CREATE',
                          'COMPLETE', 'state changed',
                          'z3455xyc-9f88-404d-a85b-5315293e67de',
                          res_properties, resource.name, resource.type(),
-                         uuid='abc123yc-9f88-404d-a85b-531529456xyz')
-        event_id = ev.store()
-        return event_object.Event.get_by_id(self.context, event_id)
+                         uuid=ev_uuid)
+        ev.store()
+        return event_object.Event.get_all_by_stack(
+            self.context, self.stack.id, filters={'uuid': ev_uuid})[0]
 
     def test_format_stack_resource(self):
         self.stack.created_time = datetime(2015, 8, 3, 17, 5, 1)
@@ -282,7 +285,8 @@ class FormatTest(common.HeatTestCase):
 
     def test_format_stack_resource_with_parent_stack(self):
         res = self.stack['generic1']
-        res.stack.parent_resource_name = 'foobar'
+        res.stack.defn._parent_info = parent_rsrc.ParentResourceProxy(
+            self.stack.context, 'foobar', None)
 
         formatted = api.format_stack_resource(res, False)
         self.assertEqual('foobar', formatted[rpc_api.RES_PARENT_RESOURCE])
@@ -319,7 +323,8 @@ class FormatTest(common.HeatTestCase):
         resource._update_stored_properties()
         resource.store()
         event = self._dummy_event(res_properties=resource._rsrc_prop_data)
-        formatted = api.format_event(event, self.stack.identifier())
+        formatted = api.format_event(event, self.stack.identifier(),
+                                     include_rsrc_prop_data=True)
         self.assertEqual({'k1': 'v1'}, formatted[rpc_api.EVENT_RES_PROPERTIES])
 
     def test_format_event_legacy_prop_data(self):
@@ -329,7 +334,8 @@ class FormatTest(common.HeatTestCase):
             models.Event).filter_by(id=event.id).first()
         db_obj.update({'resource_properties': {'legacy_k1': 'legacy_v1'}})
         db_obj.save(self.stack.context.session)
-        event_legacy = event_object.Event.get_by_id(self.context, event.id)
+        event_legacy = event_object.Event.get_all_by_stack(self.context,
+                                                           self.stack.id)[0]
         formatted = api.format_event(event_legacy, self.stack.identifier())
         self.assertEqual({'legacy_k1': 'legacy_v1'},
                          formatted[rpc_api.EVENT_RES_PROPERTIES])
@@ -455,6 +461,7 @@ class FormatTest(common.HeatTestCase):
         stack.status = 'COMPLETE'
         stack['generic'].action = 'CREATE'
         stack['generic'].status = 'COMPLETE'
+        stack._update_all_resource_data(False, True)
         info = api.format_stack_outputs(stack.outputs, resolve_value=True)
         expected = [{'description': 'No description given',
                      'output_error': 'The Referenced Attribute (generic Bar) '
@@ -499,6 +506,40 @@ class FormatTest(common.HeatTestCase):
         self.assertEqual(expected, sorted(info, key=lambda k: k['output_key'],
                                           reverse=True))
 
+    def test_format_stack_params_csv(self):
+        tmpl = template.Template({
+            'heat_template_version': '2013-05-23',
+            'parameters': {
+                'foo': {
+                    'type': 'comma_delimited_list',
+                    'default': ['bar', 'baz']
+                },
+            }
+        })
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             tmpl, stack_id=str(uuid.uuid4()))
+        info = api.format_stack(stack)
+
+        # Should be 'bar,baz' NOT "[u'bar', u'baz']"
+        self.assertEqual('bar,baz', info['parameters']['foo'])
+
+    def test_format_stack_params_json(self):
+        tmpl = template.Template({
+            'heat_template_version': '2013-05-23',
+            'parameters': {
+                'foo': {
+                    'type': 'json',
+                    'default': {'bar': 'baz'}
+                },
+            }
+        })
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             tmpl, stack_id=str(uuid.uuid4()))
+        info = api.format_stack(stack)
+
+        # Should be '{"bar": "baz"}' NOT "{u'bar': u'baz'}"
+        self.assertEqual('{"bar": "baz"}', info['parameters']['foo'])
+
 
 class FormatValidateParameterTest(common.HeatTestCase):
 
