@@ -35,6 +35,7 @@ from heat.engine import service
 from heat.engine import stack as parser
 from heat.engine import template as templatem
 from heat.objects import stack as stack_object
+from heat.rpc import api as rpc_api
 from heat.tests import common
 from heat.tests.engine import tools
 from heat.tests import generic_resource as generic_rsrc
@@ -208,7 +209,7 @@ class StackConvergenceServiceCreateUpdateTest(common.HeatTestCase):
 
     def setUp(self):
         super(StackConvergenceServiceCreateUpdateTest, self).setUp()
-        cfg.CONF.set_override('convergence_engine', True, enforce_type=True)
+        cfg.CONF.set_override('convergence_engine', True)
         self.ctx = utils.dummy_context()
         self.man = service.EngineService('a-host', 'a-topic')
         self.man.thread_group_mgr = tools.DummyThreadGroupManager()
@@ -230,6 +231,7 @@ class StackConvergenceServiceCreateUpdateTest(common.HeatTestCase):
                                 template=tools.string_template_five,
                                 convergence=True)
 
+        stack.converge = None
         self.m.StubOutWithMock(templatem, 'Template')
         self.m.StubOutWithMock(environment, 'Environment')
         self.m.StubOutWithMock(parser, 'Stack')
@@ -295,14 +297,16 @@ class StackConvergenceServiceCreateUpdateTest(common.HeatTestCase):
                      convergence=old_stack.convergence,
                      current_traversal=old_stack.current_traversal,
                      prev_raw_template_id=old_stack.prev_raw_template_id,
-                     current_deps=old_stack.current_deps).AndReturn(stack)
+                     current_deps=old_stack.current_deps,
+                     converge=False).AndReturn(stack)
 
         self.m.StubOutWithMock(stack, 'validate')
         stack.validate().AndReturn(None)
 
         self.m.ReplayAll()
 
-        api_args = {'timeout_mins': 60, 'disable_rollback': False}
+        api_args = {'timeout_mins': 60, 'disable_rollback': False,
+                    rpc_api.PARAM_CONVERGE: False}
         result = self.man.update_stack(self.ctx, old_stack.identifier(),
                                        template, params, None, api_args)
         self.assertTrue(old_stack.convergence)
@@ -710,7 +714,7 @@ class StackServiceTest(common.HeatTestCase):
 
     @tools.stack_context('service_export_stack')
     def test_export_stack(self):
-        cfg.CONF.set_override('enable_stack_abandon', True, enforce_type=True)
+        cfg.CONF.set_override('enable_stack_abandon', True)
         self.m.StubOutWithMock(parser.Stack, 'load')
         parser.Stack.load(self.ctx,
                           stack=mox.IgnoreArg()).AndReturn(self.stack)
@@ -743,7 +747,7 @@ class StackServiceTest(common.HeatTestCase):
 
     @tools.stack_context('service_abandon_stack')
     def test_abandon_stack(self):
-        cfg.CONF.set_override('enable_stack_abandon', True, enforce_type=True)
+        cfg.CONF.set_override('enable_stack_abandon', True)
         self.m.StubOutWithMock(parser.Stack, 'load')
         parser.Stack.load(self.ctx,
                           stack=mox.IgnoreArg()).AndReturn(self.stack)
@@ -1213,7 +1217,7 @@ class StackServiceTest(common.HeatTestCase):
 
     @mock.patch.object(stack_object.Stack, 'count_all')
     def test_validate_new_stack_checks_stack_limit(self, mock_db_count):
-        cfg.CONF.set_override('max_stacks_per_tenant', 99, enforce_type=True)
+        cfg.CONF.set_override('max_stacks_per_tenant', 99)
         mock_db_count.return_value = 99
         template = templatem.Template(
             {'HeatTemplateFormatVersion': '2012-12-12'})
@@ -1248,7 +1252,7 @@ class StackServiceTest(common.HeatTestCase):
         self.assertEqual(msg, six.text_type(ex))
 
     def test_validate_new_stack_checks_resource_limit(self):
-        cfg.CONF.set_override('max_resources_per_stack', 5, enforce_type=True)
+        cfg.CONF.set_override('max_resources_per_stack', 5)
         template = {'HeatTemplateFormatVersion': '2012-12-12',
                     'Resources': {
                         'Res1': {'Type': 'GenericResource1'},
@@ -1337,9 +1341,7 @@ class StackServiceTest(common.HeatTestCase):
             mock.call(self.ctx, 'bar'),
         ])
         mock_stack_load.assert_called_once_with(self.ctx,
-                                                stack=db_stack,
-                                                service_check_defer=True,
-                                                resource_validate=False)
+                                                stack=db_stack)
         self.assertTrue(lock2.release.called)
         reason = ('Engine went down during stack %s' % fake_stack.action)
         mock_thread.start_with_acquired_lock.assert_called_once_with(
@@ -1348,7 +1350,7 @@ class StackServiceTest(common.HeatTestCase):
         )
 
     def test_parse_adopt_stack_data_without_parameters(self):
-        cfg.CONF.set_override('enable_stack_adopt', True, enforce_type=True)
+        cfg.CONF.set_override('enable_stack_adopt', True)
         template = {"heat_template_version": "2015-04-30",
                     "resources": {
                         "myres": {
@@ -1373,7 +1375,7 @@ class StackServiceTest(common.HeatTestCase):
             self.ctx, 'stack_name', template, {}, {}, None, args)
 
     def test_parse_adopt_stack_data_with_parameters(self):
-        cfg.CONF.set_override('enable_stack_adopt', True, enforce_type=True)
+        cfg.CONF.set_override('enable_stack_adopt', True)
         template = {"heat_template_version": "2015-04-30",
                     "parameters": {
                         "volsize": {"type": "number"}
