@@ -47,6 +47,8 @@ class EnvironmentTest(common.HeatTestCase):
                     u'event_sinks': [],
                     u'resource_registry': {u'resources': {}}}
         env = environment.Environment(old)
+        self.assertEqual(expected, env.env_as_dict())
+        del(expected['encrypted_param_names'])
         self.assertEqual(expected, env.user_env_as_dict())
 
     def test_load_new_env(self):
@@ -57,6 +59,8 @@ class EnvironmentTest(common.HeatTestCase):
                    u'resource_registry': {u'OS::Food': u'fruity.yaml',
                                           u'resources': {}}}
         env = environment.Environment(new_env)
+        self.assertEqual(new_env, env.env_as_dict())
+        del(new_env['encrypted_param_names'])
         self.assertEqual(new_env, env.user_env_as_dict())
 
     def test_global_registry(self):
@@ -155,7 +159,7 @@ class EnvironmentTest(common.HeatTestCase):
                             'b.yaml',
                             path=['resources', 'res_x', 'test::two'])
 
-        self.assertEqual(env.user_env_as_dict(), env2.user_env_as_dict())
+        self.assertEqual(env.env_as_dict(), env2.env_as_dict())
 
     def test_constraints(self):
         env = environment.Environment({})
@@ -166,7 +170,7 @@ class EnvironmentTest(common.HeatTestCase):
         env.register_constraint("constraint2", second_constraint)
         self.assertIs(first_constraint, env.get_constraint("constraint1"))
         self.assertIs(second_constraint, env.get_constraint("constraint2"))
-        self.assertIs(None, env.get_constraint("no_constraint"))
+        self.assertIsNone(env.get_constraint("no_constraint"))
 
     def test_constraints_registry(self):
         constraint_content = '''
@@ -188,7 +192,7 @@ def constraint_mapping():
 
         self.assertEqual("MyConstraint",
                          env.get_constraint("constraint1").__name__)
-        self.assertIs(None, env.get_constraint("no_constraint"))
+        self.assertIsNone(env.get_constraint("no_constraint"))
 
     def test_constraints_registry_error(self):
         constraint_content = '''
@@ -213,7 +217,7 @@ def constraint_mapping():
 
         self.assertEqual("FlavorConstraint",
                          env.get_constraint("nova.flavor").__name__)
-        self.assertIs(None, env.get_constraint("no_constraint"))
+        self.assertIsNone(env.get_constraint("no_constraint"))
 
     def test_event_sinks(self):
         env = environment.Environment(
@@ -514,7 +518,7 @@ class ChildEnvTest(common.HeatTestCase):
                     'event_sinks': [],
                     'resource_registry': {'resources': {}}}
         cenv = environment.get_child_environment(penv, new_params)
-        self.assertEqual(expected, cenv.user_env_as_dict())
+        self.assertEqual(expected, cenv.env_as_dict())
 
     def test_params_normal(self):
         new_params = {'parameters': {'foo': 'bar', 'tester': 'Yes'}}
@@ -525,7 +529,7 @@ class ChildEnvTest(common.HeatTestCase):
                     'resource_registry': {'resources': {}}}
         expected.update(new_params)
         cenv = environment.get_child_environment(penv, new_params)
-        self.assertEqual(expected, cenv.user_env_as_dict())
+        self.assertEqual(expected, cenv.env_as_dict())
 
     def test_params_parent_overwritten(self):
         new_params = {'parameters': {'foo': 'bar', 'tester': 'Yes'}}
@@ -537,7 +541,7 @@ class ChildEnvTest(common.HeatTestCase):
                     'resource_registry': {'resources': {}}}
         expected.update(new_params)
         cenv = environment.get_child_environment(penv, new_params)
-        self.assertEqual(expected, cenv.user_env_as_dict())
+        self.assertEqual(expected, cenv.env_as_dict())
 
     def test_registry_merge_simple(self):
         env1 = {u'resource_registry': {u'OS::Food': u'fruity.yaml'}}
@@ -587,7 +591,7 @@ class ChildEnvTest(common.HeatTestCase):
         rr = cenv.user_env_as_dict()['resource_registry']
         self.assertIn('OS::Food', rr)
         self.assertNotIn('OS::Food', rr['resources']['abc'])
-        # make sure the parent env is uneffected
+        # make sure the parent env is unaffected
         innocent2 = penv.get_resource_info('OS::Food', resource_name='abc')
         self.assertEqual(['resources', 'abc', 'OS::Food'], innocent2.path)
 
@@ -834,7 +838,8 @@ class ResourceRegistryTest(common.HeatTestCase):
             generic_resource.GenericResource,
             'is_service_available',
             side_effect=exception.ClientNotAvailable(client_name='generic'))
-        types = registry.get_types(utils.dummy_context())
+        types = registry.get_types(utils.dummy_context(),
+                                   type_name='GenericResourceType')
         self.assertNotIn('GenericResourceType', types)
 
     def test_list_type_with_invalid_type_name(self):
