@@ -579,7 +579,7 @@ class ProviderTemplateTest(common.HeatTestCase):
         temp_res = template_resource.TemplateResource('test_t_res',
                                                       definition, stack)
         self.assertEqual('test_resource.template',
-                         temp_res.template_name)
+                         temp_res.template_url)
 
     def test_resource_info_special(self):
         provider = {
@@ -614,7 +614,7 @@ class ProviderTemplateTest(common.HeatTestCase):
         temp_res = template_resource.TemplateResource('foo',
                                                       definition, stack)
         self.assertEqual('foo.template',
-                         temp_res.template_name)
+                         temp_res.template_url)
 
     def test_get_error_for_invalid_template_name(self):
         # assertion: if the name matches {.yaml|.template} and is valid
@@ -908,7 +908,8 @@ class ProviderTemplateTest(common.HeatTestCase):
                                                       stack)
         err = self.assertRaises(exception.StackValidationFailed,
                                 temp_res.validate)
-        self.assertIn('Error parsing template: ', six.text_type(err))
+        self.assertIn('Error parsing template http://heatr/bad_tmpl.yaml',
+                      six.text_type(err))
 
         self.m.VerifyAll()
 
