@@ -20,7 +20,6 @@ Unit Tests for heat.rpc.client
 import copy
 
 import mock
-from mox import stubout
 from oslo_messaging._drivers import common as rpc_common
 from oslo_utils import reflection
 
@@ -35,7 +34,6 @@ class EngineRpcAPITestCase(common.HeatTestCase):
 
     def setUp(self):
         super(EngineRpcAPITestCase, self).setUp()
-        self.stubs = stubout.StubOutForTesting()
         self.identity = dict(identifier.HeatIdentifier('engine_test_tenant',
                                                        '6',
                                                        'wordpress'))
@@ -58,6 +56,24 @@ class EngineRpcAPITestCase(common.HeatTestCase):
                          reflection.get_class_name(exr, fully_qualified=False))
         self.assertEqual('NotFound', self.rpcapi.local_error_name(exr))
 
+    def test_ignore_error_by_name(self):
+        ex = exception.NotFound()
+        exr = self._to_remote_error(ex)
+
+        filter_exc = self.rpcapi.ignore_error_by_name('NotFound')
+
+        with filter_exc:
+            raise ex
+        with filter_exc:
+            raise exr
+
+        def should_raise(exc):
+            with self.rpcapi.ignore_error_by_name('NotSupported'):
+                raise exc
+
+        self.assertRaises(exception.NotFound, should_raise, ex)
+        self.assertRaises(exception.NotFound, should_raise, exr)
+
     def test_ignore_error_named(self):
         ex = exception.NotFound()
         exr = self._to_remote_error(ex)
@@ -87,8 +103,8 @@ class EngineRpcAPITestCase(common.HeatTestCase):
             expected_message = self.rpcapi.make_msg(method, **kwargs)
 
         cast_and_call = ['delete_stack']
-        if rpc_method == 'call' and method in cast_and_call:
-            kwargs['cast'] = False
+        if method in cast_and_call:
+            kwargs['cast'] = rpc_method != 'call'
 
         with mock.patch.object(self.rpcapi, rpc_method) as mock_rpc_method:
             mock_rpc_method.return_value = expected_retval
@@ -112,7 +128,6 @@ class EngineRpcAPITestCase(common.HeatTestCase):
             'marker': mock.ANY,
             'sort_dir': mock.ANY,
             'filters': mock.ANY,
-            'tenant_safe': mock.ANY,
             'show_deleted': mock.ANY,
             'show_nested': mock.ANY,
             'show_hidden': mock.ANY,
@@ -126,7 +141,6 @@ class EngineRpcAPITestCase(common.HeatTestCase):
     def test_count_stacks(self):
         default_args = {
             'filters': mock.ANY,
-            'tenant_safe': mock.ANY,
             'show_deleted': mock.ANY,
             'show_nested': mock.ANY,
             'show_hidden': mock.ANY,
@@ -166,18 +180,24 @@ class EngineRpcAPITestCase(common.HeatTestCase):
         call_kwargs['user_creds_id'] = None
         call_kwargs['stack_user_project_id'] = None
         call_kwargs['parent_resource_name'] = None
+        call_kwargs['template_id'] = None
         expected_message = self.rpcapi.make_msg('create_stack', **call_kwargs)
         kwargs['expected_message'] = expected_message
         self._test_engine_api('create_stack', 'call', **kwargs)
 
     def test_update_stack(self):
+        kwargs = dict(stack_identity=self.identity,
+                      template={u'Foo': u'bar'},
+                      params={u'InstanceType': u'm1.xlarge'},
+                      files={},
+                      environment_files=['foo.yaml'],
+                      args=mock.ANY)
+        call_kwargs = copy.deepcopy(kwargs)
+        call_kwargs['template_id'] = None
+        expected_message = self.rpcapi.make_msg('update_stack', **call_kwargs)
         self._test_engine_api('update_stack', 'call',
-                              stack_identity=self.identity,
-                              template={u'Foo': u'bar'},
-                              params={u'InstanceType': u'm1.xlarge'},
-                              files={},
-                              environment_files=['foo.yaml'],
-                              args=mock.ANY)
+                              expected_message=expected_message,
+                              **kwargs)
 
     def test_preview_update_stack(self):
         self._test_engine_api('preview_update_stack', 'call',
@@ -216,10 +236,12 @@ class EngineRpcAPITestCase(common.HeatTestCase):
                               support_status=None,
                               type_name=None,
                               heat_version=None,
-                              version='1.16')
+                              with_description=False,
+                              version='1.30')
 
     def test_resource_schema(self):
-        self._test_engine_api('resource_schema', 'call', type_name="TYPE")
+        self._test_engine_api('resource_schema', 'call', type_name="TYPE",
+                              with_description=False, version='1.30')
 
     def test_generate_template(self):
         self._test_engine_api('generate_template', 'call',
@@ -231,7 +253,8 @@ class EngineRpcAPITestCase(common.HeatTestCase):
                   'marker': None,
                   'sort_keys': None,
                   'sort_dir': None,
-                  'filters': None}
+                  'filters': None,
+                  'nested_depth': None}
         self._test_engine_api('list_events', 'call', **kwargs)
 
     def test_describe_stack_resource(self):
@@ -297,8 +320,7 @@ class EngineRpcAPITestCase(common.HeatTestCase):
 
     def test_list_software_configs(self):
         self._test_engine_api('list_software_configs', 'call',
-                              limit=mock.ANY, marker=mock.ANY,
-                              tenant_safe=mock.ANY)
+                              limit=mock.ANY, marker=mock.ANY)
 
     def test_show_software_config(self):
         self._test_engine_api('show_software_config', 'call',
@@ -328,6 +350,11 @@ class EngineRpcAPITestCase(common.HeatTestCase):
         self._test_engine_api('show_software_deployment', 'call',
                               deployment_id=deployment_id)
 
+    def test_check_software_deployment(self):
+        deployment_id = '86729f02-4648-44d8-af44-d0ec65b6abc9'
+        self._test_engine_api('check_software_deployment', 'call',
+                              deployment_id=deployment_id, timeout=100)
+
     def test_create_software_deployment(self):
         self._test_engine_api(
             'create_software_deployment', 'call',
@@ -401,3 +428,13 @@ class EngineRpcAPITestCase(common.HeatTestCase):
                               mark_unhealthy=True,
                               resource_status_reason="Any reason",
                               version='1.26')
+
+    def test_get_environment(self):
+        self._test_engine_api(
+            'get_environment', 'call', stack_identity=self.identity,
+            version='1.28')
+
+    def test_get_files(self):
+        self._test_engine_api(
+            'get_files', 'call', stack_identity=self.identity,
+            version='1.32')
