@@ -11,6 +11,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import contextlib
 import json
 import uuid
 
@@ -23,6 +24,7 @@ import six
 from heat.common import exception
 from heat.common import identifier
 from heat.common import template_format
+from heat.engine import node_data
 from heat.engine import resource
 from heat.engine.resources import stack_resource
 from heat.engine import stack as parser
@@ -30,6 +32,7 @@ from heat.engine import template as templatem
 from heat.objects import raw_template
 from heat.objects import stack as stack_object
 from heat.objects import stack_lock
+from heat.rpc import api as rpc_api
 from heat.tests import common
 from heat.tests import generic_resource as generic_rsrc
 from heat.tests import utils
@@ -77,7 +80,7 @@ main_template = '''
 heat_template_version: 2013-05-23
 resources:
   volume_server:
-    type: nested.yaml
+      type: file://tmp/nested.yaml
 '''
 
 my_wrong_nested_template = '''
@@ -203,7 +206,16 @@ class StackResourceTest(StackResourceBaseTest):
         self.assertEqual({}, ret)
 
     def test_abandon_nested_sends_rpc_abandon(self):
-        rpcc = mock.Mock()
+        rpcc = mock.MagicMock()
+
+        @contextlib.contextmanager
+        def exc_filter(*args):
+            try:
+                yield
+            except exception.NotFound:
+                pass
+
+        rpcc.ignore_error_by_name.side_effect = exc_filter
         self.parent_resource.rpc_client = rpcc
         self.parent_resource.resource_id = 'fake_id'
 
@@ -349,7 +361,8 @@ class StackResourceTest(StackResourceBaseTest):
         nest_stack = stk_resource._parse_nested_stack(
             "test_nest_stack", stk_resource.child_template(),
             stk_resource.child_params())
-        self.assertEqual(nest_stack._parent_stack, self.parent_stack)
+        self.assertEqual(self.parent_stack,
+                         nest_stack.parent_resource._stack())
 
     def test_preview_dict_validates_nested_resources(self):
         parent_t = self.parent_stack.t
@@ -384,14 +397,15 @@ class StackResourceTest(StackResourceBaseTest):
     def test_validate_error_reference(self):
         stack_name = 'validate_error_reference'
         tmpl = template_format.parse(main_template)
-        files = {'nested.yaml': my_wrong_nested_template}
+        files = {'file://tmp/nested.yaml': my_wrong_nested_template}
         stack = parser.Stack(utils.dummy_context(), stack_name,
                              templatem.Template(tmpl, files=files))
         rsrc = stack['volume_server']
-        raise_exc_msg = ('Failed to validate: resources.volume_server: '
-                         'The specified reference "instance" '
-                         '(in volume_attachment.Properties.instance_uuid) '
-                         'is incorrect.')
+        raise_exc_msg = ('InvalidTemplateReference: '
+                         'resources.volume_server<file://tmp/nested.yaml>: '
+                         'The specified reference "instance" (in '
+                         'volume_attachment.Properties.instance_uuid) is '
+                         'incorrect.')
         exc = self.assertRaises(exception.StackValidationFailed,
                                 rsrc.validate)
         self.assertEqual(raise_exc_msg, six.text_type(exc))
@@ -448,21 +462,20 @@ class StackResourceTest(StackResourceBaseTest):
     def test_get_attribute_autoscaling_convg(self):
         t = template_format.parse(heat_autoscaling_group_template)
         tmpl = templatem.Template(t)
-        cache_data = {'my_autoscaling_group': {
+        cache_data = {'my_autoscaling_group': node_data.NodeData.from_dict({
             'uuid': mock.ANY,
             'id': mock.ANY,
             'action': 'CREATE',
             'status': 'COMPLETE',
             'attrs': {'current_size': 4}
-        }}
+        })}
         stack = parser.Stack(utils.dummy_context(), 'test_att', tmpl,
                              cache_data=cache_data)
-        rsrc = stack['my_autoscaling_group']
-        self.assertEqual(4, rsrc.FnGetAtt(rsrc.CURRENT_SIZE))
+        rsrc = stack.defn['my_autoscaling_group']
+        self.assertEqual(4, rsrc.FnGetAtt('current_size'))
 
     def test__validate_nested_resources_checks_num_of_resources(self):
-        stack_resource.cfg.CONF.set_override('max_resources_per_stack', 2,
-                                             enforce_type=True)
+        stack_resource.cfg.CONF.set_override('max_resources_per_stack', 2)
         tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
                 'Resources': {
                     'r': {
@@ -508,8 +521,17 @@ class StackResourceTest(StackResourceBaseTest):
 
     def test_delete_nested_not_found_nested_stack(self):
         self.parent_resource.resource_id = 'fake_id'
-        rpcc = mock.Mock()
+        rpcc = mock.MagicMock()
         self.parent_resource.rpc_client = rpcc
+
+        @contextlib.contextmanager
+        def exc_filter(*args):
+            try:
+                yield
+            except exception.NotFound:
+                pass
+
+        rpcc.return_value.ignore_error_by_name.side_effect = exc_filter
         rpcc.return_value.delete_stack = mock.Mock(
             side_effect=exception.NotFound())
         self.assertIsNone(self.parent_resource.delete_nested())
@@ -910,9 +932,9 @@ class WithTemplateTest(StackResourceBaseTest):
         rpcc.return_value._create_stack.assert_called_once_with(
             self.ctx,
             stack_name=res_name,
-            args={'disable_rollback': True,
-                  'adopt_stack_data': adopt_data_str,
-                  'timeout_mins': self.timeout_mins},
+            args={rpc_api.PARAM_DISABLE_ROLLBACK: True,
+                  rpc_api.PARAM_ADOPT_STACK_DATA: adopt_data_str,
+                  rpc_api.PARAM_TIMEOUT: self.timeout_mins},
             environment_files=None,
             stack_user_project_id='aprojectid',
             parent_resource_name='test',
@@ -959,9 +981,9 @@ class WithTemplateTest(StackResourceBaseTest):
         rpcc.return_value._create_stack.assert_called_once_with(
             self.ctx,
             stack_name=res_name,
-            args={'disable_rollback': True,
-                  'adopt_stack_data': adopt_data_str,
-                  'timeout_mins': self.timeout_mins},
+            args={rpc_api.PARAM_DISABLE_ROLLBACK: True,
+                  rpc_api.PARAM_ADOPT_STACK_DATA: adopt_data_str,
+                  rpc_api.PARAM_TIMEOUT: self.timeout_mins},
             environment_files=None,
             stack_user_project_id='aprojectid',
             parent_resource_name='test',
@@ -1004,7 +1026,8 @@ class WithTemplateTest(StackResourceBaseTest):
             template=None,
             params=None,
             files=None,
-            args={'timeout_mins': self.timeout_mins})
+            args={rpc_api.PARAM_TIMEOUT: self.timeout_mins,
+                  rpc_api.PARAM_CONVERGE: False})
 
     def test_update_with_template_failure(self):
         class StackValidationFailed_Remote(exception.StackValidationFailed):
@@ -1040,7 +1063,8 @@ class WithTemplateTest(StackResourceBaseTest):
             template=None,
             params=None,
             files=None,
-            args={'timeout_mins': self.timeout_mins})
+            args={rpc_api.PARAM_TIMEOUT: self.timeout_mins,
+                  rpc_api.PARAM_CONVERGE: False})
         self.assertIsNotNone(template_id.match)
         self.assertRaises(exception.NotFound,
                           raw_template.RawTemplate.get_by_id,
