@@ -16,7 +16,6 @@ import hashlib
 import json
 
 import fixtures
-from oslotest import mockpatch
 import six
 from stevedore import extension
 
@@ -29,8 +28,10 @@ from heat.engine.clients.os import nova
 from heat.engine import environment
 from heat.engine import function
 from heat.engine.hot import template as hot_t
+from heat.engine import node_data
 from heat.engine import rsrc_defn
 from heat.engine import stack
+from heat.engine import stk_defn
 from heat.engine import template
 from heat.tests import common
 from heat.tests.openstack.nova import fakes as fakes_nova
@@ -109,7 +110,7 @@ class TemplatePluginFixture(fixtures.Fixture):
             template._template_classes = None
 
         clear_template_classes()
-        self.useFixture(mockpatch.PatchObject(
+        self.useFixture(fixtures.MockPatchObject(
             template,
             '_get_template_extension_manager',
             new=self._get_template_extension_manager))
@@ -142,6 +143,9 @@ class TestTemplatePluginManager(common.HeatTestCase):
             def add_resource(self, definition, name=None):
                 pass
 
+            def outputs(self, stack):
+                pass
+
             def __getitem__(self, section):
                 return {}
 
@@ -695,11 +699,12 @@ class TemplateTest(common.HeatTestCase):
 
     @staticmethod
     def resolve(snippet, template, stack=None):
-        return function.resolve(template.parse(stack, snippet))
+        return function.resolve(template.parse(stack and stack.defn, snippet))
 
     @staticmethod
     def resolve_condition(snippet, template, stack=None):
-        return function.resolve(template.parse_condition(stack, snippet))
+        return function.resolve(template.parse_condition(stack and stack.defn,
+                                                         snippet))
 
     def test_defaults(self):
         empty = template.Template(empty_template)
@@ -728,7 +733,8 @@ class TemplateTest(common.HeatTestCase):
                                     template.Template, invalid_hot_version_tmp)
         valid_versions = ['2013-05-23', '2014-10-16',
                           '2015-04-30', '2015-10-15', '2016-04-08',
-                          '2016-10-14', '2017-02-24', 'newton', 'ocata']
+                          '2016-10-14', '2017-02-24', '2017-09-01',
+                          'newton', 'ocata', 'pike']
         ex_error_msg = ('The template version is invalid: '
                         '"heat_template_version: 2012-12-12". '
                         '"heat_template_version" should be one of: %s'
@@ -888,7 +894,7 @@ class TemplateTest(common.HeatTestCase):
         tmpl = template.Template(parameter_template, env=env)
         stk = stack.Stack(self.ctx, 'test', tmpl)
         tmpl.env = environment.Environment({})
-        stk.parameters = cfn_p.CfnParameters(stk.identifier(), tmpl)
+        stk.defn.parameters = cfn_p.CfnParameters(stk.identifier(), tmpl)
         snippet = {"Ref": "foo"}
         self.assertRaises(exception.UserParameterMissing,
                           self.resolve,
@@ -897,21 +903,19 @@ class TemplateTest(common.HeatTestCase):
     def test_resource_refs(self):
         tmpl = template.Template(resource_template)
         stk = stack.Stack(self.ctx, 'test', tmpl)
+        stk.validate()
 
-        self.m.StubOutWithMock(stk['foo'], 'FnGetRefId')
-        stk['foo'].FnGetRefId().MultipleTimes().AndReturn('bar')
-        self.m.ReplayAll()
-
+        data = node_data.NodeData.from_dict({'reference_id': 'bar'})
+        stk_defn.update_resource_data(stk.defn, 'foo', data)
         r_snippet = {"Ref": "foo"}
         self.assertEqual("bar", self.resolve(r_snippet, tmpl, stk))
-        self.m.VerifyAll()
 
     def test_resource_refs_param(self):
         tmpl = template.Template(resource_template)
         stk = stack.Stack(self.ctx, 'test', tmpl)
 
         p_snippet = {"Ref": "baz"}
-        parsed = tmpl.parse(stk, p_snippet)
+        parsed = tmpl.parse(stk.defn, p_snippet)
         self.assertIsInstance(parsed, cfn_funcs.ParamRef)
 
     def test_select_from_list(self):
@@ -1306,13 +1310,18 @@ class TemplateTest(common.HeatTestCase):
             'parent', 'SomeType',
             deletion_policy=rsrc_defn.ResourceDefinition.RETAIN,
             update_policy={"blarg": "wibble"})
-
+        tmpl = copy.deepcopy(empty_template)
+        tmpl['Resources'] = {'parent': {'Type': 'SomeType',
+                                        'DeletionPolicy': 'Retain',
+                                        'UpdatePolicy': {"blarg": "wibble"}}}
         parent_resource.stack = stack.Stack(self.ctx, 'toplevel_stack',
-                                            template.Template(empty_template))
+                                            template.Template(tmpl))
+        parent_resource.stack._resources = {'parent': parent_resource}
+
         stk = stack.Stack(self.ctx, 'test_stack',
                           template.Template(empty_template),
                           parent_resource='parent', owner_id=45)
-        stk._parent_stack = dict(parent=parent_resource)
+        stk.set_parent_stack(parent_resource.stack)
         self.assertEqual({"foo": "bar"},
                          self.resolve(metadata_snippet, stk.t, stk))
         self.assertEqual('Retain',
@@ -1325,18 +1334,22 @@ class TemplateTest(common.HeatTestCase):
 
         parent_resource = DummyClass()
         parent_resource.metadata_set({"foo": "bar"})
-        parent_resource.stack = stack.Stack(self.ctx, 'toplevel_stack',
-                                            template.Template(empty_template))
-        del_policy = cfn_funcs.Join(parent_resource.stack,
+        del_policy = cfn_funcs.Join(None,
                                     'Fn::Join', ['eta', ['R', 'in']])
         parent_resource.t = rsrc_defn.ResourceDefinition(
             'parent', 'SomeType',
             deletion_policy=del_policy)
+        tmpl = copy.deepcopy(empty_template)
+        tmpl['Resources'] = {'parent': {'Type': 'SomeType',
+                                        'DeletionPolicy': del_policy}}
+        parent_resource.stack = stack.Stack(self.ctx, 'toplevel_stack',
+                                            template.Template(tmpl))
+        parent_resource.stack._resources = {'parent': parent_resource}
 
         stk = stack.Stack(self.ctx, 'test_stack',
                           template.Template(empty_template),
                           parent_resource='parent')
-        stk._parent_stack = dict(parent=parent_resource)
+        stk.set_parent_stack(parent_resource.stack)
         self.assertEqual('Retain',
                          self.resolve(deletion_policy_snippet, stk.t, stk))
 
@@ -1354,13 +1367,16 @@ class TemplateTest(common.HeatTestCase):
         parent_resource = DummyClass()
         parent_resource.metadata_set({"foo": "bar"})
         parent_resource.t = rsrc_defn.ResourceDefinition('parent', 'SomeType')
+        tmpl = copy.deepcopy(empty_template)
+        tmpl['Resources'] = {'parent': {'Type': 'SomeType'}}
 
         parent_resource.stack = stack.Stack(self.ctx, 'toplevel_stack',
-                                            template.Template(empty_template))
+                                            template.Template(tmpl))
+        parent_resource.stack._resources = {'parent': parent_resource}
         stk = stack.Stack(self.ctx, 'test_stack',
                           template.Template(empty_template),
                           parent_resource='parent', owner_id=78)
-        stk._parent_stack = dict(parent=parent_resource)
+        stk.set_parent_stack(parent_resource.stack)
         self.assertEqual('Delete', self.resolve(snippet, stk.t, stk))
 
     def test_prevent_parameters_access(self):
@@ -1414,6 +1430,23 @@ class TemplateTest(common.HeatTestCase):
 
         self.assertEqual(cfn_tpl['Resources'], empty.t['Resources'])
 
+    def test_add_output(self):
+        cfn_tpl = template_format.parse('''
+        AWSTemplateFormatVersion: 2010-09-09
+        Outputs:
+          output1:
+            Description: An output
+            Value: foo
+        ''')
+        source = template.Template(cfn_tpl)
+        empty = template.Template(copy.deepcopy(empty_template))
+        stk = stack.Stack(self.ctx, 'test_stack', source)
+
+        for defn in six.itervalues(source.outputs(stk)):
+            empty.add_output(defn)
+
+        self.assertEqual(cfn_tpl['Outputs'], empty.t['Outputs'])
+
     def test_create_empty_template_default_version(self):
         empty_template = template.Template.create_empty_template()
         self.assertEqual(hot_t.HOTemplate20150430, empty_template.__class__)
@@ -1622,7 +1655,7 @@ class ResolveDataTest(common.HeatTestCase):
                                  template.Template(empty_template))
 
     def resolve(self, snippet):
-        return function.resolve(self.stack.t.parse(self.stack, snippet))
+        return function.resolve(self.stack.t.parse(self.stack.defn, snippet))
 
     def test_join_split(self):
         # join
