@@ -98,8 +98,7 @@ class YamlMinimalTest(common.HeatTestCase):
 
     def test_long_yaml(self):
         template = {'HeatTemplateFormatVersion': '2012-12-12'}
-        config.cfg.CONF.set_override('max_template_size', 10,
-                                     enforce_type=True)
+        config.cfg.CONF.set_override('max_template_size', 10)
         template['Resources'] = ['a'] * int(
             config.cfg.CONF.max_template_size / 3)
         limit = config.cfg.CONF.max_template_size
@@ -166,9 +165,11 @@ class YamlParseExceptions(common.HeatTestCase):
             yaml_loader.side_effect = self.raised_exception
 
             err = self.assertRaises(ValueError,
-                                    template_format.parse, text)
+                                    template_format.parse, text,
+                                    'file://test.yaml')
 
-            self.assertIn('Error parsing template: ', six.text_type(err))
+            self.assertIn('Error parsing template file://test.yaml',
+                          six.text_type(err))
 
 
 class JsonYamlResolvedCompareTest(common.HeatTestCase):
