@@ -11,6 +11,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 import mock
 import six
 
@@ -26,6 +27,10 @@ from heat.tests import common
 
 class TestTranslationRule(common.HeatTestCase):
 
+    def setUp(self):
+        super(TestTranslationRule, self).setUp()
+        self.props = mock.Mock(spec=properties.Properties)
+
     def test_translation_rule(self):
         for r in translation.TranslationRule.RULE_KEYS:
             props = properties.Properties({}, {})
@@ -33,7 +38,7 @@ class TestTranslationRule(common.HeatTestCase):
                 props,
                 r,
                 ['any'],
-                ['value'] if r == 'Add' else 'value',
+                ['value'] if r == 'Add' else None,
                 'value_name' if r == 'Replace' else None,
                 'client_plugin' if r == 'Resolve' else None,
                 'finder' if r == 'Resolve' else None)
@@ -41,21 +46,45 @@ class TestTranslationRule(common.HeatTestCase):
             self.assertEqual(rule.rule, r)
             if r == 'Add':
                 self.assertEqual(['value'], rule.value)
-            else:
-                self.assertEqual('value', rule.value)
             if r == 'Replace':
                 self.assertEqual('value_name', rule.value_name)
             else:
                 self.assertIsNone(rule.value_name)
 
+    def test_cmp_rules(self):
+        rules = [
+            translation.TranslationRule(
+                mock.Mock(spec=properties.Properties),
+                translation.TranslationRule.DELETE,
+                ['any']
+            ),
+            translation.TranslationRule(
+                mock.Mock(spec=properties.Properties),
+                translation.TranslationRule.ADD,
+                ['any']
+            ),
+            translation.TranslationRule(
+                mock.Mock(spec=properties.Properties),
+                translation.TranslationRule.RESOLVE,
+                ['any'],
+                client_plugin=mock.ANY,
+                finder=mock.ANY
+            ),
+            translation.TranslationRule(
+                mock.Mock(spec=properties.Properties),
+                translation.TranslationRule.REPLACE,
+                ['any']
+            )
+        ]
+        expected = [translation.TranslationRule.ADD,
+                    translation.TranslationRule.REPLACE,
+                    translation.TranslationRule.RESOLVE,
+                    translation.TranslationRule.DELETE]
+        result = [rule.rule for rule in sorted(rules)]
+        self.assertEqual(expected, result)
+
     def test_invalid_translation_rule(self):
         props = properties.Properties({}, {})
-        exc = self.assertRaises(ValueError,
-                                translation.TranslationRule,
-                                'proppy', mock.ANY,
-                                mock.ANY)
-        self.assertEqual('Properties must be Properties type. '
-                         'Found %s.' % str, six.text_type(exc))
 
         exc = self.assertRaises(ValueError,
                                 translation.TranslationRule,
@@ -73,8 +102,9 @@ class TestTranslationRule(common.HeatTestCase):
                                 translation.TranslationRule.ADD,
                                 'networks.network',
                                 'value')
-        self.assertEqual('translation_path should be a list with path instead '
-                         'of %s.' % str, six.text_type(exc))
+        self.assertEqual('"translation_path" should be non-empty list '
+                         'with path to translate.',
+                         six.text_type(exc))
 
         exc = self.assertRaises(ValueError,
                                 translation.TranslationRule,
@@ -82,7 +112,8 @@ class TestTranslationRule(common.HeatTestCase):
                                 translation.TranslationRule.ADD,
                                 [],
                                 mock.ANY)
-        self.assertEqual('translation_path must be non-empty list with path.',
+        self.assertEqual('"translation_path" should be non-empty list '
+                         'with path to translate.',
                          six.text_type(exc))
 
         exc = self.assertRaises(ValueError,
@@ -91,10 +122,11 @@ class TestTranslationRule(common.HeatTestCase):
                                 translation.TranslationRule.ADD,
                                 ['any'],
                                 'value',
-                                'value_name')
-        self.assertEqual('Either value or value_name should be specified for '
-                         'rule Add.',
-                         six.text_type(exc))
+                                'value_name',
+                                'some_path')
+        self.assertEqual('"value_path", "value" and "value_name" are '
+                         'mutually exclusive and cannot be specified '
+                         'at the same time.', six.text_type(exc))
 
         exc = self.assertRaises(ValueError,
                                 translation.TranslationRule,
@@ -102,7 +134,7 @@ class TestTranslationRule(common.HeatTestCase):
                                 translation.TranslationRule.ADD,
                                 ['any'],
                                 'value')
-        self.assertEqual('value must be list type when rule is Add.',
+        self.assertEqual('"value" must be list type when rule is Add.',
                          six.text_type(exc))
 
     def test_add_rule_exist(self):
@@ -128,16 +160,22 @@ class TestTranslationRule(common.HeatTestCase):
             ],
             'bar': 'dak'
         }
-        props = properties.Properties(schema, data)
+        props = properties.Properties(schema, copy.copy(data))
 
         rule = translation.TranslationRule(
             props,
             translation.TranslationRule.ADD,
             ['far'],
             [{'red': props.get('bar')}])
-        rule.execute_rule()
 
-        self.assertIn({'red': 'dak'}, props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+
+        self.assertTrue(tran.has_translation('far'))
+        result = tran.translate('far', data['far'])
+        self.assertEqual([{'red': 'blue'}, {'red': 'dak'}], result)
+        self.assertEqual([{'red': 'blue'}, {'red': 'dak'}],
+                         tran.resolved_translations['far'])
 
     def test_add_rule_dont_exist(self):
         schema = {
@@ -159,16 +197,21 @@ class TestTranslationRule(common.HeatTestCase):
         data = {
             'bar': 'dak'
         }
-        props = properties.Properties(schema, data)
+        props = properties.Properties(schema, copy.copy(data))
 
         rule = translation.TranslationRule(
             props,
             translation.TranslationRule.ADD,
             ['far'],
             [{'red': props.get('bar')}])
-        rule.execute_rule()
 
-        self.assertEqual([{'red': 'dak'}], props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+
+        self.assertTrue(tran.has_translation('far'))
+        result = tran.translate('far')
+        self.assertEqual([{'red': 'dak'}], result)
+        self.assertEqual([{'red': 'dak'}], tran.resolved_translations['far'])
 
     def test_add_rule_invalid(self):
         schema = {
@@ -195,10 +238,15 @@ class TestTranslationRule(common.HeatTestCase):
             translation.TranslationRule.ADD,
             ['far'],
             [props.get('bar')])
-        exc = self.assertRaises(ValueError, rule.execute_rule)
 
-        self.assertEqual('Add rule must be used only for lists.',
-                         six.text_type(exc))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+
+        self.assertTrue(tran.has_translation('far'))
+        ex = self.assertRaises(ValueError, tran.translate, 'far', 'tran')
+        self.assertEqual('Incorrect translation rule using - cannot '
+                         'resolve Add rule for non-list translation '
+                         'value "far".', six.text_type(ex))
 
     def test_replace_rule_map_exist(self):
         schema = {
@@ -225,9 +273,14 @@ class TestTranslationRule(common.HeatTestCase):
             translation.TranslationRule.REPLACE,
             ['far', 'red'],
             props.get('bar'))
-        rule.execute_rule()
 
-        self.assertEqual({'red': 'dak'}, props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+
+        self.assertTrue(tran.has_translation('far.red'))
+        result = tran.translate('far.red', data['far']['red'])
+        self.assertEqual('dak', result)
+        self.assertEqual('dak', tran.resolved_translations['far.red'])
 
     def test_replace_rule_map_dont_exist(self):
         schema = {
@@ -253,9 +306,14 @@ class TestTranslationRule(common.HeatTestCase):
             translation.TranslationRule.REPLACE,
             ['far', 'red'],
             props.get('bar'))
-        rule.execute_rule()
 
-        self.assertEqual({'red': 'dak'}, props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+
+        self.assertTrue(tran.has_translation('far.red'))
+        result = tran.translate('far.red')
+        self.assertEqual('dak', result)
+        self.assertEqual('dak', tran.resolved_translations['far.red'])
 
     def test_replace_rule_list_different(self):
         schema = {
@@ -286,9 +344,14 @@ class TestTranslationRule(common.HeatTestCase):
             translation.TranslationRule.REPLACE,
             ['far', 'red'],
             props.get('bar'))
-        rule.execute_rule()
 
-        self.assertEqual([{'red': 'dak'}, {'red': 'dak'}], props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+
+        self.assertTrue(tran.has_translation('far.red'))
+        result = tran.translate('far.0.red', data['far'][0]['red'])
+        self.assertEqual('dak', result)
+        self.assertEqual('dak', tran.resolved_translations['far.0.red'])
 
     def test_replace_rule_list_same(self):
         schema = {
@@ -319,11 +382,22 @@ class TestTranslationRule(common.HeatTestCase):
             ['far', 'red'],
             None,
             'blue')
-        rule.execute_rule()
 
-        self.assertEqual([{'red': 'white', 'blue': None},
-                          {'blue': None, 'red': 'roses'}],
-                         props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+
+        self.assertTrue(tran.has_translation('far.0.red'))
+        result = tran.translate('far.0.red', data['far'][0].get('red'),
+                                data['far'][0])
+        self.assertEqual('white', result)
+        self.assertEqual('white', tran.resolved_translations['far.0.red'])
+        self.assertIsNone(tran.resolved_translations['far.0.blue'])
+        self.assertTrue(tran.has_translation('far.1.red'))
+        result = tran.translate('far.1.red', data['far'][1]['red'],
+                                data['far'][1])
+        self.assertEqual('roses', result)
+        self.assertEqual('roses', tran.resolved_translations['far.1.red'])
+        self.assertIsNone(tran.resolved_translations['far.1.blue'])
 
     def test_replace_rule_str(self):
         schema = {
@@ -340,10 +414,14 @@ class TestTranslationRule(common.HeatTestCase):
             translation.TranslationRule.REPLACE,
             ['bar'],
             props.get('far'))
-        rule.execute_rule()
 
-        self.assertEqual('one', props.get('bar'))
-        self.assertEqual('one', props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+
+        self.assertTrue(tran.has_translation('bar'))
+        result = tran.translate('bar', data['bar'])
+        self.assertEqual('one', result)
+        self.assertEqual('one', tran.resolved_translations['bar'])
 
     def test_replace_rule_str_value_path_error(self):
         schema = {
@@ -360,11 +438,15 @@ class TestTranslationRule(common.HeatTestCase):
             translation.TranslationRule.REPLACE,
             ['bar'],
             value_path=['far'])
-        ex = self.assertRaises(exception.ResourcePropertyConflict,
-                               rule.execute_rule)
-        self.assertEqual("Cannot define the following properties at the "
-                         "same time: ['bar', 'far'].",
-                         six.text_type(ex))
+
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+
+        self.assertTrue(tran.has_translation('bar'))
+        ex = self.assertRaises(exception.StackValidationFailed,
+                               tran.translate, 'bar', data['bar'])
+        self.assertEqual('Cannot define the following properties at '
+                         'the same time: bar, far', six.text_type(ex))
 
     def test_replace_rule_str_value_path(self):
         schema = {
@@ -372,19 +454,23 @@ class TestTranslationRule(common.HeatTestCase):
             'bar': properties.Schema(properties.Schema.STRING)
         }
 
-        data = {'far': 'one'}
-
-        props = properties.Properties(schema, data)
+        props = properties.Properties(schema, {'far': 'one'})
 
         rule = translation.TranslationRule(
             props,
             translation.TranslationRule.REPLACE,
             ['bar'],
             value_path=['far'])
-        rule.execute_rule()
 
-        self.assertEqual('one', props.get('bar'))
-        self.assertIsNone(props.get('far'))
+        props = properties.Properties(schema, {'far': 'one'})
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+
+        self.assertTrue(tran.has_translation('bar'))
+        result = tran.translate('bar')
+        self.assertEqual('one', result)
+        self.assertEqual('one', tran.resolved_translations['bar'])
+        self.assertIsNone(tran.resolved_translations['far'])
 
     def test_replace_rule_str_invalid(self):
         schema = {
@@ -401,8 +487,7 @@ class TestTranslationRule(common.HeatTestCase):
             translation.TranslationRule.REPLACE,
             ['bar'],
             props.get('far'))
-        rule.execute_rule()
-
+        props.update_translation([rule])
         exc = self.assertRaises(exception.StackValidationFailed,
                                 props.validate)
         self.assertEqual("Property error: bar: Value 'one' is not an integer",
@@ -435,18 +520,12 @@ class TestTranslationRule(common.HeatTestCase):
             props,
             translation.TranslationRule.DELETE,
             ['far', 'red'])
-        rule.execute_rule()
 
-        self.assertEqual([{'check': None, 'red': None},
-                          {'check': None, 'red': None}], props.get('far'))
-
-        # check if no data translation still is correct
-        props = properties.Properties(schema, {'far': [{'check': 'yep'}]})
-        rule = translation.TranslationRule(
-            props,
-            translation.TranslationRule.DELETE,
-            ['far', 'red'])
-        self.assertIsNone(rule.execute_rule())
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far.red'))
+        self.assertIsNone(tran.translate('far.red'))
+        self.assertIsNone(tran.resolved_translations['far.red'])
 
     def test_delete_rule_other(self):
         schema = {
@@ -461,17 +540,17 @@ class TestTranslationRule(common.HeatTestCase):
             props,
             translation.TranslationRule.DELETE,
             ['far'])
-        rule.execute_rule()
 
-        self.assertIsNone(props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far'))
+        self.assertIsNone(tran.translate('far'))
+        self.assertIsNone(tran.resolved_translations['far'])
 
-    def _test_resolve_rule(self, is_list=False,
-                           check_error=False):
+    def _test_resolve_rule(self, is_list=False):
         class FakeClientPlugin(object):
             def find_name_id(self, entity=None,
                              src_value='far'):
-                if check_error:
-                    raise exception.NotFound()
                 if entity == 'rose':
                     return 'pink'
                 return 'yellow'
@@ -510,9 +589,13 @@ class TestTranslationRule(common.HeatTestCase):
             client_plugin=client_plugin,
             finder='find_name_id'
             )
-        rule.execute_rule()
-        self.assertEqual([{'red': 'yellow'}, {'red': 'yellow'}],
-                         props.get('far'))
+
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far.red'))
+        result = tran.translate('far.0.red', data['far'][0]['red'])
+        self.assertEqual('yellow', result)
+        self.assertEqual('yellow', tran.resolved_translations['far.0.red'])
 
     def test_resolve_rule_list_with_function(self):
         client_plugin, schema = self._test_resolve_rule(is_list=True)
@@ -531,9 +614,13 @@ class TestTranslationRule(common.HeatTestCase):
             client_plugin=client_plugin,
             finder='find_name_id'
             )
-        rule.execute_rule()
-        self.assertEqual([{'red': 'yellow'}, {'red': 'yellow'}],
-                         props.get('far'))
+
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far.red'))
+        result = tran.translate('far.0.red', data['far'][0]['red'])
+        self.assertEqual('yellow', result)
+        self.assertEqual('yellow', tran.resolved_translations['far.0.red'])
 
     def test_resolve_rule_list_with_ref(self):
         client_plugin, schema = self._test_resolve_rule(is_list=True)
@@ -562,8 +649,13 @@ class TestTranslationRule(common.HeatTestCase):
             client_plugin=client_plugin,
             finder='find_name_id'
             )
-        rule.execute_rule()
-        self.assertEqual(data, props.data)
+
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far.red'))
+        result = tran.translate('far.0.red', data['far'][0]['red'])
+        self.assertEqual('yellow', result)
+        self.assertEqual('yellow', tran.resolved_translations['far.0.red'])
 
     def test_resolve_rule_list_strings(self):
         client_plugin, schema = self._test_resolve_rule()
@@ -582,39 +674,12 @@ class TestTranslationRule(common.HeatTestCase):
             client_plugin=client_plugin,
             finder='find_name_id')
 
-        rule.execute_rule()
-        self.assertEqual(['yellow', 'pink'], props.get('far'))
-
-    def test_resolve_rule_list_empty(self):
-        client_plugin, schema = self._test_resolve_rule(is_list=True)
-        data = {
-            'far': [],
-        }
-        props = properties.Properties(schema, data)
-
-        rule = translation.TranslationRule(
-            props,
-            translation.TranslationRule.RESOLVE,
-            ['far', 'red'],
-            client_plugin=client_plugin,
-            finder='find_name_id'
-            )
-        rule.execute_rule()
-        self.assertEqual([], props.get('far'))
-
-    def test_resolve_rule_ignore_error(self):
-        client_plugin, schema = self._test_resolve_rule(check_error=True)
-        data = {'far': 'one'}
-        props = properties.Properties(schema, data)
-        rule = translation.TranslationRule(
-            props,
-            translation.TranslationRule.RESOLVE,
-            ['far'],
-            client_plugin=client_plugin,
-            finder='find_name_id')
-
-        rule.execute_rule(ignore_resolve_error=True)
-        self.assertEqual('one', props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far'))
+        result = tran.translate('far', data['far'])
+        self.assertEqual(['yellow', 'pink'], result)
+        self.assertEqual(['yellow', 'pink'], tran.resolved_translations['far'])
 
     def test_resolve_rule_other(self):
         client_plugin, schema = self._test_resolve_rule()
@@ -627,8 +692,12 @@ class TestTranslationRule(common.HeatTestCase):
             client_plugin=client_plugin,
             finder='find_name_id')
 
-        rule.execute_rule()
-        self.assertEqual('yellow', props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far'))
+        result = tran.translate('far', data['far'])
+        self.assertEqual('yellow', result)
+        self.assertEqual('yellow', tran.resolved_translations['far'])
 
     def test_resolve_rule_other_with_ref(self):
         client_plugin, schema = self._test_resolve_rule()
@@ -654,8 +723,11 @@ class TestTranslationRule(common.HeatTestCase):
             client_plugin=client_plugin,
             finder='find_name_id')
 
-        rule.execute_rule()
-        self.assertEqual(data, props.data)
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far'))
+        result = tran.translate('far', data['far'])
+        self.assertEqual('yellow', result)
 
     def test_resolve_rule_other_with_function(self):
         client_plugin, schema = self._test_resolve_rule()
@@ -670,8 +742,12 @@ class TestTranslationRule(common.HeatTestCase):
             client_plugin=client_plugin,
             finder='find_name_id')
 
-        rule.execute_rule()
-        self.assertEqual(data, props.data)
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far'))
+        result = tran.translate('far', data['far'])
+        self.assertEqual('yellow', result)
+        self.assertEqual('yellow', tran.resolved_translations['far'])
 
     def test_resolve_rule_other_with_get_attr(self):
         client_plugin, schema = self._test_resolve_rule()
@@ -693,14 +769,14 @@ class TestTranslationRule(common.HeatTestCase):
             ['far'],
             client_plugin=client_plugin,
             finder='find_name_id')
-        rule.execute_rule(client_resolve=False)
-        self.assertEqual(data, props.data)
 
-        mock_getatt = self.patchobject(attr_func, 'result',
-                                       return_value='rose')
-        rule.execute_rule()
-        self.assertEqual('pink', props.get('far'))
-        self.assertEqual(1, mock_getatt.call_count)
+        tran = translation.Translation(props)
+        tran.set_rules([rule], client_resolve=False)
+        self.assertFalse(tran.store_translated_values)
+        self.assertFalse(tran.has_translation('far'))
+        result = tran.translate('far', 'no_check', data['far'])
+        self.assertEqual('no_check', result)
+        self.assertIsNone(tran.resolved_translations.get('far'))
 
     def test_resolve_rule_other_with_entity(self):
         client_plugin, schema = self._test_resolve_rule()
@@ -714,8 +790,12 @@ class TestTranslationRule(common.HeatTestCase):
             finder='find_name_id',
             entity='rose')
 
-        rule.execute_rule()
-        self.assertEqual('pink', props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far'))
+        result = tran.translate('far', data['far'])
+        self.assertEqual('pink', result)
+        self.assertEqual('pink', tran.resolved_translations['far'])
 
     def test_property_json_param_correct_translation(self):
         """Test case when property with sub-schema takes json param."""
@@ -746,7 +826,7 @@ class TestTranslationRule(common.HeatTestCase):
                 '{"dar": "rad"}').value()}
         data = {'far': param}
 
-        props = properties.Properties(schema, data)
+        props = properties.Properties(schema, data, resolver=function.resolve)
 
         rule = translation.TranslationRule(
             props,
@@ -754,9 +834,13 @@ class TestTranslationRule(common.HeatTestCase):
             ['far', 'bar'],
             value_path=['far', 'dar'])
 
-        rule.execute_rule()
-
-        self.assertEqual('rad', props.get('far').get('bar'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far.bar'))
+        prop_data = props['far']
+        result = tran.translate('far.bar', prop_data['bar'], prop_data)
+        self.assertEqual('rad', result)
+        self.assertEqual('rad', tran.resolved_translations['far.bar'])
 
     def test_property_json_param_to_list_correct_translation(self):
         """Test case when list property with sub-schema takes json param."""
@@ -790,7 +874,7 @@ class TestTranslationRule(common.HeatTestCase):
                 '{"dar": "rad"}').value()}
         data = {'far': [param]}
 
-        props = properties.Properties(schema, data)
+        props = properties.Properties(schema, data, resolver=function.resolve)
 
         rule = translation.TranslationRule(
             props,
@@ -798,9 +882,14 @@ class TestTranslationRule(common.HeatTestCase):
             ['far', 'bar'],
             value_name='dar')
 
-        rule.execute_rule()
-
-        self.assertEqual([{'dar': None, 'bar': 'rad'}], props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far.0.bar'))
+        prop_data = props['far']
+        result = tran.translate('far.0.bar', prop_data[0]['bar'],
+                                prop_data[0])
+        self.assertEqual('rad', result)
+        self.assertEqual('rad', tran.resolved_translations['far.0.bar'])
 
     def test_property_commadelimitedlist_param_correct_translation(self):
         """Test when property with sub-schema takes comma_delimited_list."""
@@ -830,92 +919,21 @@ class TestTranslationRule(common.HeatTestCase):
                 "white,roses").value()}
         data = {'far': param, 'boo': 'chrysanthemums'}
 
-        props = properties.Properties(schema, data)
+        props = properties.Properties(schema, data, resolver=function.resolve)
 
         rule = translation.TranslationRule(
             props,
             translation.TranslationRule.ADD,
             ['far'],
             [props.get('boo')])
-        rule.execute_rule()
 
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far'))
+        result = tran.translate('far', props['far'])
+        self.assertEqual(['white', 'roses', 'chrysanthemums'], result)
         self.assertEqual(['white', 'roses', 'chrysanthemums'],
-                         props.get('far'))
-
-    def test_property_no_translation_if_user_parameter_missing(self):
-        """Test translation in the case of missing parameter"""
-        schema = {
-            'source': properties.Schema(
-                properties.Schema.STRING
-            ),
-            'destination': properties.Schema(
-                properties.Schema.STRING
-            )}
-
-        class DummyStack(dict):
-            @property
-            def parameters(self):
-                return mock.Mock()
-
-        param = hot_funcs.GetParam(DummyStack(),
-                                   'get_param',
-                                   'source_param')
-
-        param.parameters = {}
-
-        data = {'source': param, 'destination': ''}
-        props = properties.Properties(schema, data,
-                                      resolver=function.resolve)
-
-        rule = translation.TranslationRule(
-            props,
-            translation.TranslationRule.REPLACE,
-            ['destination'],
-            value_path=['source'])
-
-        rule.execute_rule()
-
-        # ensure that translation rule was not applied
-        self.assertEqual({'source': param, 'destination': ''},
-                         data)
-
-    def test_property_get_attr_translation_successfully_skipped(self):
-        schema = {
-            'source': properties.Schema(
-                properties.Schema.LIST,
-                schema=properties.Schema(
-                    properties.Schema.MAP,
-                    schema={
-                        'sub-source': properties.Schema(
-                            properties.Schema.STRING
-                        )
-                    }
-                )
-            ),
-            'destination': properties.Schema(
-                properties.Schema.STRING
-            )}
-
-        class DummyStack(dict):
-            pass
-
-        class rsrc(object):
-            pass
-
-        stack = DummyStack(res=rsrc())
-        attr_func = cfn_funcs.GetAtt(stack, 'Fn::GetAtt',
-                                     ['res', 'sub-sources'])
-
-        data = {'source': attr_func, 'destination': ''}
-        props = properties.Properties(schema, data)
-
-        rule = translation.TranslationRule(
-            props,
-            translation.TranslationRule.REPLACE,
-            ['source', 'sub-sources'],
-            value_path=['destination'])
-        rule.execute_rule()
-        self.assertEqual({'source': attr_func, 'destination': ''}, props.data)
+                         tran.resolved_translations['far'])
 
     def test_list_list_add_translation_rule(self):
         schema = {
@@ -936,7 +954,7 @@ class TestTranslationRule(common.HeatTestCase):
 
         data = {'far': [{'bar': ['shar'], 'car': 'man'}, {'car': 'first'}]}
 
-        props = properties.Properties(schema, data)
+        props = properties.Properties(schema, data, resolver=function.resolve)
 
         rule = translation.TranslationRule(
             props,
@@ -944,10 +962,18 @@ class TestTranslationRule(common.HeatTestCase):
             ['far', 'bar'],
             value_name='car'
         )
-        rule.execute_rule()
 
-        self.assertIn({'bar': ['shar', 'man'], 'car': 'man'}, props.get('far'))
-        self.assertIn({'bar': ['first'], 'car': 'first'}, props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far.0.bar'))
+        result = tran.translate('far.0.bar', props['far'][0]['bar'],
+                                props['far'][0])
+        self.assertEqual(['shar', 'man'], result)
+        self.assertEqual(['shar', 'man'],
+                         tran.resolved_translations['far.0.bar'])
+        result = tran.translate('far.1.bar', prop_data=props['far'][1])
+        self.assertEqual(['first'], result)
+        self.assertEqual(['first'], tran.resolved_translations['far.1.bar'])
 
     def test_replace_rule_map_with_custom_value_path(self):
         schema = {
@@ -976,10 +1002,13 @@ class TestTranslationRule(common.HeatTestCase):
             value_path=['bar'],
             custom_value_path=['red']
         )
-        rule.execute_rule()
 
-        self.assertEqual({'red': 'dak'}, props.get('far'))
-        self.assertEqual({}, props.get('bar'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far.red'))
+        result = tran.translate('far.red')
+        self.assertEqual('dak', result)
+        self.assertEqual('dak', tran.resolved_translations['far.red'])
 
     def test_replace_rule_list_with_custom_value_path(self):
         schema = {
@@ -1011,11 +1040,13 @@ class TestTranslationRule(common.HeatTestCase):
             value_name='blue',
             custom_value_path=['black', 'white']
         )
-        rule.execute_rule()
 
-        self.assertEqual([{'red': 'daisy', 'blue': {'black': {}}},
-                          {'blue': None, 'red': 'roses'}],
-                         props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far.0.red'))
+        result = tran.translate('far.0.red', prop_data=data['far'][0])
+        self.assertEqual('daisy', result)
+        self.assertEqual('daisy', tran.resolved_translations['far.0.red'])
 
     def test_add_rule_list_with_custom_value_path(self):
         schema = {
@@ -1048,58 +1079,86 @@ class TestTranslationRule(common.HeatTestCase):
             value_name='blue',
             custom_value_path=['black', 'check']
         )
-        rule.execute_rule()
 
-        self.assertEqual([{'red': ['one'],
-                           'blue': {'black': {'white': 'daisy',
-                                              'check': ['one']}}},
-                          {'blue': None, 'red': ['roses']}],
-                         props.get('far'))
+        tran = translation.Translation(props)
+        tran.set_rules([rule])
+        self.assertTrue(tran.has_translation('far.0.red'))
+        result = tran.translate('far.0.red', data['far'][0].get('red'),
+                                data['far'][0])
+        self.assertEqual(['one'], result)
+        self.assertEqual(['one'], tran.resolved_translations['far.0.red'])
+        self.assertEqual(['roses'], tran.translate('far.1.red',
+                                                   data['far'][1]['red'],
+                                                   data['far'][1]))
 
-        # check whether if data will contain list-type property to add
-        data = {
-            'far': [{'red': ['roses'], 'blue': {'black': {'check': ['two']}}}]}
-        props = properties.Properties(schema, data)
-        rule.properties = props
-        rule.execute_rule()
-        self.assertEqual([{'red': ['roses', 'two'],
-                           'blue': {'black': {'check': ['two']}}}],
-                         props.get('far'))
+    def test_set_rules_none(self):
+        tran = translation.Translation()
+        self.assertEqual({}, tran._rules)
 
-    def test_list_list_error_translation_rule(self):
-        schema = {
-            'far': properties.Schema(
-                properties.Schema.LIST,
-                schema=properties.Schema(
-                    properties.Schema.MAP,
-                    schema={
-                        'car': properties.Schema(properties.Schema.STRING),
-                        'dar': properties.Schema(properties.Schema.STRING),
-                    }
-                )
-            ),
-            'bar': properties.Schema(
-                properties.Schema.LIST,
-                schema=properties.Schema(
-                    properties.Schema.MAP,
-                    schema={
-                        'car': properties.Schema(properties.Schema.STRING),
-                        'dar': properties.Schema(properties.Schema.STRING),
-                    }
-                )
-            ),
-        }
+    def test_set_no_resolve_rules(self):
+        rules = [
+            translation.TranslationRule(
+                self.props,
+                translation.TranslationRule.RESOLVE,
+                ['a'],
+                client_plugin=mock.ANY,
+                finder='finder'
+            )
+        ]
 
-        data = {'far': [{'car': 'man'}], 'bar': [{'dar': 'check'}]}
+        tran = translation.Translation()
+        tran.set_rules(rules, client_resolve=False)
+        self.assertEqual({}, tran._rules)
 
-        props = properties.Properties(schema, data)
+    def test_translate_add(self):
+        rules = [
+            translation.TranslationRule(
+                self.props,
+                translation.TranslationRule.ADD,
+                ['a', 'b'],
+                value=['check']
+            )
+        ]
 
-        rule = translation.TranslationRule(
-            props,
-            translation.TranslationRule.REPLACE,
-            ['far'],
-            value_path=['bar', 'car']
-        )
-        ex = self.assertRaises(ValueError, rule.execute_rule)
-        self.assertEqual('Cannot use value_path for properties inside '
-                         'list-type properties', six.text_type(ex))
+        tran = translation.Translation()
+        tran.set_rules(rules)
+
+        result = tran.translate('a.b', ['test'])
+        self.assertEqual(['test', 'check'], result)
+        self.assertEqual(['test', 'check'], tran.resolved_translations['a.b'])
+
+        # Test without storing
+        tran.resolved_translations = {}
+        tran.store_translated_values = False
+        result = tran.translate('a.b', ['test'])
+        self.assertEqual(['test', 'check'], result)
+        self.assertEqual({}, tran.resolved_translations)
+        tran.store_translated_values = True
+
+        # Test no prop_value
+        self.assertEqual(['check'], tran.translate('a.b', None))
+
+        # Check digits in path skipped for rule
+        self.assertEqual(['test', 'check'], tran.translate('a.0.b', ['test']))
+
+    def test_translate_delete(self):
+        rules = [
+            translation.TranslationRule(
+                self.props,
+                translation.TranslationRule.DELETE,
+                ['a']
+            )
+        ]
+
+        tran = translation.Translation()
+        tran.set_rules(rules)
+
+        self.assertIsNone(tran.translate('a'))
+        self.assertIsNone(tran.resolved_translations['a'])
+
+        # Test without storing
+        tran.resolved_translations = {}
+        tran.store_translated_values = False
+        self.assertIsNone(tran.translate('a'))
+        self.assertEqual({}, tran.resolved_translations)
+        tran.store_translated_values = True
