@@ -30,4 +30,4 @@ openstack flavor delete m1.heat_int
 openstack flavor delete m1.heat_micro
 
 # delete the image created
-openstack image delete fedora-heat-test-image
\ No newline at end of file
+openstack image delete Fedora-Cloud-Base-24-1.2.x86_64
