@@ -18,6 +18,7 @@ import time
 
 import fixtures
 from heatclient import exc as heat_exceptions
+from keystoneauth1 import exceptions as kc_exceptions
 from neutronclient.common import exceptions as network_exceptions
 from oslo_log import log as logging
 from oslo_utils import timeutils
@@ -60,7 +61,7 @@ def call_until_true(duration, sleep_for, func, *args, **kwargs):
 
 
 def rand_name(name=''):
-    randbits = str(random.randint(1, 0x7fffffff))
+    randbits = six.text_type(random.randint(1, 0x7fffffff))
     if name:
         return name + '-' + randbits
     else:
@@ -118,7 +119,7 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
         try:
             linux_client.validate_authentication()
         except exceptions.SSHTimeout:
-            LOG.exception('ssh connection to %s failed' % ip)
+            LOG.exception('ssh connection to %s failed', ip)
             raise
 
         return linux_client
@@ -194,6 +195,15 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
             return False
         return True
 
+    def is_service_available(self, service_type):
+        try:
+            self.identity_client.get_endpoint_url(
+                service_type, self.conf.region)
+        except kc_exceptions.EndpointNotFound:
+            return False
+        else:
+            return True
+
     @staticmethod
     def _stack_output(stack, output_key, validate_errors=True):
         """Return a stack output value for a given key."""
@@ -519,6 +529,20 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
         stack_link = [l for l in r.links if l.get('rel') == 'stack'][0]
         return stack_link['href'].split("/")[-1]
 
+    def get_physical_resource_id(self, stack_identifier, resource_name):
+        try:
+            resource = self.client.resources.get(
+                stack_identifier, resource_name)
+            return resource.physical_resource_id
+        except Exception:
+            raise Exception('Resource (%s) not found in stack (%s)!' %
+                            (stack_identifier, resource_name))
+
+    def get_stack_output(self, stack_identifier, output_key,
+                         validate_errors=True):
+        stack = self.client.stacks.get(stack_identifier)
+        return self._stack_output(stack, output_key, validate_errors)
+
     def check_input_values(self, group_resources, key, value):
         # Check inputs for deployment and derived config
         for r in group_resources:
@@ -541,12 +565,13 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
                      parameters=None, environment=None, tags=None,
                      expected_status='CREATE_COMPLETE',
                      disable_rollback=True, enable_cleanup=True,
-                     environment_files=None):
+                     environment_files=None, timeout=None):
         name = stack_name or self._stack_rand_name()
         templ = template or self.template
         templ_files = files or {}
         params = parameters or {}
         env = environment or {}
+        timeout_mins = timeout or self.conf.build_timeout
         self.client.stacks.create(
             stack_name=name,
             template=templ,
@@ -555,7 +580,8 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
             parameters=params,
             environment=env,
             tags=tags,
-            environment_files=environment_files
+            environment_files=environment_files,
+            timeout_mins=timeout_mins
         )
         if expected_status not in ['ROLLBACK_COMPLETE'] and enable_cleanup:
             self.addCleanup(self._stack_delete, name)
@@ -648,7 +674,7 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
                 rsrc_events = self.client.events.list(stack_identifier,
                                                       resource_name=rsrc_name)
             except heat_exceptions.HTTPNotFound:
-                LOG.debug("No events yet found for %s" % rsrc_name)
+                LOG.debug("No events yet found for %s", rsrc_name)
             else:
                 matched = [e for e in rsrc_events
                            if e.resource_status_reason == reason]
@@ -656,10 +682,14 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
                     return matched
             time.sleep(build_interval)
 
-    def check_autoscale_complete(self, stack_id, expected_num):
+    def check_autoscale_complete(self, stack_id, expected_num, parent_stack,
+                                 policy):
         res_list = self.client.resources.list(stack_id)
         all_res_complete = all(res.resource_status in ('UPDATE_COMPLETE',
                                                        'CREATE_COMPLETE')
                                for res in res_list)
         all_res = len(res_list) == expected_num
-        return all_res and all_res_complete
+        if all_res and all_res_complete:
+            metadata = self.client.resources.metadata(parent_stack, policy)
+            return not metadata.get('scaling_in_progress')
+        return False
