@@ -25,7 +25,6 @@ if [ "$DISABLE_CONVERGENCE" == "true" ] ; then
 fi
 
 echo -e 'stack_scheduler_hints=true\n' >> $localconf
-echo -e 'notification_driver=messagingv2\n' >> $localconf
 echo -e 'hidden_stack_tags=hidden\n' >> $localconf
 echo -e 'encrypt_parameters_and_properties=True\n' >> $localconf
 echo -e 'logging_exception_prefix=%(asctime)s.%(msecs)03d %(process)d TRACE %(name)s %(instance)s\n' >> $localconf
@@ -38,19 +37,16 @@ echo -e '[cache]\nenabled=True\n' >> $localconf
 
 echo -e '[eventlet_opts]\nclient_socket_timeout=120\n' >> $localconf
 
+echo -e '[oslo_messaging_notifications]\ndriver=messagingv2\n' >> $localconf
+
+# Disable nova quota check.
+echo -e '[[post-config|$NOVA_CONF]]\n[DEFAULT]\n' >> $localconf
+echo -e 'quota_driver=nova.quota.NoopQuotaDriver\n' >> $localconf
 
 echo "[[local|localrc]]" >> $localconf
+# Create the images required for testing
+echo "IMAGE_URLS+=http://fedora.bhs.mirrors.ovh.net/linux/releases/24/CloudImages/x86_64/images/Fedora-Cloud-Base-24-1.2.x86_64.qcow2" >> $localconf
 
-# NOTE(mnaser): This will use the region local mirrors to avoid going out
-#               to network
-if [[ -e /etc/ci/mirror_info.sh ]]; then
-	source /etc/ci/mirror_info.sh
-	echo "IMAGE_URLS+=${NODEPOOL_FEDORA_MIRROR}/releases/26/CloudImages/x86_64/images/Fedora-Cloud-Base-26-1.5.x86_64.qcow2" >> $localconf
-else
-	echo "IMAGE_URLS+=https://download.fedoraproject.org/pub/fedora/linux/releases/26/CloudImages/x86_64/images/Fedora-Cloud-Base-26-1.5.x86_64.qcow2" >> $localconf
-fi
-
-echo "CEILOMETER_BACKEND=mysql" >> $localconf
 echo "CEILOMETER_PIPELINE_INTERVAL=60" >> $localconf
 echo "HEAT_ENABLE_ADOPT_ABANDON=True" >> $localconf
 # Use the lbaas v2 namespace driver for devstack integration testing since
